function HttpDataSource(config) {
	// Debug flag
	var debug = 0;
	
	// Set the configuration fields
	var $scope 		= config.scope;
	var configurl 	= config.configurl;
	var controller 	= $scope.controllerName;

	// Set the logging fields
	var $init 		= angular.injector(['ng']);
	var $log 		= $init.get('$log');
	
	// Initialize a local variable to hold current reference
	var self 		= this;

	/** 
	 * Load the local configuration and then connect 
	 */
	this.init = function() {
		this.processConfiguredServices();
	};
	
	/** 
	 * Processed all configured services.
	 */
	this.processConfiguredServices = function() {
		// If there is a configurl, load services
		if(configurl != null) {
			$.ajax({
				type: 'GET',
				url: configurl,
				dataType: 'json',
				success: function(data, textStatus, xhr) {
					// Add each loaded query to the services collection
					for(var idx = 0; idx < data.services.length; idx++) {
						if(data.services[idx].disabled != true)
							self.register(data.services[idx]);
					}
				},
				error: function(xhr, textStatus, errorThrown) {
					$log.error(textStatus);
					$log.error(xhr);
				}
			});	
		}
	};
	
	/** 
	 * Executes the HTTP datasource as a snapshot 
	 */
	var snapshot = function(service, onSuccess, onError) {
		service.status = 'COMPLETE';
		$.ajax({
			type: 'GET',
			url: service.url,
			data: JSON.stringify(service.request),
			dataType: 'json',
			success: function(data, textStatus, xhr) {
				// Get the schema fields
				var fields = data.response.schema.fields;
				
				// Set the schema on the datasource
				service.datasource.setSchema(fields);

				// Set the schema on the dataset
				for(var idx = 0; idx < service.dataset.length; idx++)
					$scope[service.dataset[idx]].setSchema(fields);

				// Call snapshot start
				for(var idx = 0; idx < service.dataset.length; idx++)
					$scope[service.dataset[idx]].operation('onSnapshotStart');
				
				// Get the name of the data array property
				var dataArray = Object.keys(data.response.data)[0];

				// Loop over the event data and send event for each
				for(var idx = 0; idx < data.response.data[dataArray].length; idx++) {
					var fieldMap = data.response.data[dataArray][idx];
					for(var dsIndex = 0; dsIndex < service.dataset.length; dsIndex++)
						$scope[service.dataset[dsIndex]].operation('onInsert', fieldMap);
				}

				for(var idx = 0; idx < service.dataset.length; idx++)
					$scope[service.dataset[idx]].operation('onSnapshotEnd');

				if(debug >= 1) $log.debug('[' + controller + '] ' + 'service:' + service.id + ':snapshot:onSuccess');					
			},
			error: function(xhr, textStatus, errorThrown) {
				$log.error(textStatus);
				$log.error(xhr);
			}
		});			
	};
	
	/** 
	 * Register a new service datasource on the system. 
	 */
	this.register = function(service, onSuccess, onError) {
		// Link to the datasets
		for(var idx = 0; idx < service.dataset.length; idx++) {			
			// Retrieve the dataset
			var datasetName = service.dataset[idx];
			var dataset = $scope[datasetName];
			
			// If dataset is not found, create it
			if(dataset == undefined) {
				dataset = new Dataset($scope, datasetName, service.primaryKey);
				$scope[datasetName] = dataset;
				$scope.$apply();
			}
			
			// OnReload function for the data source
			var onreload = function(onSuccess, onError) {
				snapshot(service, onSuccess, onError);
			};
			
			// Create the datasource
			service.datasource = new DataSource(service.name, 'HTTP', false, service, this, onreload);
			
			// Register the data source with the dataset
			dataset.registerSource(service.datasource);
		}

		snapshot(service, onSuccess, onError);
	};
}