app.directive('datasetModal', function($log, $filter, $window) {
	return {
		templateUrl: './templates/dataset-modal.html',
		restrict: 'E',
		scope: false,
		replace: true,
		controller: function($scope) {
		},
		link: function(scope, element, attrs) {	
			//$log.debug(scope.inputContext);
			//scope.someFunction();

			// Set the dataset directly on the scope
			scope.dataset = scope.inputContext.dataset;
			
			// Set the initial mode
			scope.mode = 'view';
			
			// Allow edit function
			scope.allowEdit = scope.inputContext.editor;
			
			// Set the default datasource
			if(scope.dataset.getSources().length >= 1) {
				var sorted = $filter('orderByFunction')(scope.dataset.getSources(), 'getName', 'asc');
				scope.datasource = sorted[0];
				scope.$applyAsync();
			}

			// Watch for datasource change
			scope.$watch('datasource', function() {
				scope.mode = 'view';
			});
			
			// Switch to edit mode
			scope.edit = function() {
				scope.mode = 'edit';
			};
			
			// Switch to view mode
			scope.cancel = function() {
				scope.mode = 'view';
			};			
		}
	};	
});

app.directive('datasetDefinition', function($log, $window) {
	return {
		templateUrl: './templates/dataset-modal-definition.html',
		restrict: 'E',
		scope: false,
		replace: true,
		controller: function($scope) {
		},
		link: function(scope, element, attrs) {	
		}
	};	
});

app.directive('datasetQueryView', function($log, $window) {
	return {
		templateUrl: './templates/dataset-modal-query-view.html',
		restrict: 'E',
		scope: false,
		replace: true,
		controller: function($scope) {
		},
		link: function(scope, element, attrs) {	
		}
	};	
});

app.directive('datasetQueryEdit', function($log, $window) {
	return {
		templateUrl: './templates/dataset-modal-query-edit.html',
		restrict: 'E',
		scope: false,
		replace: true,
		controller: function($scope) {
		},
		link: function(scope, element, attrs) {
			// Get the connection and query from the datasource
			var conn = scope.datasource.getConnection();
			var query = scope.datasource.getSource();
			
			// Has the user tried to submit the modal
			scope.isModalSubmit 	= false;
			
			// Query and fields valid state
			scope.isQueryValid 		= null;
			scope.isFormDataValid 	= null;
					
			// Get the table list and setup the initial query
			scope.tables = {};
			conn.listTables(false, function(result) {
				for(var idx = 0; idx < result.tables.length; idx++) {
					var tableName = result.tables[idx].name;
					scope.tables[tableName] = result.tables[idx];
				}

				// Setup the local query
				if(query != undefined) {
					var elements = query.template.split(new RegExp('FROM', 'i'));
					var select = elements[0].split(new RegExp('SELECT', 'i'))[1].trim();
					var predicateIdx = elements[1].trim().indexOf(' ');
					var tableName = predicateIdx == -1 ? elements[1].trim() : elements[1].substring(0, predicateIdx + 1).trim();
					var predicate = predicateIdx == -1 ? null : elements[1].substring(predicateIdx + 1).trim();

					// Setup the parameters first
					if(query.parameters != undefined) {
						scope.parameters = {};
						for(var idx = 0; idx < Object.keys(query.parameters).length; idx++) {
							var key = Object.keys(query.parameters)[idx];
							scope.parameters[key] = query.parameters[key];
						}
					}
					
					// Setup the query
					scope.query = {
						id: 			query.id,
						table: 			scope.tables[tableName],
						type:			query.type,
						name:			query.name,
						select:			select,
						predicate:		predicate,
						template:		query.template
					};
				}
				
				scope.$applyAsync();
			});

			// On change of the table
			scope.$watch('query.table', function() {
				if(scope.query == null || scope.query.table == null) return;				
				scope.updateTemplate();
				scope.isFormDataValid = scope.validateFormData();
				scope.isQueryValid = null;
			});

			// On change of the query id
			scope.$watch('query.id', function() {
				if(scope.query == null || scope.query.id == null) return;				
				scope.isFormDataValid = scope.validateFormData();
			});
			
			// On change of the select clause
			scope.$watch('query.select', function() {
				if(scope.query == null || scope.query.select == null) return;				
				scope.updateTemplate();
				scope.isQueryValid = null;
				scope.isFormDataValid = scope.validateFormData();
			});
			
			// On change of the predicate clause
			scope.$watch('query.predicate', function() {
				if(scope.query == null || scope.query.predicate == null) return;				
				scope.updateTemplate();
				scope.isQueryValid = null;
				scope.isFormDataValid = scope.validateFormData();
			});
			
			// Watch for changes in the template
			scope.$watch('query.template', function() {
				// If there is a query template, parse it into parameters
				if(scope.query != undefined && scope.query.template != undefined) {
					if(scope.parameters == undefined) scope.parameters = {};
					var matches = scope.query.template.match(/\@.\w*/g);
					if(matches != null) {
						for(var idx = 0; idx < matches.length; idx++) {
							var key = matches[idx];
							if(scope.parameters[key] == null) {
								scope.parameters[key] = 'null';
							}
						}
					}
					var keys = [];
					for(var idx = 0; idx < Object.keys(scope.parameters).length; idx++) {
						var key = Object.keys(scope.parameters)[idx];
						if(matches == null || matches.indexOf(key) == -1)
							keys.push(key);
					}
					for(var idx = 0; idx < keys.length; idx++)
						delete scope.parameters[keys[idx]];
					
					if(Object.keys(scope.parameters) == 0) scope.parameters = undefined;
				}				
			});
			
			// On change of isFormDataValue
			scope.$watch('isFormDataValid', function() {
				if(scope.isQueryValid == true && scope.isFormDataValid == true)
					$(element).find('.btn-submit').removeAttr('disabled');
				else
					$(element).find('.btn-submit').attr('disabled','disabled');
			});
			
			// On change of isQueryValid
			scope.$watch('isQueryValid', function() {
				if(scope.isQueryValid == true && scope.isFormDataValid == true)
					$(element).find('.btn-submit').removeAttr('disabled');
				else
					$(element).find('.btn-submit').attr('disabled','disabled');
			});
			
			// Schema field double click
			scope.schemaFieldSelect = function(field) {
				if(scope.query.select == undefined || scope.query.select == null || scope.query.select.trim().length == 0)
					scope.query.select = field.name;
				else
					scope.query.select = scope.query.select + ", " + field.name;
				scope.isFormDataValid = scope.validateFormData();
			};
			
			// Set parameter
			scope.setParameter = function(paramName, val) {
				scope.parameters[paramName] = val;
				scope.isQueryValid = null;
				scope.$applyAsync();
			};

			// Create a template from the Select and Predicate form fields
			scope.updateTemplate = function() {
				if(scope.query == null || scope.query.select == undefined || scope.query.table == undefined)
					return;
				
				var template = "select " + scope.query.select + " from " + scope.query.table.name;
				if(scope.query.predicate != undefined && scope.query.predicate != null && scope.query.predicate.length > 0)
					template = template + " " + scope.query.predicate;
				scope.query.template = template;
			};	

			// Add validation error to element
			scope.addValidationError = function(elementId, text) {
				scope.removeValidationError(elementId);
				var group = $(element).find(elementId);
				var control = group.find('.form-control');
				group.addClass('query-form-error');
				control.tooltip({
					animation:	false,
					placement:	'right',
					title:		text,
					trigger:	'manual'
				});
				control.tooltip('show');
			};

			// Remove validation error from element
			scope.removeValidationError = function(elementId) {
				// Remove all form errors
				if(elementId == undefined) {
					$(element).find('.query-form-error').removeClass('query-form-error');
					$(element).find('.form-control').tooltip('destroy');
				}
				// Remove specific element error
				else {
					var group = $(element).find(elementId);
					var control = group.find('.form-control');
					group.removeClass('query-form-error');
					control.tooltip('destroy');
				}
			};
			
			// Validate the form data
			scope.validateFormData = function() {
				// Wait until the user tries to submit the modal or validate the fields
				if(scope.isModalSubmit == false) return null;
					
				var valid = true;
				if(scope.query.table == undefined || scope.query.table == null) {
					valid = false;
					scope.addValidationError('#queryTableGroup', 'Select a Table');
				}
				else
					scope.removeValidationError('#queryTableGroup');
				
				if(scope.query.id == undefined || scope.query.id == null || scope.query.id.length == 0) {
					valid = false;
					scope.addValidationError('#queryIdGroup', 'Enter a Query ID');
				}
				else if(scope.mode == 'add' && scope.connection.getQuery(scope.query.id) != undefined) {
					valid = false;
					scope.addValidationError('#queryIdGroup', 'Query ID already exists');
				}				
				else
					scope.removeValidationError('#queryIdGroup');
					
				if(scope.query.select == undefined || scope.query.select == null || scope.query.select.length == 0) {
					valid = false;									
					scope.addValidationError('#querySelectGroup', 'Enter fields to Select');
				}
				else 
					scope.removeValidationError('#querySelectGroup');
				
				return valid;
			};

			// Validate the query
			scope.validateQuery = function() {
				scope.isModalSubmit = true;
				scope.isFormDataValid = scope.validateFormData();
				if(scope.isFormDataValid == false) return;
				
				// Execute the snapshot and get the results
				var validationQuery = {
					id: 		scope.query.id,
					name: 		scope.query.name,
					template: 	scope.query.template,
					parameters: scope.parameters
				};
				
				conn.validate(validationQuery, 
					function(response) {
						scope.isQueryValid = response.valid;
						scope.error = response.message;
						scope.$apply();
					});					
			};
			
			// Update the query
			scope.update = function() {
				scope.isModalSubmit = true;
				scope.isFormDataValid = scope.validateFormData();
				if(scope.isFormDataValid == false) return;

				// Generate the new query
				var q = {
					id: 		scope.query.id,
					name: 		scope.query.name,
					template: 	scope.query.template,
					parameters: scope.parameters,
					primaryKey: query.primaryKey,
					snapshot:   query.snapshot,
					fullRow:	query.fullRow,
					dataset:	query.dataset
				};

				// Remove the existing query and register the new query
				conn.unregister(query);
				conn.register(q);

				// Close the modal
				scope.close();
			};
		}
	};	
});

app.directive('datasetLocalView', function($log, $window) {
	return {
		templateUrl: './templates/dataset-modal-local-view.html',
		restrict: 'E',
		scope: false,
		replace: true,
		controller: function($scope) {
		},
		link: function(scope, element, attrs) {	
		}
	};	
});

app.directive('datasetServiceView', function($log, $window) {
	return {
		templateUrl: './templates/dataset-modal-service-view.html',
		restrict: 'E',
		scope: false,
		replace: true,
		controller: function($scope) {
		},
		link: function(scope, element, attrs) {	
		}
	};	
});

// TODO Dataset Add Mode
