try {
	// This adds context menu to a Highcharts point
	(function (Highcharts) {
		
		Highcharts.wrap(Highcharts.Chart.prototype, 'firstRender', function (proceed) {

			proceed.call(this);
			
			var chart = this,
				container = this.container,
				plotLeft = this.plotLeft,
				plotTop = this.plotTop,
				plotWidth = this.plotWidth,
				plotHeight = this.plotHeight,
				inverted = this.inverted,
				pointer = this.pointer;

			// Note: 
			// - Safari 5, IE8: mousedown, contextmenu, click
			// - Firefox 5: mousedown contextmenu
			container.oncontextmenu = function(e) { 
				var hoverPoint = chart.hoverPoint,
					chartPosition = pointer.chartPosition;
			
				this.rightClick = true;

				e = pointer.normalize(e);
				
				// These prevent the context menu from appearing if not clicking a point
				/*e.cancelBubble = true; // IE specific
				e.returnValue = false; // IE 8 specific
				if (e.stopPropagation) {
					e.stopPropagation();
				}
				if (e.preventDefault) {
					e.preventDefault();
				}*/

				if (!pointer.hasDragged) {
					 if (hoverPoint && pointer.inClass(e.target, 'highcharts-tracker')) {
						 var plotX = hoverPoint.plotX,
							 plotY = hoverPoint.plotY;
							 
						 // add page position info
						 Highcharts.extend(hoverPoint, {
							 pageX: chartPosition.left + plotLeft + 
								 (inverted ? plotWidth - plotY : plotX),
							 pageY: chartPosition.top + plotTop + 
								 (inverted ? plotHeight - plotX : plotY)
						 });
						 
						// the series click event
						/*HighchartsAdapter.fireEvent(
							 hoverPoint.series, 'contextmenu', 
							 Highcharts.extend(e, {
								 point: hoverPoint
							 })
						);*/
						 
						 // the point click event
						 hoverPoint.firePointEvent('contextmenu', e);
						 
						 // These prevent the context menu from firing again for the underlying UI
						e.cancelBubble = true; // IE specific
						e.returnValue = false; // IE 8 specific
						if (e.stopPropagation) {
							e.stopPropagation();
						}
						if (e.preventDefault) {
							e.preventDefault();
						}
					 }
			   }
			}
		});
		
	}(Highcharts));
}
catch(err) {
}


app.directive('livechart', function($log, $filter, Overlay, Modal, EventListener, ContextMenu, Clipboard) {
	return {
		templateUrl: './templates/live-chart.html',
		replace: false,
		transclude: true,
		restrict: 'E',
		scope: {
			dataset: 	'=',
		},
		controller: function($scope) {
		},
		link: function(scope, element, attrs, ctrl, transclude) {
			// Add flexbox classes to the directive element
			$(element).addClass('flex-vertical-container fill-remainder');

			// Dependencies check
			scope.dependencies = function() {
				try {
					var h = Highcharts;
					return true;
				}
				catch(err) {
					return false;
				}
				return false;
			};
			
			// DEFINE OBJECTS
			{
				// Color Scale object
				{
					function ColorScale(scale) {
						this.scale = scale;
						this.lastUsed = -1;
						this.colorMap = {};
					};
					
					ColorScale.prototype.color = function(key) {
						if(this.colorMap[key] != undefined) 
							return this.colorMap[key];
						this.lastUsed = this.lastUsed + 1;
						if(this.lastUsed == this.scale.length) 
							this.lastUsed = 0;
						var color = this.scale[this.lastUsed];
						this.colorMap[key] = color;
						return color;
					};
				}	
				
				// LiveChart object
				{
					function LiveChart(attrs, options) {
						var self 		= this;

						// These properties are used for microbatching the updates
						this.microbatch = {
							rtime:		new Date(1, 1, 2000, 12,00,00),
							timeout:	false,
							delta:		100
						}				
									
						this.category 	= attrs.category;
						this.series 	= new Collection(['id']);
						this.highcharts = new Highcharts.Chart(options, function() {
							self.chartReady(this);
						});
					};
					
					LiveChart.prototype.clear = function() {
						for(var idx = 0; idx < this.series.length(); idx++) {
							var s = this.series.getByIndex(idx);
							s.removeAllPoints(false);
						}
					}
					
					LiveChart.prototype.chartReady = function(chart) {						
						// Create the series from the configuration
						var seriesSetConfig = eval(attrs.series);
						var maxPoints = attrs.seriesMaxPoints;
						for(var idx = 0; idx < seriesSetConfig.length; idx++) {
							var seriesConfig = seriesSetConfig[idx];
							var series = new LiveChartSeries(seriesConfig, chart, maxPoints);
							this.series.put(series);
						}
						
						// Redraw the chart
						chart.redraw();
					};
					
					LiveChart.prototype.resize = function() {
						this.highcharts.setSize(element.parent().width(), element.parent().height(), false);
					};
					
					LiveChart.prototype.update = function() {
						var self = this;
						
						// Microbatch handler
						if(self.microbatch.timeout === false) {
							self.microbatch.timeout = true;
							setTimeout(eventEnd, self.microbatch.delta);
						}					
						function eventEnd() {
							if (new Date() - self.microbatch.rtime < self.microbatch.delta)
								setTimeout(eventEnd, self.microbatch.delta);
							else {
								self.microbatch.rtime = new Date();
								self.microbatch.timeout = false;
								run();
							}               
						};
						
						// Run the update
						function run() {
							self.highcharts.redraw();
						};
					};
				}
				
				// LiveChartSeries object
				
				// Series object configuration in html
				// series = {
				//   close:         Candlestick only. Field name containing the close point value.
				//   color:			All charts. Function that evaluates to a color based on the object data. This will
				//                    override the default handling for the point color. Must be in the format:
				//                	    function(object) {return <color expression>;}
				//   colorValue:	All charts. Field name that determines the point color using the color scale. This
				//                    will override the default handling for the point color. May also set the attribute
				//                    color-category on the livechart to set the array of colors that are used.
				//   data: 			All charts. Field name containing the data point value to plot on the chart.
				//   high: 			Arearange and Candlestick only. Field name containing the high data point value.
				//   low:			Arearange and Candlestick only. field name containing the low data point value.
				//   name:			All charts. Text string describing the field name for legend, etc.
				//   open:          Candlestick only. Field name containing the open point value.
				//   parent:		Treemap only. Array of objects defining the parent properties of each object
				//					[{
				//						data:	Field name containing the data point value to plot on the chart.
				//						name:	Text string describing the field name for legend, etc.
				//					}]
				//   radius:		Bubble only. Field name containing the value for the bubble radius.
				//   seriesColor:	All charts. Expression that evaluates to a color constant that will override the
				//                    color for a given series.
				//   value: 		Heatmap and Treemap with color axisonly. Field name containing the value for the 
				// 				  	  heat intensity on the heatmap or treemap.
				// }
				{
					function LiveChartSeries(seriesConfig, chart, maxPoints) {
						var self			= this;
						
						this.id				= 'series.' + seriesConfig.data;
						this.close 			= seriesConfig.close;
						this.color 			= seriesConfig.color;
						this.colorValue 	= seriesConfig.colorValue;
						this.data 			= seriesConfig.data;
						this.high 			= seriesConfig.high;
						this.low 			= seriesConfig.low;
						this.maxPoints		= maxPoints;
						this.name 			= seriesConfig.name;
						this.open 			= seriesConfig.open;
						this.parent 		= seriesConfig.parent;
						this.radius 		= seriesConfig.radius;
						this.seriesColor	= seriesConfig.seriesColor;
						this.value 			= seriesConfig.value;
						
						this.chart			= chart;
						this.pointData 		= new Collection(['id']);

						// Setup Highcharts series options for all series
						this.options = {
							id:		self.id,
							type:	attrs.chartType,
							name: 	self.name == undefined ? self.data : self.name,
							color:	self.seriesColor,
						}
						
						// Setup Highcharts series options based on chart type
						if(attrs.chartType == 'pie') {
							this.options.innerSize = attrs.pieInnerSize;
						}
						else if(attrs.chartType == 'polar') {
							this.options.type = seriesConfig.type;
						}
						else if(attrs.chartType == 'heatmap') {
							this.options.borderWidth = 1;
						}
						else if(attrs.chartType == 'treemap') {
							this.options.levels = [];
							if(this.parent != undefined) {
								for(var levIdx = 0; levIdx < this.parent.length; levIdx++) {
									this.options.levels.push({
										borderWidth: 1,
										level: levIdx + 1,
										dataLabels: {
											enabled: true,
											align: levIdx % 2 == 0 ? 'left' : 'right',
											verticalAlign: 'top',
											style: {
												fontSize: (15 - 2 * levIdx) + 'px',
												fontWeight: 'bold'
											}
										}
									});
								}
							}
						}
						
						// Create the Highcharts series
						this.series = chart.addSeries(this.options, false);
					}
					
					// Adds a new point to the series
					LiveChartSeries.prototype.addPoint = function(datum, redraw) {
						this.pointData.put(new LiveChartPoint(datum, this, this.chart));
						var animation = false;
						var shift = this.maxPoints != undefined && this.pointData.length() > this.maxPoints;
						if(this.maxPoints == undefined || this.pointData.length() <= this.maxPoints)
							this.series.addPoint(datum, redraw, shift, animation);
						else {
							this.series.addPoint(datum, redraw, shift, animation);
						}
					};
					
					// Remove the point from the series
					LiveChartSeries.prototype.removePoint = function(point) {
						this.pointData.remove(point.id);
					};
					
					// Remove all points from the series
					LiveChartSeries.prototype.removeAllPoints = function(redraw) {
						this.series.setData([]);
						this.pointData.removeAll();
					};
					
					// Get the point data for the given id
					LiveChartSeries.prototype.getPoint = function(id) {
						return this.pointData.get(id);
					};
				}
			
				// LiveChartPoint object
				{
					function LiveChartPoint(datum, series, chart) {
						this.id 	= datum.id;
						this.datum 	= datum;
						this.series = series;
						this.chart  = chart;
						this.point	= null;
					};
					
					LiveChartPoint.prototype.remove = function(redraw) {
						// Setup function to remove the point
						var self = this;
						var removePoint = function(point) {
							point.remove(redraw);
							self.series.removePoint(self);
						};
						
						// If the point is not yet set, wait for it to be set on the chart
						if(this.point == null)
							this.waitForPoint(this.id, removePoint);
						else
							removePoint(this.point);
					};
					
					LiveChartPoint.prototype.update = function(datum, redraw) {
						this.datum = datum;

						// Setup function to update the point
						var updatePoint = function(point) {
							point.update(datum, redraw);
						};
						
						// If the point is not yet set, wait for it to be set on the chart
						if(this.point == null)
							this.waitForPoint(datum.id, updatePoint);
						else
							updatePoint(this.point);
					}
					
					LiveChartPoint.prototype.waitForPoint = function(id, onReady) {
						var self = this;

						this.point = this.chart.get(id);						

						// This is a bit of Highcharts weirdness. Because series.addPoint() doesn't
						//   return the point, have to go back and do chart.get() to retrieve the point
						//   object. However, the point does not exist until the chart is redrawn, so 
						//   it's possible this function will return null if there is a quick update after
						//   the initial create before the point is initially drawn. If that happens, 
						//   then this.point will still be null and we need to wait for a microbatch interval
						//   before trying again.
						if(this.point == null) {
							setTimeout(function() {
								self.point = self.chart.get(id);
								if(self.point == null)
									self.waitForPoint(id, onReady);
							},250);
						}
						else {
							onReady(this.point);
						}
					};
					
				}
			}
			
			// Transclude own scope
			transclude(scope, function (clone, scope) {
				element.append(clone);
			});
			
			// Add flexbox classes to the directive element
			$(element).find('.chart-container').addClass('flex-vertical-container fill-remainder');
			
			// Set the element ID to scope if it exists
			if(attrs.id != undefined) scope.elementId = attrs.id;
			
			// Set the name to scope if it exists
			if(attrs.name != undefined) scope.elementName = attrs.name;
			
			// Check dependencies
			if(scope.dependencies() == false) return;
			
			// Define color categories
			scope.color_categories = {
				color_highcharts:	Highcharts.getOptions().colors,
				
				color_category10:	[
									  '#1f77b4', '#ff7f0e', '#2ca02c', '#d62728', '#9467bd',
									  '#8c564b', '#e377c2', '#7f7f7f', '#bcbd22', '#17becf'
									],
									
				color_category20a:	[
									  '#1f77b4', '#aec7e8', '#ff7f0e', '#ffbb78',
									  '#2ca02c', '#98df8a', '#d62728', '#ff9896',
									  '#9467bd', '#c5b0d5', '#8c564b', '#c49c94',
									  '#e377c2', '#f7b6d2', '#7f7f7f', '#c7c7c7',
									  '#bcbd22', '#dbdb8d', '#17becf', '#9edae5'
									],

				color_category20b:	[
									  '#393b79', '#5254a3', '#6b6ecf', '#9c9ede',
									  '#637939', '#8ca252', '#b5cf6b', '#cedb9c',
									  '#8c6d31', '#bd9e39', '#e7ba52', '#e7cb94',
									  '#843c39', '#ad494a', '#d6616b', '#e7969c',
									  '#7b4173', '#a55194', '#ce6dbd', '#de9ed6'
									],

				color_category20c: 	[
									  '#3182bd', '#6baed6', '#9ecae1', '#c6dbef',
									  '#e6550d', '#fd8d3c', '#fdae6b', '#fdd0a2',
									  '#31a354', '#74c476', '#a1d99b', '#c7e9c0',
									  '#756bb1', '#9e9ac8', '#bcbddc', '#dadaeb',
									  '#636363', '#969696', '#bdbdbd', '#d9d9d9'
									],
			};

			// Read the category and series
			//   Category is the X-axis, series is an array
			//   of objects defining the data series
			scope.categories = {xaxis:[], yaxis:[]};
			scope.parents = [];
			
			// Setup a color array based on the configured scope
			scope.colors = attrs.colorCategory != undefined ? eval('scope.color_categories.' + attrs.colorCategory) : scope.color_categories.color_highcharts;
			
			// Setup a color scale
			scope.colorScale = new ColorScale(scope.colors);
			
			// Read the options into scope
			scope.options = {
				"configIcon": 		attrs.configIcon == undefined ? 
										false : 
										eval(attrs.configIcon),
									// Optional. Show or hide config icon. May be true or false.
										
				"datasetIcon": 		attrs.datasetIcon == undefined ? 
										false : 
										eval(attrs.datasetIcon),
									// Optional. Show or hide dataset icon field. May be true or false.
										
				"datasetModal":		attrs.datasetModal == "R" || attrs.datasetModal == "RW" ?
										attrs.datasetModal :
										undefined,
									// Optional. Show the dataset modal if set. May be one of:
									//   R -- read-only modal
									//   RW -- read-write modal
			};
			
			// Read the chart options into scope
			scope.chartOptions = {
				chart: {
					animation:				attrs.chartAnimation == undefined ? true : eval(attrs.chartAnimation),
											// Optional. Enable chart animation. Default is true.
									
					backgroundColor:		attrs.chartBackgroundColor == undefined ? '#FFFFFF' : attrs.chartBackgroundColor,
											// Optional. Background color for the chart. Default is #FFFFFF.
										
					borderColor:			attrs.chartBorderColor,
											// Optional. Border color for the chart. Default is #4572A7.
										
					borderRadius:			attrs.chartBorderRadius == undefined ? 0 : eval(attrs.chartBorderRadius),
											// Optional. Border radius for corners. Default is 0.
										
					borderWidth:			attrs.chartBorderWidth == undefined ? 0 : eval(attrs.chartBorderWidth),
											// Optional. Border width for the chart. Default is 0.
						
					inverted:				attrs.chartInverted == undefined ? false : eval(attrs.chartInverted),
					
					margin:					attrs.chartMargin == undefined ? undefined : eval(attrs.chartMargin),
											// Optional. Margin between outer edge of chart and plot area. Array in the form
											//   [top, right, bottom, left]. Default is 0 margin on all dimensions.
										
					panKey:					attrs.chartPanKey,
											// Optional. Key to press to switch between zooming and panning. 
									
					panning:				attrs.chartPanning == undefined ? true : eval(attrs.chartPanning),
											// Optional. Enable chart panning. Default is false.
									
					plotBackgroundColor:	attrs.chartPlotBackgroundColor,
											// Optional. Plot background color for the chart.
										
					plotBackgroundImage:	attrs.chartPlotBackgroundImage,
											// Optional. Plot background image for the chart.
										
					plotBorderColor:		attrs.chartPlotBorderColor,
											// Optional. Plot border color for the chart.
										
					plotBorderWidth:		attrs.chartPlotBorderWidth,
											// Optional. Plot border width for the chart.
										
					renderTo: 				$(element).find('.chart-container')[0],
											// Mandatory, location to render the chart
													
					spacing:				attrs.chartSpacing == undefined ? [10,10,15,10] : eval(attrs.chartSpacing), 
											// Optional. Distance between outer edge of chart and content. Array in the form
											//   [top, right, bottom, left]. Default is [10,10,15,10].
											
					style:					attrs.chartStyle == undefined ? {} : eval('(' + attrs.chartStyle + ')'),
											// Optional. CSS object styling to apply to chart. 
									
					type: 					attrs.chartType == undefined ? 'line' : attrs.chartType,
											// Optional. Type of chart to display. May be one of area, arearange, areaspline, 
											//   areasplinerange, bar, boxplot, bubble, column, columnrange, errorbar, funnel, 
											//   gauge, heatmap, line, pie, polygon, pyramid, scatter, series, solidgauge, 
											//   spline, treemap, waterfall. Defaults to line if omitted.
											
					zoomType:				attrs.chartZoomType,
											// Optional. Decides in what dimensions the user can zoom by dragging the mouse. 
											//   Can be one of x, y or xy. Default is no zoom.
				},
				colorAxis: {
					min:					attrs.colorAxisMin == undefined ? undefined : eval(attrs.colorAxisMin),
					minColor:				attrs.colorAxisMinColor == undefined ? undefined : attrs.colorAxisMinColor,
					maxColor:				attrs.colorAxisMaxColor == undefined ? undefined : eval(attrs.colorAxisMaxColor),
				},
				credits: {
					enabled:				false,
											// Mandatory. Whether to show the credits text.
				},
				legend: {
					align:					attrs.legendAlign == undefined ? "center" : attrs.legendAlign,
											// Optional. Legend alignment. May be left, center, or right. Default is center.

					backgroundColor:		attrs.legendBackgroundColor == undefined ? "rgba(255, 255, 255, 1.00)" : attrs.legendBackgroundColor,
											// Optional. Legend background color for the chart.
					
					borderColor:			attrs.legendBorderColor == undefined ? '#909090' : attrs.legendBorderColor,
											// Optional. Legend border color.
											
					borderRadius:			attrs.legendBorderRadius == undefined ? 0 : eval(attrs.legendBorderRadius),
											// Optional. Legend border radius.

					borderWidth:			attrs.legendBorderWidth == undefined ? 0 : eval(attrs.legendBorderWidth),
											// Optional. Legend border width.

					enabled:				attrs.legendEnabled == undefined ? false : eval(attrs.legendEnabled),
											// Optional. Display the legend. Default is true.
											
					floating:				attrs.legendFloating == undefined ? false : eval(attrs.legendFloating),
											// Optional. Float the legend. Default is false.
											
					itemDistance:			attrs.legendItemDistance == undefined ? 20 : eval(attrs.legendItemDistance),
											// Optional. In a legend with horizontal layout, the itemDistance defines the pixel 
											//   distance between each item. Defaults to 20.
					
					itemMarginBottom:		attrs.legendItemMarginBottom == undefined ? 0 : eval(attrs.legendItemMarginBottom),
											// Optional. The pixel bottom margin for each legend item. Defaults to 0.
											
					itemMarginTop:			attrs.legendItemMarginTop == undefined ? 0 : eval(attrs.legendItemMarginTop),
											// Optional. The pixel top margin for each legend item. Defaults to 0.
					
					labelFormat:			attrs.legendLabelFormat,
											// Optional. A format string for each legend label. Available variables relates to 
											//   properties on the series, or the point in case of pies. Defaults to {name}.
					
					layout:					attrs.legendLayout == undefined ? "horizontal" : attrs.legendLayout,
											// Optional. The layout of the legend items. Can be one of horizontal or vertical. 
											//   Defaults to horizontal.
											
					margin:					attrs.legendMargin == undefined ? 12: eval(attrs.legendMargin),
											// Optional. If the plot area sized is calculated automatically and the legend is not 
											//   floating, the legend margin is the space between the legend and the axis labels 
											//   or plot area. Defaults to 12.
											
					maxHeight:				attrs.legendMaxHeight == undefined ? null : eval(attrs.legendMaxHeight),
											// Optional. Maximum pixel height for the legend. When the maximum height is extended, 
											//   navigation will show.
											
					padding: 				attrs.legendPadding == undefined ? 8 : eval(attrs.legendPadding),
											// Optional. Inner padding of the legend box. Default is 8.

					shadow:					attrs.legendShadow == undefined ? null : eval(attrs.legendShadow),
											// Optional. Whether to apply a drop shadow to the legend. A backgroundColor 
											//   also needs to be applied for this to take effect. Defaults to false.
											
					symbolHeight:			attrs.legendSymbolHeight == undefined ? null : eval(attrs.legendSymbolHeight),
											// Optional. The pixel height of the symbol for series types that use a rectangle in the 
											//   legend. Defaults to the font size of legend items.
											
					symbolPadding:			attrs.legendSymbolPadding == undefined ? 5 : eval(attrs.legendSymbolPadding),
											// Optional. The pixel padding between the legend item symbol and the legend item text. 
											//   Defaults to 5.
											
					symbolRadius:			attrs.legendSymbolRadius == undefined ? 0 : eval(attrs.legendSymbolRadius),
											// Optional. The border radius of the symbol for series types that use a rectangle in 
											//   the legend. Defaults to 0.

					symbolWidth:			attrs.legendSymbolWidth == undefined ? 16 : eval(attrs.legendSymbolWidth),
											// Optional. The pixel width of the legend item symbol. Defaults to 16.
											
					title: {
						style:				attrs.legendTitleStyle == undefined ? {} : eval('(' + attrs.legendTitleStyle + ')'),
											// Optional. Style to apply to legend title.
											
						text:				attrs.legendTitleText,
											// Optional. Legend title text.
					},
					
					useHTML:				attrs.legendUseHtml == undefined ? false : eval(attrs.legendUseHtml),
											// Optional. Use HTML in legend.
											
					verticalAlign:			attrs.legendVerticalAlign == undefined ? "middle" : attrs.legendVerticalAlign,
											// Optional. Vertical alignment. May be top, middle, or bottom.
										
					//width:				attrs.legendWidth == undefined ? undefined : eval(attrs.legendWidth),
											// Optional. Legend width.
											
					x:						attrs.legendX == undefined ? 0 : eval(attrs.legendX),
					
					y:						attrs.legendY == undefined ? 0 : eval(attrs.legendY),
				},
				
				// Applies only to polar and gauge.
				pane: {
					center:					attrs.paneCenter == undefined ? ['50%', '50%'] : eval(attrs.paneCenter),
											// Optional. Center of the pane.
											
					endAngle:				attrs.paneEndAngle == undefined ? 360 : eval(attrs.paneEndAngle),
											// Optional. End angle.
					
					size:					attrs.paneSize == undefined ? "85%" : attrs.paneSize,
											// Optional. Pane size.
											
					startAngle:				attrs.paneStartAngle == undefined ? 0 : eval(attrs.paneStartAngle),
											// Optional. Start angle.
											
				},
				
				plotOptions: {
					series: {
						dataLabels:	{
							//format:			attrs.seriesDataLabelsFormat == undefined ? undefined : attrs.seriesDataLabelsFormat,
						},
						//stacking: 			attrs.seriesStacking == undefined ? undefined : eval(attrs.seriesStacking),
						states: {
							hover: {
								enabled:		attrs.seriesStatesHoverEnabled == undefined ? true : eval(attrs.seriesStatesHoverEnabled),
							}
						},
					}
				},
				subtitle: {
					align:			attrs.subtitleAlign == undefined ? "center" : attrs.subtitleAlign,
									// Optional. Sub-title alignment. Must be left, center, or right. Default is left if omitted.
									
					floating:		attrs.subtitleFloating == undefined ? undefined : eval(attrs.subtitleFloating),
									// Optional. When floating, the plot area will not make room for the title. Default is false.
									
					style:			attrs.subtitleStyle == undefined ? {} : eval('(' + attrs.subtitleStyle + ')'),
									// Optional. CSS styling to apply to sub-title. 
									
					text: 			attrs.subtitleText,
									// Optional. Sub-title.

					useHTML: 		attrs.subtitleUseHtml,
									// Optional. Use HTML for main title. Default is false.
				},
				title: {
					align:			attrs.titleAlign == undefined ? "center" : attrs.titleAlign,
									// Optional. Main title alignment. Must be left, center, or right. Default is left if omitted.
									
					floating:		attrs.titleFloating == undefined ? undefined : eval(attrs.titleFloating),
									// Optional. When floating, the plot area will not make room for the title. Default is false.
									
					text: 			attrs.titleText,
									// Optional. Main title.

					style:			attrs.titleStyle == undefined ? {} : eval('(' + attrs.titleStyle + ')'),
									// Optional. CSS styling to apply to title. 
									
					useHTML: 		attrs.titleUseHtml,
									// Optional. Use HTML for main title. Default is false.
				},
				tooltip: {
					animation:				attrs.tooltipAnimation == undefined ? true : eval(attrs.tooltipAnimation),
											// Optional. Enable or disable animation of the tooltip. Default is true.
											
					backgroundColor:		attrs.tooltipBackgroundColor == undefined ? "rgba(255, 255, 255, 0.85)" : attrs.tooltipBackgroundColor,
											// Optional. The background color or gradient for the tooltip.
											
					borderColor:			attrs.tooltipBorderColor,
											// Optional. The color of the tooltip border. When null, the border takes the color of the corresponding 
											//   series or point. Defaults to null.
											
					borderRadius:			attrs.tooltipBorderRadius == undefined ? 3 : eval(attrs.tooltipBorderRadius),
											// Optional. The radius of the rounded border corners. Defaults to 3.
											
					borderWidth:			attrs.tooltipBorderWidth == undefined ? 1 : eval(attrs.tooltipBorderWidth),
											// Optional. The pixel width of the tooltip border. Defaults to 1.
											
					crosshairs:				attrs.tooltipCrosshairs == undefined ? undefined : eval(attrs.tooltipCrosshairs),
											// Optional. Display crosshairs to connect the points with their corresponding axis values. The 
											//   crosshairs can be defined as a boolean, an array of booleans or an object.
											
					dateTimeLabelFormats:	attrs.tooltipDateTimeFormats == undefined ? {} : eval('(' + attrs.tooltipDateTimeFormats + ')'),
											// Optional. For series on a datetime axes, the date format in the tooltip's header will by default 
											//   be guessed based on the closest data points. This member gives the default string representations 
											//   used for each unit. 
											
					enabled:				attrs.tooltipEnabled == undefined ? true : eval(attrs.tooltipEnabled),
											// Optional. Enable or disable the tooltip. Defaults to true.
											
					followPointer:			attrs.tooltipFollowPointer == undefined ? false : eval(attrs.tooltipFollowPointer),
											// Optional. Whether the tooltip should follow the mouse as it moves across columns, pie slices and other 
											//   point types with an extent. Default is true for scatter, bubble, and pie series.
											
					followTouchMove:		attrs.tooltipFollowTouchMove = undefined ? false : eval(attrs.tooltipFollowTouchMove),
											// Optional. Whether the tooltip should follow the finger as it moves on a touch device. Defaults to true.
											
					//footerFormat:			attrs.tooltipFooterFormat == undefined ? false : attrs.tooltipFooterFormat,
											// Optional. A string to append to the tooltip format. Defaults to false. Seems to be an issue with layout
											//   for this attribute. Use formatter instead.
											
					formatter:				attrs.tooltipFormatter == undefined ? undefined : function(d) { return eval(attrs.tooltipFormatter); },
											// Optional. Callback function to format the text of the tooltip. Return false to disable tooltip for a 
											//   specific point on series. HTML attribute must evaluate to a JS expression that will be returned by
											//   this function.
											
					//headerFormat:			attrs.tooltipHeaderFormat == undefined ? "<span style='font-size:10px;'>{point.key}</span><br/><span style='padding-left:2px'></span>" : attrs.tooltipHeaderFormat, 
					//headerFormat:			attrs.tooltipHeaderFormat == undefined ? undefined : attrs.tooltipHeaderFormat, 
											// Optional. The HTML of the tooltip header line. Variables are enclosed by curly brackets. Available 
											//   variables are point.key, series.name, series.color and other members from the point and series 
											//   objects. The point.key variable contains the category name, x value or datetime string depending 
											//   on the type of axis. For datetime axes, the point.key date format can be set using tooltip.xDateFormat.
											// Defaults to <span style="font-size: 10px">{point.key}</span><br/>
											
					hideDelay:				attrs.tooltipHideDelay == undefined ? 500 : eval(attrs.tooltipHideDelay),
											// Optional. The number of milliseconds to wait until the tooltip is hidden when mouse out from a point or 
											//   chart. Defaults to 500.
											
					//pointFormat:			attrs.tooltipPointFormat == undefined ? "<span style='color:{point.color}'>\u25CF</span> {series.name}: <b>{point.y}</b><br/>" : attrs.tooltipPointFormat,
					//pointFormat:			attrs.tooltipPointFormat == undefined ? undefined : attrs.tooltipPointFormat,
											// Optional. The HTML of the point's line in the tooltip. Variables are enclosed by curly brackets. Available 
											//   variables are point.x, point.y, series.name and series.color and other properties on the same form. 
											//   Furthermore, point.y can be extended by the tooltip.valuePrefix and tooltip.valueSuffix variables. 
											//   This can also be overridden for each series, which makes it a good hook for displaying units.
											// Defaults to <span style="color:{point.color}">\u25CF</span> {series.name}: <b>{point.y}</b><br/>.
											
					shadow:					attrs.tooltipShadow == undefined ? true : eval(attrs.tooltipShadow == undefined),
											// Optional. Whether to apply a drop shadow to the tooltip. Defaults to true.
											
					shape:					attrs.tooltipShape == undefined ? "callout" : attrs.tooltipShape,
											// Optional. The name of a symbol to use for the border around the tooltip. 
											//   In Highcharts 3.x and less, the shape was square. Defaults to callout.
											
					style:					attrs.tooltipStyle == undefined ? {} : eval('(' + attrs.tooltipStyle + ')'),
											// Optional. CSS styles for the tooltip. 
											
					useHTML:				true,
											// Optional. Use HTML to render the contents of the tooltip instead of SVG. 
											
					valueDecimals:			attrs.tooltipValueDecimals,
											// Optional. How many decimals to show in each series' y value. This is overridable in each series' tooltip 
											//   options object. The default is to preserve all decimals.
											
					valuePrefix:			attrs.tooltipValuePrefix,
											// Optional. A string to prepend to each series' y value. Overridable in each series' tooltip options object.
											
					valueSuffix:			attrs.tooltipValueSuffix,
											// Optional. A string to append to each series' y value. Overridable in each series' tooltip options object.
											
					xDateFormat:			attrs.tooltipXDateFormat,
											// Optional. The format for the date in the tooltip header if the X axis is a datetime axis. The default is a 
											//   best guess based on the smallest distance between points in the chart.
				},
				xAxis: {
					allowDecimals:			attrs.xaxisAllowDecimals == undefined ? true : eval(attrs.xAxisAllowDecimals),
											// Optional. Whether to allow decimals in this axis' ticks. Defaults to true.
											
					alternateGridColor:		attrs.xaxisAlternateGridColor,
											// Optional. When using an alternate grid color, a band is painted across the plot area between every other 
											//   grid line.
											
					categories:				attrs.xaxisCategories == undefined ? undefined : eval(attrs.xaxisCategories),
											// Optional. If categories are present for the xAxis, names are used instead of numbers for that axis. 

					ceiling:				attrs.xaxisCeiling == undefined ? undefined : eval(attrs.xaxisCeiling),
											// Optional. The highest allowed value for automatically computed axis extremes.
											
					dateTimeLabelFormats:	attrs.xaxisDateTimeFormats == undefined ? {} : eval('(' + attrs.xaxisDateTimeFormats + ')'),
											// Optional. For a datetime axis, the scale will automatically adjust to the appropriate unit. This member 
											//   gives the default string representations used for each unit. 
											
					endOnTick:				attrs.xaxisEndOnTick == undefined ? false : eval(attrs.xaxisEndOnTick),
											// Optional. Whether to force the axis to end on a tick. Use this option with the maxPadding option to 
											//   control the axis end. Defaults to false.
											
					floor:					attrs.xaxisFloor == undefined ? undefined : eval(attrs.xaxisFloor),
											// Optional. The lowest allowed value for automatically computed axis extremes. Defaults to null.
											
					gridLineColor:			attrs.xaxisGridLineColor == undefined ? '#D8D8D8' : attrs.xaxisGridLineColor,
											// Optional. Color of the grid lines extending the ticks across the plot area. Defaults to #D8D8D8.
											
					gridLineDashStyle:		attrs.xaxisGridLineDashStyle,
											// Optional. The dash or dot style of the grid lines. 
											
					gridLineWidth:			attrs.xaxisGridLineWidth == undefined ? 0 : eval(attrs.xaxisGridLineWidth),
											// Optional. The width of the grid lines extending the ticks across the plot area. Defaults to 0.
											
					labels: {
						align:				attrs.xaxisLabelsAlign,
											// Optional. What part of the string the given position is anchored to. Can be one of "left", 
											//   "center" or "right". Defaults to an intelligent guess.
											
						enabled:			attrs.xaxisLabelsEnabled == undefined ? true : eval(attrs.xaxisLabelsEnabled),
											// Optional. Enable or disable the axis labels. Defaults to true.
											
						format:				attrs.xaxisLabelsFormat,
											// Optional. A format string for the axis label. Defaults to {value}.
											
						padding:			attrs.xaxisLabelsPadding == undefined ? undefined : eval(attrs.xaxisLabelsPadding),
											// Optional. The pixel padding for axis labels, to ensure white space between them. Defaults to 5.
											
						rotation:			attrs.xaxisLabelsRotation == undefined ? 0 : eval(attrs.xaxisLabelsRotation), 
											// Optional. Rotation of the labels in degrees. Defaults to 0.
											
						staggerLines:		attrs.xaxisLabelsStaggerLines == undefined ? undefined : eval(attrs.xaxisLabelsStaggerLines),
											// Optional. Horizontal axes only. The number of lines to spread the labels over to make room or 
											//   tighter labels.
											
						step:				attrs.xaxisLabelsStep == undefined ? undefined : eval(attrs.xaxisLabelsStep),
											// Optional. To show only every n'th label on the axis, set the step to n. Setting the step to 2 
											//   shows every other label.
											
						style:				attrs.xaxisLabelsStyle == undefined ? {} : eval('(' + attrs.xaxisLabelsStyle + ')'),
											// Optional. CSS styles for the label. 
											
						useHTML:			attrs.xaxisLabelsUseHtml == undefined ? false : eval(attrs.xaxisLabelsUseHtml),
											// Optional. Whether to use HTML to render the labels. Defaults to false.
					},
					
					lineColor:				attrs.xaxisLineColor == undefined ? "#C0D0E0" : attrs.xaxisLineColor,
											// Optional. The color of the line marking the axis itself. Defaults to #C0D0E0.
											
					lineWidth:				attrs.xaxisLineWidth == undefined ? 1 : eval(attrs.xaxisLineWidth),
											// Optional. The width of the line marking the axis itself. Defaults to 1.
											
					max:					attrs.xaxisMax == undefined ? undefined : eval(attrs.xaxisMax),
											// Optional. The maximum value of the axis. If null, the max value is automatically calculated. If the endOnTick 
											//   option is true, the max value might be rounded up. The actual maximum value is also influenced by 
											//   chart.alignTicks.
											
					maxPadding:				attrs.xaxisMaxPadding == undefined ? 0.05 : eval(attrs.xaxisMaxPadding),
											// Optional. Padding of the max value relative to the length of the axis. 
											
					min:					attrs.xaxisMin == undefined ? undefined : eval(attrs.xaxisMin),
											// Optional. The minimum value of the axis. If null the min value is automatically calculated. If the startOnTick 
											//   option is true, the min value might be rounded down.
											
					minPadding:				attrs.xaxisMinPadding == undefined ? 0.05 : eval(attrs.xaxisMinPadding),
											// Optional. Padding of the min value relative to the length of the axis. 
											
					minRange:				attrs.xaxisMinRange == undefined ? undefined : eval(attrs.xaxisMinRange),
											// Optional. The minimum range to display on this axis. 
											
					minTickInterval:		attrs.xaxisMinTickInterval == undefined ? undefined : eval(attrs.xaxisMinTickInterval),
											// Optional. The minimum tick interval allowed in axis values. Defaults to the closest distance between two points 
											//   on the axis.
											
					minorGridLineColor:		attrs.xaxisMinorGridLineColor,
											// Optional. Color of the minor, secondary grid lines. Defaults to #E0E0E0.
					
					minorGridLineDashStyle:	attrs.xaxisMinorGridLineDashStyle,
											// Optional. The dash or dot style of the minor grid lines. Defaults to Solid.
											
					minorGridLineWidth:		attrs.xaxisMinorGridLineWidth == undefined ? undefined : eval(attrs.xaxisMinorGridLineWidth),
											// Optional. Width of the minor, secondary grid lines. Defaults to 1.
											
					minorTickColor:			attrs.xaxisMinorTickColor,
											// Optional. Color for the minor tick marks. Defaults to #A0A0A0.
											
					minorTickInterval:		attrs.xaxisMinorTickInterval,
											// Optional. Tick interval in scale units for the minor ticks. On a linear axis, if "auto", the minor tick interval 
											//   is calculated as a fifth of the tickInterval. If null, minor ticks are not shown.
											
					minorTickLength:		attrs.xaxisMinorTickLength == undefined ? undefined : eval(attrs.xaxisMinorTickLength),
											// Optional. The pixel length of the minor tick marks. Defaults to 2.
											
					minorTickPosition:		attrs.xaxisMinorTickPosition,
											// Optional. The position of the minor tick marks relative to the axis line. Can be one of inside and outside. 
											//   Defaults to outside.
											
					minorTickWidth:			attrs.xaxisMinorTickWidth == undefined ? undefined : eval(attrs.xaxisMinorTickWidth),
											// Optional. The pixel width of the minor tick mark. Defaults to 0.
											
					offset:					attrs.xaxisOffset == undefined ? 0 : eval(attrs.xaxisOffset),
											// Optional. The distance in pixels from the plot area to the axis line. A positive offset moves the axis with it's 
											//   line, labels and ticks away from the plot area. This is typically used when two or more axes are displayed on 
											//   the same side of the plot. Defaults to 0.
											
					opposite:				attrs.xaxisOpposite == undefined ? false : eval(attrs.xaxisOpposite),
											// Optional. Whether to display the axis on the opposite side of the normal. The normal is on the left side for 
											//   vertical axes and bottom for horizontal, so the opposite sides will be right and top respectively. This is 
											//   typically used with dual or multiple axes. Defaults to false.
											
					plotBands:				attrs.xaxisPlotBands == undefined ? undefined : eval(attrs.xaxisPlotBands),
											// Optional. Array of objects defining plot bands on the X axis. The object is defined with the following fields:
											//   borderColor, borderWidth, color, events, from, id, innerRadius, label, outerRadius, thickness, to, zIndex.
					
					reversed:				attrs.xaxisReversed == undefined ? undefined : eval(attrs.xaxisReversed),
											// Optional. Whether to reverse the axis so that the highest number is closest to the origin. If the chart is inverted, 
											//   the x axis is reversed by default. Defaults to false.
											
					showEmpty:				attrs.xaxisShowEmpty == undefined ? true : eval(attrs.xaxisShowEmpty),
											// Optional. Whether to show the axis line and title when the axis has no data. Defaults to true.
											
					showFirstLabel:			attrs.xaxisShowFirstLabel == undefined ? true : eval(attrs.xaxisShowFirstLabel),
											// Optional. Whether to show the first tick label. Defaults to true.
											
					showLastLabel:			attrs.xaxisShowLastLabel == undefined ? true : eval(attrs.xaxisShowLastLabel),
											// Optional. Whether to show the last tick label. Defaults to true.
											
					startOfWeek:			attrs.xaxisStartOfWeek == undefined ? 1 : eval(attrs.xaxisStartOfWeek),
											// Optional. For datetime axes, this decides where to put the tick between weeks. 0 = Sunday, 1 = Monday. Defaults to 1.
											
					startOnTick:			attrs.xaxisStartOnTick == undefined ? false : eval(attrs.xaxisStartOnTick),
											// Optional. Whether to force the axis to start on a tick. Use this option with the minPadding option to control the axis 
											//   start. Defaults to false.
											
					tickAmount:				attrs.xaxisTickAmount == undefined ? undefined : eval(attrs.xaxisTickAmount),
											// Optional. The amount of ticks to draw on the axis. This opens up for aligning the ticks of multiple charts or panes 
											//   within a chart. This option overrides the tickPixelInterval option.
											
					tickColor:				attrs.xaxisTickColor == undefined ? '#C0D0E0' : attrs.xaxisTickColor,
											// Optional. Color for the main tick marks. Defaults to #C0D0E0.
											
					tickInterval:			attrs.xaxisTickInterval == undefined ? undefined : eval(attrs.xaxisTickInterval),
											// Optional. The interval of the tick marks in axis units. 
											
					tickLength:				attrs.xaxisTickLength == undefined ? 10 : eval(attrs.xaxisTickLength),
											// Optional. The pixel length of the main tick marks. Defaults to 10.
											
					tickPixelInterval:		attrs.xaxisTickPixelInterval == undefined ? 100 : eval(attrs.xaxisTickPixelInterval),
											// Optional. If tickInterval is null this option sets the approximate pixel interval of the tick marks. Not applicable 
											//   to categorized axis. Defaults to 72 for the Y axis and 100 for	the X axis.
											
					tickPosition:			attrs.xaxisTickPosition,
											// Optional. The position of the major tick marks relative to the axis line. Can be one of inside and outside. 
											//   Defaults to outside.
											
					tickWidth:				attrs.xaxisTickWidth == undefined ? 1 : eval(attrs.xaxisTickWidth),
											// Optional. The pixel width of the major tick marks. Defaults to 1.
											
					tickmarkPlacement:		attrs.xaxisTickmarkPlacement == undefined ? "between" : attrs.xaxisTickmarkPlacement,
											// Optional. For categorized axes only. If on the tick mark is placed in the center of the category, if between the tick 
											//   mark is placed between categories. The default is between if the tickInterval is 1, else on. Defaults to null.
					title: {
						align:				attrs.xaxisTitleAlign,
											// Optional. Alignment of the title relative to the axis values. Possible values are "low", "middle" or "high". 
											//   Defaults to middle.
											
						margin:				attrs.xaxisTitleMargin == undefined ? null : eval(attrs.xaxisTitleMargin),
											// Optional. The pixel distance between the axis labels or line and the title. Defaults to 0 for horizontal axes, 
											//   10 for vertical
											
						offset:				attrs.xaxisTitleOffset == undefined ? null : eval(attrs.xaxisTitleOffset),
											// Optional. The distance of the axis title from the axis line. By default, this distance is computed from the 
											//   offset width of the labels, the labels' distance from the axis and the title's margin. However when the 
											//   offset option is set, it overrides all this.
											
						rotation:			attrs.xaxisTitleRotation == undefined ? 0 : eval(attrs.xaxisTitleRotation),
											// Optional. The rotation of the text in degrees. 0 is horizontal, 270 is vertical reading from bottom to top. 
											//   Defaults to 0.
											
						style:				attrs.xaxisTitleStyle == undefined ? {} : eval('(' + attrs.xaxisTitleStyle + ')'),
											// Optional. CSS styles for the title. 
											
						text: 				attrs.xaxisTitleText,
											// Optional. The actual text of the axis title. It can contain basic HTML text markup like <b>, <i> and 
											//   spans with style.
					},
					
					type:					attrs.xaxisType,
											// Optional. The type of axis. Can be one of "linear", "logarithmic", "datetime" or "category". 
					
					units:					attrs.xaxisUnits == undefined ? undefined : eval(attrs.xaxisUnits),		
											// Optional. Datetime axis only. An array determining what time intervals the ticks are allowed to fall on. Each array 
											//   item is an array where the first value is the time unit and the second value another array of allowed multiples. 
				},
				yAxis: {
					allowDecimals:			attrs.yaxisAllowDecimals == undefined ? true : eval(attrs.yaxisAllowDecimals),
											// Optional. Whether to allow decimals in this axis' ticks. Defaults to true.

					alternateGridColor:		attrs.yaxisAlternateGridColor,
											// Optional. When using an alternate grid color, a band is painted across the plot area between every other 
											//   grid line.
					
					categories:				attrs.yaxisCategories == undefined ? undefined : eval(attrs.yaxisCategories),
											// Optional. If categories are present for the xAxis, names are used instead of numbers for that axis. 
					
					ceiling:				attrs.yaxisCeiling == undefined ? undefined : eval(attrs.yaxisCeiling),
											// Optional. The highest allowed value for automatically computed axis extremes.
					
					dateTimeLabelFormats:	attrs.yaxisDateTimeFormats == undefined ? {} : eval('(' + attrs.yaxisDateTimeFormats + ')'),
											// Optional. For a datetime axis, the scale will automatically adjust to the appropriate unit. This member 
											//   gives the default string representations used for each unit. 
					
					endOnTick:				attrs.yaxisEndOnTick == undefined ? false : eval(attrs.yaxisEndOnTick),
											// Optional. Whether to force the axis to end on a tick. Use this option with the maxPadding option to 
											//   control the axis end. Defaults to false.
					
					floor:					attrs.yaxisFloor == undefined ? undefined : eval(attrs.yaxisFloor),
											// Optional. The lowest allowed value for automatically computed axis extremes. Defaults to null.
					
					gridLineColor:			attrs.yaxisGridLineColor == undefined ? '#D8D8D8' : attrs.yaxisGridLineColor,
											// Optional. Color of the grid lines extending the ticks across the plot area. Defaults to #D8D8D8.
					
					gridLineDashStyle:		attrs.yaxisGridLineDashStyle,
											// Optional. The dash or dot style of the grid lines. 
					
					gridLineWidth:			attrs.yaxisGridLineWidth == undefined ? 1 : eval(attrs.yaxisGridLineWidth),
											// Optional. The width of the grid lines extending the ticks across the plot area. Defaults to 0.
					
					labels: {
						align:				attrs.yaxisLabelsAlign,
											// Optional. What part of the string the given position is anchored to. Can be one of "left", 
											//   "center" or "right". Defaults to an intelligent guess.
						
						enabled:			attrs.yaxisLabelsEnabled == undefined ? true : eval(attrs.yaxisLabelsEnabled),
											// Optional. Enable or disable the axis labels. Defaults to true.
						
						format:				attrs.yaxisLabelsFormat,
											// Optional. A format string for the axis label. Defaults to {value}.
						
						padding:			attrs.yaxisLabelsPadding == undefined ? undefined : eval(attrs.yaxisLabelsPadding),
											// Optional. The pixel padding for axis labels, to ensure white space between them. Defaults to 5.
						
						rotation:			attrs.yaxisLabelsRotation == undefined ? 0 : eval(attrs.yaxisLabelsRotation), 
											// Optional. Rotation of the labels in degrees. Defaults to 0.

						staggerLines:		attrs.yaxisLabelsStaggerLines == undefined ? undefined : eval(attrs.yaxisLabelsStaggerLines),
											// Optional. Horizontal axes only. The number of lines to spread the labels over to make room or 
											//   tighter labels.

						step:				attrs.yaxisLabelsStep == undefined ? undefined : eval(attrs.yaxisLabelsStep),
											// Optional. To show only every n'th label on the axis, set the step to n. Setting the step to 2 
											//   shows every other label.
						
						style:				attrs.yaxisLabelsStyle == undefined ? {} : eval('(' + attrs.yaxisLabelsStyle + ')'),
											// Optional. CSS styles for the label. 
						
						useHTML:			attrs.yaxisLabelsUseHtml == undefined ? false : eval(attrs.yaxisLabelsUseHtml),
											// Optional. Whether to use HTML to render the labels. Defaults to false.						
					},
					
					lineColor:				attrs.yaxisLineColor == undefined ? "#C0D0E0" : attrs.yaxisLineColor,
											// Optional. The color of the line marking the axis itself. Defaults to #C0D0E0.
					
					lineWidth:				attrs.yaxisLineWidth == undefined ? 1 : eval(attrs.yaxisLineWidth),
											// Optional. The width of the line marking the axis itself. Defaults to 1.
					
					max:					attrs.yaxisMax == undefined ? undefined : eval(attrs.yaxisMax),
											// Optional. The maximum value of the axis. If null, the max value is automatically calculated. If the endOnTick 
											//   option is true, the max value might be rounded up. The actual maximum value is also influenced by 
											//   chart.alignTicks.
					
					maxPadding:				attrs.yaxisMaxPadding == undefined ? 0.05 : eval(attrs.yaxisMaxPadding),
											// Optional. Padding of the max value relative to the length of the axis. 
					
					min:					attrs.yaxisMin == undefined ? undefined : eval(attrs.yaxisMin),
											// Optional. The minimum value of the axis. If null the min value is automatically calculated. If the startOnTick 
											//   option is true, the min value might be rounded down.
					
					minPadding:				attrs.yaxisMinPadding == undefined ? 0.05 : eval(attrs.yaxisMinPadding),
											// Optional. Padding of the min value relative to the length of the axis. 
					
					minRange:				attrs.yaxisMinRange == undefined ? undefined : eval(attrs.yaxisMinRange),
											// Optional. The minimum range to display on this axis. 
					
					minTickInterval:		attrs.yaxisMinTickInterval == undefined ? undefined : eval(attrs.yaxisMinTickInterval),
											// Optional. The minimum tick interval allowed in axis values. Defaults to the closest distance between two points 
											//   on the axis.
					
					minorGridLineColor:		attrs.yaxisMinorGridLineColor,
											// Optional. Color of the minor, secondary grid lines. Defaults to #E0E0E0.
					
					minorGridLineDashStyle:	attrs.yaxisMinorGridLineDashStyle,
											// Optional. The dash or dot style of the minor grid lines. Defaults to Solid.
					
					minorGridLineWidth:		attrs.yaxisMinorGridLineWidth == undefined ? undefined : eval(attrs.yaxisMinorGridLineWidth),
											// Optional. Width of the minor, secondary grid lines. Defaults to 1.
					
					minorTickColor:			attrs.yaxisMinorTickColor,
											// Optional. Color for the minor tick marks. Defaults to #A0A0A0.
					
					minorTickInterval:		attrs.yaxisMinorTickInterval,
											// Optional. Tick interval in scale units for the minor ticks. On a linear axis, if "auto", the minor tick interval 
											//   is calculated as a fifth of the tickInterval. If null, minor ticks are not shown.
					
					minorTickLength:		attrs.yaxisMinorTickLength == undefined ? undefined : eval(attrs.yaxisMinorTickLength),
											// Optional. The pixel length of the minor tick marks. Defaults to 2.
					
					minorTickPosition:		attrs.yaxisMinorTickPosition,
											// Optional. The position of the minor tick marks relative to the axis line. Can be one of inside and outside. 
											//   Defaults to outside.
					
					minorTickWidth:			attrs.yaxisMinorTickWidth == undefined ? undefined : eval(attrs.yaxisMinorTickWidth),
											// Optional. The pixel width of the minor tick mark. Defaults to 0.
					
					offset:					attrs.yaxisOffset == undefined ? 0 : eval(attrs.yaxisOffset),
											// Optional. The distance in pixels from the plot area to the axis line. A positive offset moves the axis with it's 
											//   line, labels and ticks away from the plot area. This is typically used when two or more axes are displayed on 
											//   the same side of the plot. Defaults to 0.
					
					opposite:				attrs.yaxisOpposite == undefined ? false : eval(attrs.yaxisOpposite),
											// Optional. Whether to display the axis on the opposite side of the normal. The normal is on the left side for 
											//   vertical axes and bottom for horizontal, so the opposite sides will be right and top respectively. This is 
											//   typically used with dual or multiple axes. Defaults to false.
					
					plotBands:				attrs.yaxisPlotBands == undefined ? undefined : eval(attrs.yaxisPlotBands),
											// Optional. Array of objects defining plot bands on the Y axis. The object is defined with the following fields:
											//   borderColor, borderWidth, color, events, from, id, innerRadius, label, outerRadius, thickness, to, zIndex.
					
					reversed:				attrs.yaxisReversed == undefined ? undefined : eval(attrs.yaxisReversed),
											// Optional. Whether to reverse the axis so that the highest number is closest to the origin. Defaults to false.
					
					showEmpty:				attrs.yaxisShowEmpty == undefined ? true : eval(attrs.yaxisShowEmpty),
											// Optional. Whether to show the axis line and title when the axis has no data. Defaults to true.
					
					showFirstLabel:			attrs.yaxisShowFirstLabel == undefined ? true : eval(attrs.yaxisShowFirstLabel),
											// Optional. Whether to show the first tick label. Defaults to true.
					
					showLastLabel:			attrs.yaxisShowLastLabel == undefined ? true : eval(attrs.yaxisShowLastLabel),
											// Optional. Whether to show the last tick label. Defaults to true.
					
					startOfWeek:			attrs.yaxisStartOfWeek == undefined ? 1 : eval(attrs.yaxisStartOfWeek),
											// Optional. For datetime axes, this decides where to put the tick between weeks. 0 = Sunday, 1 = Monday. Defaults to 1.
					
					startOnTick:			attrs.yaxisStartOnTick == undefined ? false : eval(attrs.yaxisStartOnTick),
											// Optional. Whether to force the axis to start on a tick. Use this option with the minPadding option to control the axis 
											//   start. Defaults to false.
					
					tickAmount:				attrs.yaxisTickAmount == undefined ? undefined : eval(attrs.yaxisTickAmount),
											// Optional. The amount of ticks to draw on the axis. This opens up for aligning the ticks of multiple charts or panes 
											//   within a chart. This option overrides the tickPixelInterval option.
					
					tickColor:				attrs.yaxisTickColor == undefined ? '#C0D0E0' : attrs.yaxisTickColor,
											// Optional. Color for the main tick marks. Defaults to #C0D0E0.
					
					tickInterval:			attrs.yaxisTickInterval == undefined ? undefined : eval(attrs.yaxisTickInterval),
											// Optional. The interval of the tick marks in axis units. 
					
					tickLength:				attrs.yaxisTickLength == undefined ? 10 : eval(attrs.yaxisTickLength),
											// Optional. The pixel length of the main tick marks. Defaults to 10.
					
					tickPixelInterval:		attrs.yaxisTickPixelInterval == undefined ? 72 : eval(attrs.yaxisTickPixelInterval),
											// Optional. If tickInterval is null this option sets the approximate pixel interval of the tick marks. Not applicable 
											//   to categorized axis. Defaults to 72 for the Y axis and 100 for	the X axis.
					
					tickPosition:			attrs.yaxisTickPosition,
											// Optional. The position of the major tick marks relative to the axis line. Can be one of inside and outside. 
											//   Defaults to outside.
					
					tickWidth:				attrs.yaxisTickWidth == undefined ? 1 : eval(attrs.yaxisTickWidth),
											// Optional. The pixel width of the major tick marks. Defaults to 1.
					
					tickmarkPlacement:		attrs.yaxisTickmarkPlacement == undefined ? "between" : attrs.yaxisTickmarkPlacement,
											// Optional. For categorized axes only. If on the tick mark is placed in the center of the category, if between the tick 
											//   mark is placed between categories. The default is between if the tickInterval is 1, else on. Defaults to null.
										
					title: {
						align:				attrs.yaxisTitleAlign,
											// Optional. Alignment of the title relative to the axis values. Possible values are "low", "middle" or "high". 
											//   Defaults to middle.

						margin:				attrs.yaxisTitleMargin == undefined ? null : eval(attrs.yaxisTitleMargin),
											// Optional. The pixel distance between the axis labels or line and the title. Defaults to 0 for horizontal axes, 
											//   10 for vertical

						offset:				attrs.yaxisTitleOffset == undefined ? null : eval(attrs.yaxisTitleOffset),
											// Optional. The distance of the axis title from the axis line. By default, this distance is computed from the 
											//   offset width of the labels, the labels' distance from the axis and the title's margin. However when the 
											//   offset option is set, it overrides all this.
											
						rotation:			attrs.yaxisTitleRotation == undefined ? 0 : eval(attrs.yaxisTitleRotation),
											// Optional. The rotation of the text in degrees. 0 is horizontal, 270 is vertical reading from bottom to top. 
											//   Defaults to 0.
											
						style:				attrs.yaxisTitleStyle == undefined ? {} : eval('(' + attrs.yaxisTitleStyle + ')'),
											// Optional. CSS styles for the title. 
											
						text: 				attrs.yaxisTitleText,
											// Optional. The actual text of the axis title. It can contain basic HTML text markup like <b>, <i> and 
											//   spans with style.
					},
					
					type:					attrs.yaxisType,
											// Optional. The type of axis. Can be one of "linear", "logarithmic", "datetime" or "category". 
					
					units:					attrs.yaxisUnits == undefined ? undefined : eval(attrs.yaxisUnits),
											// Optional. Datetime axis only. An array determining what time intervals the ticks are allowed to fall on. Each array 
											//   item is an array where the first value is the time unit and the second value another array of allowed multiples. 
				},				
			};

			// Tooltip doesn't like undefined settings
			if(attrs.tooltipHeaderFormat != undefined)
				scope.chartOptions.tooltip.headerFormat = attrs.tooltipHeaderFormat;
			if(attrs.tooltipPointFormat != undefined)
				scope.chartOptions.tooltip.pointFormat = attrs.tooltipPointFormat;
				
			// Add chart events
			scope.chartOptions.chart.events = {
				click:	function(event) {
					$('.context-menu').hide();
				}
			};
		
			// Plot options by chart type
			if(attrs.chartType != undefined) {
				if(attrs.chartType == "area") {
					scope.chartOptions.plotOptions.area = {
						marker: {
							enabled:		attrs.areaMarkerEnabled == undefined ? true : eval(attrs.areaMarkerEnabled),
											// Optional. Enable or disabled the markers. Defaults to true.
						},
						dataLabels: {
							enabled: 		attrs.areaDataLabelsEnabled == undefined ? false : eval(attrs.areaDataLabelsEnabled),
											// Optional. Enable or disable the data labels. Defaults to false.
											
							format: 		attrs.areaDataLabelsFormat,
											// Optional. A format string for the data label. Defaults to {y}.
											
							style: 			attrs.areaDataLabelsStyle == undefined ? {} : eval('(' + attrs.areaDataLabelsStyle + ')'),
											// Optional. Styles for the label. 
						},
						stacking: 			attrs.areaStacking,
											// Optional. Whether to stack the values of each series on top of each other. Possible 
											//   values are null to disable, "normal" to stack by value or "percent".
					};
				}
				else if(attrs.chartType == "areaspline") {
					scope.chartOptions.plotOptions.areaspline = {
						dataLabels: {
							enabled: 		attrs.areasplineDataLabelsEnabled == undefined ? false : eval(attrs.areasplineDataLabelsEnabled),
											// Optional. Enable or disable the data labels. Defaults to false.
											
							format: 		attrs.areasplineDataLabelsFormat,
											// Optional. A format string for the data label. Defaults to {y}.
											
							style: 			attrs.areasplineDataLabelsStyle == undefined ? {} : eval('(' + attrs.areasplineDataLabelsStyle + ')'),
											// Optional. Styles for the label. 
						},
						stacking: 			attrs.areasplineStacking,
											// Optional. Whether to stack the values of each series on top of each other. Possible 
											//   values are null to disable, "normal" to stack by value or "percent".
					};
				}
				else if(attrs.chartType == "arearange") {
					scope.chartOptions.plotOptions.arearange = {
						dataLabels: {
							enabled: 		attrs.areaDataLabelsEnabled == undefined ? false : eval(attrs.areaDataLabelsEnabled),
											// Optional. Enable or disable the data labels. Defaults to false.
											
							format: 		attrs.areaDataLabelsFormat,
											// Optional. A format string for the data label. Defaults to {y}.
											
							style: 			attrs.areaDataLabelsStyle == undefined ? {} : eval('(' + attrs.areaDataLabelsStyle + ')'),
											// Optional. Styles for the label. 
						},
					};
				}
				else if(attrs.chartType == "bar") {
					scope.chartOptions.plotOptions.bar = {
						dataLabels: {
							enabled: 		attrs.barDataLabelsEnabled == undefined ? false : eval(attrs.barDataLabelsEnabled),
											// Optional. Enable or disable the data labels. Defaults to false.
											
							format: 		attrs.barDataLabelsFormat,
											// Optional. A format string for the data label. Defaults to {y}.
											
							style: 			attrs.barDataLabelsStyle == undefined ? {} : eval('(' + attrs.barDataLabelsStyle + ')'),
											// Optional. Styles for the label. 
						},
						stacking: 			attrs.barStacking,
											// Optional. Whether to stack the values of each series on top of each other. Possible 
											//   values are null to disable, "normal" to stack by value or "percent".
					};
				}
				else if(attrs.chartType == "bubble") {
					scope.chartOptions.plotOptions.bubble = {
						dataLabels: {
							enabled: 		attrs.bubbleDataLabelsEnabled == undefined ? false : eval(attrs.bubbleDataLabelsEnabled),
											// Optional. Enable or disable the data labels. Defaults to false.
											
							format: 		attrs.bubbleDataLabelsFormat,
											// Optional. A format string for the data label. Defaults to {y}.
											
							style: 			attrs.bubbleDataLabelsStyle == undefined ? {} : eval('(' + attrs.bubbleDataLabelsStyle + ')'),
											// Optional. Styles for the label. 
						},
					};
				}
				else if(attrs.chartType == "candlestick") {
					scope.chartOptions.plotOptions.candlestick = {
						dataLabels: {
							enabled: 		attrs.candlestickDataLabelsEnabled == undefined ? false : eval(attrs.candlestickDataLabelsEnabled),
											// Optional. Enable or disable the data labels. Defaults to false.
											
							format: 		attrs.candlestickDataLabelsFormat,
											// Optional. A format string for the data label. Defaults to {y}.
											
							style: 			attrs.candlestickDataLabelsStyle == undefined ? {} : eval('(' + attrs.candlestickDataLabelsStyle + ')'),
											// Optional. Styles for the label. 
						},
					};
				}
				else if(attrs.chartType == "column") {
					scope.chartOptions.plotOptions.column = {
						dataLabels: {
							enabled: 		attrs.columnDataLabelsEnabled == undefined ? false : eval(attrs.columnDataLabelsEnabled),
											// Optional. Enable or disable the data labels. Defaults to false.
											
							format: 		attrs.columnDataLabelsFormat,
											// Optional. A format string for the data label. Defaults to {y}.
											
							style: 			attrs.columnDataLabelsStyle == undefined ? {} : eval('(' + attrs.columnDataLabelsStyle + ')'),
											// Optional. Styles for the label. 
						},
						stacking: 			attrs.columnStacking,
											// Optional. Whether to stack the values of each series on top of each other. Possible 
											//   values are null to disable, "normal" to stack by value or "percent".
					};
				}
				else if(attrs.chartType == "gauge") {
					scope.chartOptions.plotOptions.gauge = {
						dataLabels: {
							enabled: 		attrs.gaugeDataLabelsEnabled == undefined ? false : eval(attrs.gaugeDataLabelsEnabled),
											// Optional. Enable or disable the data labels. Defaults to false.

							format: 		attrs.gaugeDataLabelsFormat,
											// Optional. A format string for the data label. Defaults to {y}.
											
							style: 			attrs.gaugeDataLabelsStyle == undefined ? {} : eval('(' + attrs.gaugeDataLabelsStyle + ')'),
											// Optional. Styles for the label. 
						}
					};
				}
				else if(attrs.chartType == "line") {
					scope.chartOptions.plotOptions.line = {
						dataLabels: {
							enabled: 		attrs.lineDataLabelsEnabled == undefined ? false : eval(attrs.lineDataLabelsEnabled),
											// Optional. Enable or disable the data labels. Defaults to false.
											
							format: 		attrs.lineDataLabelsFormat,
											// Optional. A format string for the data label. Defaults to {y}.
											
							style: 			attrs.lineDataLabelsStyle == undefined ? {} : eval('(' + attrs.lineDataLabelsStyle + ')'),
											// Optional. Styles for the label. 
						},
						stacking: 			attrs.lineStacking == undefined ? undefined : eval(attrs.lineStacking),
											// Optional. Whether to stack the values of each series on top of each other. Possible 
											//   values are null to disable, "normal" to stack by value or "percent".
					};
				}
				else if(attrs.chartType == "heatmap") {
					scope.chartOptions.xAxis.categories = scope.categories.xaxis;
					scope.chartOptions.yAxis.categories = scope.categories.yaxis;
					scope.chartOptions.plotOptions.heatmap = {
						borderColor: 		attrs.heatmapBorderColor == undefined ? undefined : attrs.heatmapBorderColor,
						dataLabels: {
							enabled: 		attrs.heatmapDataLabelsEnabled == undefined ? false : eval(attrs.splineDataLabelsEnabled),
											// Optional. Enable or disable the data labels. Defaults to false.
											
							format: 		attrs.heatmapDataLabelsFormat,
											// Optional. A format string for the data label. Defaults to {y}.
											
							style: 			attrs.heatmapDataLabelsStyle == undefined ? {} : eval('(' + attrs.heatmapDataLabelsStyle + ')'),
											// Optional. Styles for the label. 
						},
						states: {
							hover: {
								color:		attrs.heatmapStatesHoverColor == undefined ? undefined : attrs.heatmapStatesHoverColor,
							},
						},
					};
				}
				else if(attrs.chartType == "pie") {
					scope.chartOptions.plotOptions.pie = {
						allowPointSelect:	attrs.pieAllowPointSelect == undefined ? undefined : eval(attrs.pieAllowPointSelect),
											// Optional. Allow this series' points to be selected by clicking on the markers, bars or 
											//   pie slices. Defaults to false.
											
						cursor:				attrs.pieCursor,
											// Optional. You can set the cursor to "pointer" if you have click events attached to the 
											//   series, to signal to the user that the points and lines can be clicked.
						dataLabels: {
							enabled:		attrs.pieDataLabelsEnabled == undefined ? false : eval(attrs.pieDataLabelsEnabled),
											// Optional. Enable or disable the data labels. Defaults to false.
											
							format: 		attrs.pieDataLabelsFormat,
											// Optional. A format string for the data label. Defaults to {y}.
											
							style: 			attrs.pieDataLabelsStyle  == undefined ? {} : eval('(' + attrs.pieDataLabelsStyle + ')'),
											// Optional. Styles for the label. 
						},
						showInLegend:		attrs.pieShowInLegend == undefined ? false : eval(attrs.pieShowInLegend),
											// Optional. Whether to display this particular series or series type in the legend. Defaults to false.
											
						startAngle:			attrs.pieStartAngle == undefined ? undefined : eval(attrs.pieStartAngle),
											// Optional. The start angle of the pie slices in degrees where 0 is top and 90 right. Defaults to 0.
											
						endAngle:			attrs.pieEndAngle == undefined ? undefined : eval(attrs.pieEndAngle),
											// Optional. The end angle of the pie in degrees where 0 is top and 90 is right. Defaults to startAngle 
											//   plus 360. Defaults to null.
					};
				}
				else if(attrs.chartType == "polar") {
					scope.chartOptions.chart.polar = true;
				}
				else if(attrs.chartType == "scatter") {
					scope.chartOptions.plotOptions.scatter = {
						dataLabels: {
							enabled: 		attrs.scatterDataLabelsEnabled == undefined ? false : eval(attrs.scatterDataLabelsEnabled),
											// Optional. Enable or disable the data labels. Defaults to false.
											
							format: 		attrs.scatterDataLabelsFormat,
											// Optional. A format string for the data label. Defaults to {y}.
											
							style: 			attrs.scatterDataLabelsStyle  == undefined ? {} : eval('(' + attrs.scatterDataLabelsStyle + ')'),
											// Optional. Styles for the label. 
						},
						marker: {
							radius:			attrs.scatterMarkerRadius == undefined ? undefined : eval(attrs.scatterMarkerRadius),
											// Optional. The radius of the point marker. Defaults to 4.
						},
					};
				}
				else if(attrs.chartType == "solidgauge") {
					scope.chartOptions.plotOptions.solidgauge = {
						dataLabels: {
							enabled: 		attrs.gaugeDataLabelsEnabled == undefined ? false : eval(attrs.gaugeDataLabelsEnabled),
											// Optional. Enable or disable the data labels. Defaults to false.

							format: 		attrs.gaugeDataLabelsFormat,
											// Optional. A format string for the data label. Defaults to {y}.
											
							style: 			attrs.gaugeDataLabelsStyle == undefined ? {} : eval('(' + attrs.gaugeDataLabelsStyle + ')'),
											// Optional. Styles for the label. 
						}
					};
				}
				else if(attrs.chartType == "spline") {
					scope.chartOptions.plotOptions.spline = {
						dataLabels: {
							enabled: 		attrs.splineDataLabelsEnabled == undefined ? false : eval(attrs.splineDataLabelsEnabled),
											// Optional. Enable or disable the data labels. Defaults to false.
											
							format: 		attrs.splineDataLabelsFormat,
											// Optional. A format string for the data label. Defaults to {y}.
											
							style: 			attrs.splineDataLabelsStyle == undefined ? {} : eval('(' + attrs.splineDataLabelsStyle + ')'),
											// Optional. Styles for the label. 
						},
						stacking: 			attrs.splineStacking == undefined ? undefined : eval(attrs.splineStacking),
											// Optional. Whether to stack the values of each series on top of each other. Possible 
											//   values are null to disable, "normal" to stack by value or "percent".
					};
				}
				else if(attrs.chartType == "treemap") {
					scope.chartOptions.plotOptions.treemap = {
						alternateStartingDirection:	attrs.treemapAlternateStartingDir == undefined ? undefined : eval(attrs.treemapAlternateStartingDir),
											// Optional. Enabling this option will make the treemap alternate the drawing direction between vertical and horizontal. 
											//   The next levels starting direction will always be the opposite of the previous. Defaults to false.
											
						animation:			false,
											// Optional. Sets animation to false
											
						dataLabels: {
							enabled: 		attrs.treemapDataLabelsEnabled == undefined ? false : eval(attrs.treemapDataLabelsEnabled),
											// Optional. Enable or disable the data labels. Defaults to false.
											
							format: 		attrs.treemapDataLabelsFormat,
											// Optional. A format string for the data label. Defaults to {y}.
											
							style: 			attrs.treemapDataLabelsStyle == undefined ? {} : eval('(' + attrs.treemapDataLabelsStyle + ')'),
											// Optional. Styles for the label. 
						},
						layoutAlgorithm:	attrs.treemapLayoutAlgorithm,
											// Optional. This option decides which algorithm is used for setting position and dimensions of the points. Can be one of 
											//   sliceAndDice, stripes, squarified or strip. Defaults to sliceAndDice.
					};
				}
			}
			
			// Read the context menu options from attributes
			scope.chartContextMenu = [];
			$(element).find('chart-context-menu').each(function(index, chartContextMenuElement) {
				var options = {
					"divider":				$(chartContextMenuElement).attr('divider'),
					
					"label":				$(chartContextMenuElement).attr('label'),
					
					"displayCondition":		$(chartContextMenuElement).attr('display-condition'),
					
					"eventId":				$(chartContextMenuElement).attr('event-id'),
				};
				scope.chartContextMenu.push(options);
			});
						
			// Declare the overlay counter
			scope.overlay = 0;

			// Watch for the dataset creation
			scope.$watch('dataset', function() {
				if(scope.dataset != undefined) {
					scope.registerSink();
				}
			});
			
			// Watch the overlay count and show and hide overlay div
			scope.$watch('overlay', function() {
				if(scope.overlay > 0)
					Overlay.show(element);
				else
					Overlay.hide(element);
			});
			
			// Process an object event
			scope.processObject = function(object, operation) {
				for(var idx = 0; idx < scope.chart.series.length(); idx++) {
					var series = scope.chart.series.getByIndex(idx);
					scope.processSeriesObject(series, object, operation);
				}
			};
			
			// Process an object event for a series
			scope.processSeriesObject = function(series, object, operation) {
				var pointId = 'point.' + series.data + "." + object._key;				
				var point = series.getPoint(pointId);

				if(operation == 'delete' && point != undefined) {
					point.remove(false);
				}
				else if(operation == 'insert' || operation == 'update') {
					if(object[series.data] == undefined) return;
					var datum = {
						id: 		pointId,
						events: {
							click: function(e) {
								$('.context-menu').hide();
							},
							contextmenu: function(e) {
								scope.chartMenu(object[series.data], object, series, e);
							}
						}
					};
					scope.updateDatum(datum, series, object, pointId);

					if(point == undefined)
						series.addPoint(datum, false);
					else
						point.update(datum, false);
				}
			};
			
			// Create a data point object
			scope.updateDatum = function(datum, series, object, pointId) {
				datum.name = object[scope.chart.category];
				datum.y = object[series.data];

				// If the xaxis is not a category type, then set the x value
				//   Otherwise it will be set based on the name
				if(scope.chartOptions.xAxis.type != 'category')
					datum.x = object[scope.chart.category];

				// Set the color based on series color or color scale
				if(series.color != undefined)
					datum.color = series.color(object);
				else if(series.colorValue != undefined)
					datum.color = scope.colorScale.color(object[series.colorValue]);

				// Set values based on chart type
				if(scope.chartOptions.chart.type == 'arearange') {
					// Set the high and low values for the point
					datum.high = object[series.high];
					datum.low = object[series.low];
				}
				else if(scope.chartOptions.chart.type == 'bubble') {
					// Set the radius of the bubble
					datum.z = object[series.radius];
				}
				else if(scope.chartOptions.chart.type == 'heatmap') {
					// If the object category is not found in the category array, 
					//   then add it. Oddly, this is required for heatmap and treemap
					//   but Highcharts handles other chart type categories by itself.
					if(scope.categories.xaxis.indexOf(object[scope.chart.category]) == -1) {
						scope.categories.xaxis.push(object[scope.chart.category]);
					}
					if(scope.categories.yaxis.indexOf(object[series.data]) == -1) {
						scope.categories.yaxis.push(object[series.data]);
					}

					// Set the x and y value from the categories
					datum.x = scope.categories.xaxis.indexOf(object[scope.chart.category]);
					datum.y = scope.categories.yaxis.indexOf(object[series.data]);

					// Set the value of the heat to the series value. This will determine 
					//   the color intensity.
					datum.value = object[series.value];
				}
				else if(scope.chartOptions.chart.type == 'treemap') {
					// If the series has a parent defined, setup the parent points
					if(series.parent != undefined) {
						// Loop over the defined parents. First parents are lower in the
						//   hierarchy than later parents.
						for(var pIdx = 0; pIdx < series.parent.length; pIdx++) {
							// Retrieve the parent
							var pArr = scope.parents[pIdx];

							// If parent is not found, create it and add to the parent map
							if(pArr == undefined) {
								pArr = {};
								scope.parents[pIdx] = pArr;
							}

							// Try to find the parent object for the data object. If not found, 
							//   create it and add to the parent.
							if(pArr[object[series.parent[pIdx].data]] == undefined) {
								pArr[object[series.parent[pIdx].data]] = true;
								var pDatum = {
									id:		object[series.parent[pIdx].data],
									name:	object[series.parent[pIdx].data],
									color:	series.parent[pIdx].color,
								}
								if(pIdx < series.parent.length - 1) {
									pDatum.parent = object[series.parent[pIdx + 1].data];
								}
								else {
									pDatum.color = scope.colors[Object.keys(scope.parents[pIdx]).length - 1];
								}
								series.addPoint(pDatum, false);
							}
						}							
					}
					
					// Set the name, unset the x and y, and set the value. This will determine the size of 
					//   the treemap box.
					datum.name = object[series.data];
					datum.x = undefined;
					datum.y = undefined;
					datum.value = object[series.value];
					
					// Set the color value to determine what color the box will be for colored treemaps only
					datum.colorValue = object[series.value];						

					// If there is a parent, set it
					if(series.parent != undefined)
						datum.parent = object[series.parent[0].data];
				}
				else if(scope.chartOptions.chart.type == 'candlestick') {						
					// Unset the x and y, and set the open, close, high, low values.
					datum.x = undefined;
					datum.y = undefined;
					datum.open = object[series.open];
					datum.close = object[series.close];
					datum.high = object[series.high];
					datum.low = object[series.low];
				}
			};
			
			// Create and register a sink
			scope.registerSink = function() {
				var sink = {				
					onSchemaChange:	function(schema) {
					},
					onSnapshotStart: function() {
						scope.snapshot = true;
						scope.overlay = scope.overlay + 1;
						scope.$applyAsync();
					},
					onSnapshotEnd: function() {
						scope.snapshot = undefined;
						scope.overlay = scope.overlay - 1;
						scope.$applyAsync();
					},
					onInsert: function(object) {
						scope.processObject(object, 'insert');
						scope.chart.update();
					},
					onUpdate: function(object) {
						scope.processObject(object, 'update');
						scope.chart.update();
					},
					onDelete: function(object) {
						scope.processObject(object, 'delete');
						scope.chart.update();
					},
					onClear: function() {
						scope.chart.clear();
						scope.chart.update();
					}
				};
				
				// Register the datasink with the dataset
				var datasink = new DataSink(scope.elementName, 'LiveChart', sink);
				scope.dataset.registerSink(datasink);
				
				// Create the chart and call chart ready
				scope.chart = new LiveChart(attrs, scope.chartOptions);
			}
		
			// Bind listener on window resize to reflow chart
			EventListener.windowResize(function() {
				if(scope.chart == null) return;
				setTimeout(function() {
					scope.chart.resize();
				},0);
			});
			
			// Bind listener on component display for tabs, applications, and visibility-toggle divs
			EventListener.componentDisplay(element, function() {
				if(scope.chart == null) return;
				setTimeout(function() {
					scope.chart.resize();
				},0);
			});
		
			// Display the dataset modal
			scope.displayDatasetModal = function() {
				if(scope.options.datasetModal == undefined) return;
				
				// Setup modal options
				var options = {
					template: '<dataset-modal></dataset-modal>',
					inputContext: {
						dataset: scope.dataset,
						display: scope.options.datasetModal == 'R' || scope.options.datasetModal == 'RW',
						editor:  scope.options.datasetModal == 'RW',
					}
				};

				// Show the modal with the options
				var modal = Modal.show(options);
			};

			// Display chart config modal
			scope.chartConfig = function() {
				// TODO chart config
				$log.debug('not implemented yet');
			};
			
			// CHART MENU ACTIONS
			// Display the chart context menu
			scope.chartMenu = function(point, object, series, e) {
				// Add custom chart context menu items
				var customMenuItems = [];
				for(var idx = 0; idx < scope.chartContextMenu.length; idx++) {
					var chartContextMenuItem = scope.chartContextMenu[idx];
					var customMenuItem = {
						self:		this,
						config:		chartContextMenuItem,
						divider:	chartContextMenuItem.divider != undefined ? eval(chartContextMenuItem.divider) : false,
						eventId:	chartContextMenuItem.eventId,
						label:		chartContextMenuItem.label,
						display:	function() {
							var include = true;
							if(this.config.displayCondition != undefined) {								
								include = false;
								try {
									include = eval(this.config.displayCondition);
								}
								catch(err) {
									//$log.debug(err);
								}
							}
							return include;
						},
						action:		function(e) {
							scope.customAction(point, object, series, e);
						}
					};					
					customMenuItems.push(customMenuItem);
				}

				// Add conditional menu items
				var conditionalMenuItems = [];	
				conditionalMenuItems.push({
					menuItemClass:	'copy-point',
					display:		point != undefined
				});
				conditionalMenuItems.push({
					menuItemClass:	'copy-object',
					display:		object != undefined
				});
				
				// Create the menu
				var menu = ContextMenu.show(e, element, ".chart-context-menu", customMenuItems, conditionalMenuItems);			
				
				// Unbind existing click handlers
				$(menu).find('.copy-point').unbind('click');
				$(menu).find('.copy-object').unbind('click');

				// Bind new click handlers
				$(menu).find('.copy-point').click(function(e) {
					scope.copyPoint(point, e);
				});
				
				$(menu).find('.copy-object').click(function(e) {
					scope.copyObject(object, e);
				});
				
				return false;
			};
			
			// Determine if copy is supported
			scope.copySupported = document.queryCommandSupported('copy');
			
			// Copy point value to clipboard
			scope.copyPoint = function(point, e) {
				var menu = $(element).find(".chart-context-menu");
				var copyText = point;
				Clipboard.copy(copyText, menu);				
			};

			// Copy object to clipboard
			scope.copyObject = function(object, e) {
				var mode = 'csv'
				var copyText = ''				
				if(mode == 'csv') {
					for(var idx = 0; idx < Object.keys(object).length; idx++) {
						if(idx > 0) copyText = copyText + ',';
						var k = Object.keys(object)[idx];
						copyText = copyText + object[k];
					}
				}
				else if(mode == 'json') {
					copyText = JSON.stringify(object, null, 2);
				}

				var menu = $(element).find(".chart-context-menu");				
				Clipboard.copy(copyText, menu);				
			};

			// Custom action
			scope.customAction = function(point, object, series, e) {
				var operation = $(e.target).attr('event-id');
				var topic = 'custom:' + scope.elementId;
				scope.$emit(topic, operation, point, object, series);
			};		
		}		
	};	
});

