app.directive('livemap', function($log, $filter, $compile, Overlay, Modal, ContextMenu, EventListener, Selection) {
	return {
		templateUrl: './templates/live-map.html',
		replace: false,
		transclude: true,
		restrict: 'E',
		scope: {
			display:	'=',
			selected:	'='
		},
		controller: function($scope) {
			// Adds an overlay
			this.addOverlay = function() {
				$scope.overlay = $scope.overlay + 1;
			};
			
			// Returns the layers list
			this.getLayers = function() {
				return $scope.map.layers;
			};
			
			// Returns the map
			this.getMap = function() {
				return $scope.map;
			};
			
			// Returns true of components are selectable
			this.isSelectable = function() {
				return $scope.options.selectable;
			};

			// Removes an overlay
			this.removeOverlay = function() {
				$scope.overlay = $scope.overlay - 1;
			};

			// Shows the map menu
			this.showMapMenu = function(object, layer, e) {
				$scope.mapMenu(object, layer, e);
			};
			
			// Select the component
			this.select = function(component) {
				$scope.select(component);
			};
		},
		link: function(scope, element, attrs, ctrl, transclude) {	
			// DEFINE OBJECTS
			{
				function LiveMap(options) {
					var self 		= this;
					
					this.layers 	= [];
					this.center 	= options.center == undefined ? null : L.latLng(options.center[0],options.center[1]);
					this.zoom 		= options.zoom;
					
					// Create the leaflet node
					this.node = L.map($(element).find('.map-container')[0], {
						center: 	this.center,
						zoom: 		options.zoom,
						minZoom: 	options.minZoom,
						maxZoom: 	options.maxZoom
					});

					// Add the tile layer to the node
					L.tileLayer( options.tileLayer, {
						maxZoom : 		options.maxZoom,
						attribution:	options.attribution,
						opacity:		options.opacity
					}).addTo(this.node);
				};
				
				// Resizes the map on change of view
				LiveMap.prototype.resize = function() {
					this.node.invalidateSize();
				};
				
				// Sets the center point of the map
				LiveMap.prototype.setCenter = function(center) {
					this.node.setView(new L.LatLng(center[0], center[1]), this.zoom);
				};
				
				// Sets the zoom level of the map
				LiveMap.prototype.setZoom = function(zoom) {
					this.node.setView(this.center, zoom);
				};
				
				// Add a new layer to the map
				LiveMap.prototype.addLayer = function(layer) {
					this.layers.push(layer);
					this.node.addLayer(layer.node);
				};
			}
			
			// Transclude own scope
			transclude(scope, function (clone, scope) {
				element.append(clone);
			});
			
			// Add flexbox classes to the directive element
			$(element).addClass('flex-fill-vertical-container flex-fill-vertical-remainder');
			$(element).find('.map-container').addClass('flex-fill-vertical-container flex-fill-vertical-remainder');
			
			// Set the element ID to scope if it exists
			if(attrs.id != undefined) scope.elementId = attrs.id;
			
			// Set the name to scope if it exists
			if(attrs.name != undefined) scope.elementName = attrs.name;
			
			// Read the options from attributes
			scope.options = {
				"attribution":		attrs.attribution == undefined || attrs.attribution.length == 0 ?
										attrs.tileLayer == undefined || attrs.tileLayer.length == 0 ? 
											'<a href="http://www.openstreetmap.org/copyright">OpenStreetMap</a>' :
											undefined :
										attrs.attribution,
									// Optional. Attribution for tile layer.
									// Maporama: <a href="http://www.tibco.com/">Tibco GeoAnalytics</a>
									// OSM: &copy; <a href="http://www.openstreetmap.org/copyright">OpenStreetMap</a>
									// MapQuest Open: Tiles Courtesy of <a href="http://www.mapquest.com/">MapQuest</a> &mdash; Map data &copy; <a href="http://www.openstreetmap.org/copyright">OpenStreetMap</a>
									// ArcGIS Aerial: Tiles &copy; Esri &mdash; Source: Esri, i-cubed, USDA, USGS, AEX, GeoEye, Getmapping, Aerogrid, IGN, IGP, UPR-EGP, and the GIS User Community
									
				"center":			attrs.center == undefined || attrs.center.length == 0 ?
										undefined : 
										eval(attrs.center),
									// Optional. Initial center for the map. Must be array object [lat,lon]
									//   For example [52.000000,5.300000]

				"configIcon": 		attrs.configIcon == undefined ? 
										false : 
										eval(attrs.configIcon),
									// Optional. Show or hide config icon. May be true or false.
										
				"filterPanel":		attrs.filterPanel == undefined || attrs.filterPanel.length == 0 ?
										false :
										eval(attrs.filterPanel),
									// Optional. Display the filter panel.
									
				"minZoom":			attrs.minZoom == undefined || attrs.minZoom.length == 0 ?
										undefined : 
										eval(attrs.minZoom),
									// Optional. Minimum zoom level. Must be an integer.

				"maxZoom":			attrs.maxZoom == undefined || attrs.maxZoom.length == 0 ?
										18 : 
										eval(attrs.maxZoom),
									// Optional. Maximum zoom level. Must be an integer.

				"opacity":			attrs.opacity == undefined || attrs.opacity.length == 0 ?
										1.0 : 
										eval(attrs.opacity),
									// Optional. Opacity of tile layer. Must be an integer.
				
				"selectable":		(attrs.selected != undefined && attrs.selected.length > 0),
									// Optional. User can select map component and event will be emitted upwards 
									//   in scope. If this is true, then id must be specified. May be true or false.
					
				"tileLayer":		attrs.tileLayer == undefined || attrs.tileLayer == 0 ?
										'http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png' :
										attrs.tileLayer,
									// Optional. Tile layer to display.
									/* 
										Maporama: http://tilescf.maporama.com/georasterservices/v101/standard_map/{z}/{x}/{y}.png
										OSM: http://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png
										MapQuest Open: http://otile1.mqcdn.com/tiles/1.0.0/map/{z}/{x}/{y}.png
										ArcGIS Aerial: http://server.arcgisonline.com/ArcGIS/rest/services/World_Imagery/MapServer/tile/{z}/{y}/{x}
										Other providers: http://leaflet-extras.github.io/leaflet-providers/preview/
									*/
									
									
				"zoom":				attrs.zoom == undefined || attrs.zoom.length == 0 ?
										null : 
										eval(attrs.zoom),
									// Optional. Initial zoom level. Must be an integer.
			};
			
			// Create the map object
			scope.map = new LiveMap(scope.options);
						
			// Read the context menu options from attributes
			scope.mapContextMenu = [];
			$(element).find('map-context-menu').each(function(index, mapContextMenuElement) {
				var options = {
					"divider":				$(mapContextMenuElement).attr('divider'),
					
					"label":				$(mapContextMenuElement).attr('label'),
					
					"displayCondition":		$(mapContextMenuElement).attr('display-condition'),
					
					"eventId":				$(mapContextMenuElement).attr('event-id'),
				};
				scope.mapContextMenu.push(options);
			});
						
			// Declare the overlay counter
			scope.overlay = 0;

			// Watch the overlay count and show and hide overlay div
			scope.$watch('overlay', function() {
				if(scope.overlay > 0)
					Overlay.show(element);
				else
					Overlay.hide(element);
			});
			
			// Watch the external display to update the map center and zoom
			scope.$watch('display', function() {
				if(scope.display == undefined) return;
				if(scope.map == undefined) return;
				
				if(scope.display.center != undefined)
					scope.map.setCenter(scope.display.center);
				if(scope.display.zoom != undefined)
					scope.map.setZoom(scope.display.zoom);
			});
				
			// Watch the external select to update the table
			scope.$watch('selected', function() {
				var item = undefined;
				if(scope.selected != undefined) {
					for(var idx = 0; idx < scope.map.layers.length; idx++) {
						item = scope.map.layers[idx].getComponent(scope.selected._key);
						if(item != undefined) break;
					}
				}
				item = Selection.watchSelected(scope.selectedComponent, item);
				if(item == false) return;
				if(item == undefined) {
					scope.selectedComponent = undefined;
				}
				else {
					scope.selectedComponent = item;
				}
			});
							
			// Select a component internally
			scope.select = function(component) {				
				var item = Selection.select(scope.options.selectable, scope.selectedComponent, component);
				if(item == false) return;
				if(item == undefined) {
					scope.selectedComponent = undefined;
					scope.selected = undefined;
				}
				else {
					scope.selectedComponent = item;
					scope.selected = item.data;
				}
				scope.$apply();
			};			

			// Bind listener on component display for tabs, applications, and visibility-toggle divs
			EventListener.componentDisplay(element, function() {
				if(scope.map == null) return;
				setTimeout(function() {
					scope.map.resize();
				},0);
			});
		
			// Display the dataset modal
			scope.displayDatasetModal = function() {
				if(scope.options.datasetModal == undefined) return;
				
				// Setup modal options
				var options = {
					template: '<dataset-modal></dataset-modal>',
					inputContext: {
						dataset: scope.dataset,
						display: scope.options.datasetModal == 'R' || scope.options.datasetModal == 'RW',
						editor:  scope.options.datasetModal == 'RW',
					}
				};

				// Show the modal with the options
				var modal = Modal.show(options);
			};

			// Display map config modal
			scope.mapConfig = function() {
				// TODO config icon
				$log.debug('not implemented yet');
			};
			
			// MAP MENU ACTIONS
			// Display the map context menu
			scope.mapMenu = function(extObject, extLayer, e) {
				// Add custom map context menu items
				var customMenuItems = [];
				for(var idx = 0; idx < scope.mapContextMenu.length; idx++) {
					var mapContextMenuItem = scope.mapContextMenu[idx];
					var object = extObject == undefined ? {} : extObject;
					var layer = extLayer == undefined ? {} : extLayer;
					var customMenuItem = {
						self:		this,
						config:		mapContextMenuItem,
						divider:	mapContextMenuItem.divider != undefined ? eval(mapContextMenuItem.divider) : false,
						eventId:	mapContextMenuItem.eventId,
						label:		mapContextMenuItem.label,
						display:	function() {
							var include = true;
							if(this.config.displayCondition != undefined)
								include = eval(this.config.displayCondition);
							return include;
						},
						action:		function(e) {
							scope.customAction(extObject, extLayer, e);
						}
					};					
					customMenuItems.push(customMenuItem);
				}
				
				// Create the menu
				var menu = ContextMenu.show(e, element, ".map-context-menu", customMenuItems);			
				
				// Unbind existing click handlers
				$(menu).find('.copy-cell').unbind('click');
				$(menu).find('.copy-row').unbind('click');

				// Bind new click handlers
				$(menu).find('.copy-cell').click(function(e) {
					scope.copyCell(object, row, columnModel, e);
				});
				
				$(menu).find('.copy-row').click(function(e) {
					scope.copyRow(object, row, columnModel, e);
				});
				
				return false;
			};
			
			// Custom action
			scope.customAction = function(object, layer, e) {
				var operation = $(e.target).attr('event-id');
				var topic = 'custom:' + scope.elementId;
				scope.$emit(topic, operation, object, layer);
			};	

		}
	};	
});

app.directive('livemapLayer', function($log) {
	return {
		replace: false,
		transclude: true,
		restrict: 'E',
		scope: {},
		require: '^livemap',
		controller: function($scope) {
			// When the component visibility changes
			this.onComponentVisibilityChange = function(component) {
				$scope.layer.onComponentVisibilityChange(component);
			};
			
			// Returns the color for a given key
			this.getColor = function(key) {
				if($scope.options.colorScale == undefined) 
					return undefined;
				return $scope.layer.colorScale.color(key);
			};

			// Returns the layer
			this.getLayer = function() {
				return $scope.layer;
			};
			
			// Returns the object title expression
			this.getObjectTitleExpression = function() {
				return $scope.options.objectTitle;
			};

			// Registers a new component factory
			this.registerComponentFactory = function(factory) {
				$scope.factories.push(factory);
			};

			// Selects the specified component
			this.select = function(component) {
				$scope.select(component);
			};
			
			// Shows the map menu
			this.showMapMenu = function(object, layer, e) {
				$scope.mapMenu(object, layer, e);
			};			
		},
		link: function(scope, element, attrs, ctrl, transclude) {		
			// DEFINE OBJECTS
			{
				// Color Scale object
				{
					function ColorScale(scale) {
						this.scale = scale;
						this.lastUsed = -1;
						this.colorMap = {};
					};
					
					ColorScale.prototype.color = function(key) {
						if(this.colorMap[key] != undefined) 
							return this.colorMap[key];
						this.lastUsed = this.lastUsed + 1;
						if(this.lastUsed == this.scale.length) 
							this.lastUsed = 0;
						var color = this.scale[this.lastUsed];
						this.colorMap[key] = color;
						return color;
					};
				}	
				
				// LiveMapLayer object
				{
					function LiveMapLayer(options, dataset) {
						this.dataset	= dataset;
						this.node 		= new L.LayerGroup();
						this.options	= options;
						this.title		= options.title;
						this.visible	= options.visibilityDefault;
						
						this.hideCount  = 0;
						
						this.colorScale = new ColorScale(options.colorScale);						
						this.components = new Collection(['_key']);
					};
					
					// Add a new component to the layer
					LiveMapLayer.prototype.addComponent = function(component) {
						// Set the layer and visible property based on the layer
						component.layer = this;
						component.visible = this.visible;
						
						// If the layer is fully hidden and something new is added,
						//   the increment the hide count since it will be added as hidden
						if(this.visible == false)
							this.hideCount = this.hideCount + 1;
						
						// Add the component to the collection
						this.components.put(component);						
					};
					
					// Called when the component visibility changes
					LiveMapLayer.prototype.onComponentVisibilityChange = function(component) {
						// Adjust hide count appropriately
						if(component.visible == false)
							this.hideCount = this.hideCount + 1;
						else
							this.hideCount = this.hideCount - 1;
						
						// Change the layer visible flag based on hide count
						if(this.hideCount == 0) 
							this.visible = true;
						else
							this.visible = false;
					};
					
					// Return the component with the given id
					LiveMapLayer.prototype.getComponent = function(id) {
						return this.components.get(id);
					};
					
					// Return the component collection
					LiveMapLayer.prototype.getComponents = function() {
						return this.components;
					};
					
					// Remove the component from the layer
					LiveMapLayer.prototype.removeComponent = function(id) {
						return this.components.remove(id);
					};
					
					// Update the visibility of all components
					LiveMapLayer.prototype.updateVisibility = function() {
						for(var idx = 0; idx < this.components.length(); idx++) {
							var component = this.components.getByIndex(idx);
							if(component.visible != this.visible) {
								component.visible = this.visible;
								component.updateVisibility(false);
								if(this.visible == true)
									this.hideCount = this.hideCount - 1;
								else
									this.hideCount = this.hideCount + 1;
							}
						}
					};
				}
			}

			// Define an array of component factories
			scope.factories = [];			

			// Transclude own scope
			transclude(scope, function (clone, scope) {
				element.append(clone);
			});

			// Set the element ID to scope if it exists
			if(attrs.id != undefined) scope.elementId = attrs.id;
			
			// Set the name to scope if it exists
			if(attrs.name != undefined) scope.elementName = attrs.name;
						
			// Define color categories
			scope.color_categories = {
				color_category10:	[
									  '#1f77b4', '#ff7f0e', '#2ca02c', '#d62728', '#9467bd',
									  '#8c564b', '#e377c2', '#7f7f7f', '#bcbd22', '#17becf'
									],
									
				color_category20a:	[
									  '#1f77b4', '#aec7e8', '#ff7f0e', '#ffbb78',
									  '#2ca02c', '#98df8a', '#d62728', '#ff9896',
									  '#9467bd', '#c5b0d5', '#8c564b', '#c49c94',
									  '#e377c2', '#f7b6d2', '#7f7f7f', '#c7c7c7',
									  '#bcbd22', '#dbdb8d', '#17becf', '#9edae5'
									],

				color_category20b:	[
									  '#393b79', '#5254a3', '#6b6ecf', '#9c9ede',
									  '#637939', '#8ca252', '#b5cf6b', '#cedb9c',
									  '#8c6d31', '#bd9e39', '#e7ba52', '#e7cb94',
									  '#843c39', '#ad494a', '#d6616b', '#e7969c',
									  '#7b4173', '#a55194', '#ce6dbd', '#de9ed6'
									],

				color_category20c: 	[
									  '#3182bd', '#6baed6', '#9ecae1', '#c6dbef',
									  '#e6550d', '#fd8d3c', '#fdae6b', '#fdd0a2',
									  '#31a354', '#74c476', '#a1d99b', '#c7e9c0',
									  '#756bb1', '#9e9ac8', '#bcbddc', '#dadaeb',
									  '#636363', '#969696', '#bdbdbd', '#d9d9d9'
									],
			};

			// Read the options from attributes
			scope.options = {
				"colorScale":		attrs.colorScale == undefined || attrs.colorScale.length == 0 ?
										undefined :
										scope.color_categories[attrs.colorScale],
									// Optional. Use a color scale to define fill and stroke color of path elements.
									//   Use one of the configured color scales: color_category10, color_category20a, 
									//   color_category20b, color_category20c
									//   This color value will override any configured color for stroke or fill on path
									//     elements within the layer. It will not set any color on icon markers.
									
				"datasetIcon": 		attrs.datasetIcon == undefined ? 
										false : 
										eval(attrs.datasetIcon),
									// Optional. Show or hide dataset icon field. May be true or false.
										
				"datasetModal":		attrs.datasetModal == "R" || attrs.datasetModal == "RW" ?
										attrs.datasetModal :
										undefined,
									// Optional. Show the dataset modal if set. May be one of:
									//   R -- read-only modal
									//   RW -- read-write modal

				"hideable":			attrs.hideable == undefined || attrs.hideable.length == 0 ?
										true :
										eval(attrs.hideable),
									// Optional. Allow hiding components on the layer.

				"objectTitle":		attrs.objectTitle == undefined || attrs.objectTitle.length == 0 ?
										undefined :
										attrs.objectTitle,
									// Optional. Object title to display in filter box. This is a javascript expression that
									//   can use the object data.
									//   Example: 'Flight num:' + object.flightNum
									
				"title":			attrs.title == undefined || attrs.title.length == 0 ?
										undefined : 
										attrs.title,
									// Optional. Layer title
				
				"visibilityChange":	attrs.visibilityChange == undefined || attrs.visibilityChange.length == 0 ?
										true :
										eval(attrs.visibilityChange),
									// Optional. Allow change of visibility of items on this layer. Default is true.
									
				"visibilityDefault":	attrs.visibilityDefault == undefined || attrs.visibilityDefault.length == 0 ?
										true :
										eval(attrs.visibilityDefault),
									// Optional. Default visibility for components on the layer. Default is true.
			};

			// Read the context menu options from attributes
			scope.mapContextMenu = [];
			$(element).find('map-context-menu').each(function(index, mapContextMenuElement) {
				var options = {
					"label":				$(mapContextMenuElement).attr('label'),
					
					"displayCondition":		$(mapContextMenuElement).attr('display-condition'),
					
					"eventId":				$(mapContextMenuElement).attr('event-id'),
				};
				scope.mapContextMenu.push(options);
			});
			
			// Watch for the dataset creation
			scope.$watch('$parent.$parent["' + attrs.dataset + '"]', function() {
				if(scope.$parent.$parent[attrs.dataset] == undefined) return;
				scope.dataset = scope.$parent.$parent[attrs.dataset];
				scope.registerSink();
			});					

			// Create and register a sink
			scope.registerSink = function() {
				var sink = {				
					onSchemaChange:	function(schema) {
					},
					onSnapshotStart: function() {
						scope.snapshot = true;
						ctrl.addOverlay();
						scope.$applyAsync();
					},
					onSnapshotEnd: function() {
						scope.snapshot = undefined;
						ctrl.removeOverlay();
						scope.$applyAsync();
					},
					onInsert: function(object) {
						for(var idx = 0; idx < scope.factories.length; idx++) {
							var component = scope.factories[idx].onInsert(object);
							component.draw();
						}
						scope.layer.addComponent(component);					
					},
					onUpdate: function(object) {
						var component = scope.layer.getComponent(object._key);
						if(component == undefined) return;
						component.update();
					},
					onDelete: function(object) {
						var component = scope.layer.getComponent(object._key);
						if(component == undefined) return;
						var id = component._key;
						scope.layer.node.removeLayer(component.node);
						scope.layer.removeComponent(id);
					},
					onClear: function() {
						var collection = scope.layer.getComponents();
						while(collection.length() > 0) {
							var component = collection.getByIndex(0);
							if(component == undefined) return;
							var id = component._key;
							scope.layer.node.removeLayer(component.node);							
							scope.layer.removeComponent(id);
						}
					}
				};
				
				// Register the datasink with the dataset
				var datasink = new DataSink(scope.elementName, 'LiveMapLayer', sink);
				scope.dataset.registerSink(datasink);

				// Create the layer and add to map
				scope.layer = new LiveMapLayer(scope.options, scope.dataset);
				ctrl.getMap().addLayer(scope.layer);
			}
		
			// Select a component internally
			scope.select = function(component) {
				ctrl.select(component);
			};

			// Show the map menu
			scope.mapMenu = function(object, layer, e) {
				ctrl.showMapMenu(object, layer, e);
			};
		}
	};	
});

app.directive('livemapCircleMarker', function($log, Map) {
	return {
		replace: false,
		transclude: true,
		restrict: 'E',
		scope: {},
		require: '^livemapLayer',
		link: function(scope, element, attrs, ctrl, transclude) {
			// DEFINE OBJECTS
			{
				// CircleMarkerComponent object
				{
					function CircleMarkerComponent(options, object) {
						this._key 		= object._key;
						this.added		= false;
						this.data 		= object;
						this.node 		= undefined;
						this.popup		= undefined;
						this.selected 	= false;
						this.title 		= undefined;
						this.visible 	= true;

						// Create a popup if specified
						if(options.popup != undefined) {
							this.popup 	= Map.createPopup(scope, options.popup);
						}
											
						// These properties are used for microbatching the updates
						this.microbatch = {
							rtime:		new Date(1, 1, 2000, 12,00,00),
							timeout:	false,
							delta:		0
						}	
					};
/*
					CircleMarkerComponent.prototype.bringToBack = function() { this.node.bringToBack();	};
					CircleMarkerComponent.prototype.bringToFront = function() { this.node.bringToFront(); };
*/					
					// Draw the circle marker
					CircleMarkerComponent.prototype.draw = function() {
						var self = this;
						
						// Create the Leaflet node
						var opts = $.extend({}, scope.options.path.options);
						this.node = L.circleMarker(null, opts);			
						
						// Add click event handler
						this.node.on('click', function(event) {
							ctrl.select(self);
						});

						// Add context event handler
						this.node.on('contextmenu', function(event) {
							ctrl.showMapMenu(self.data, ctrl.getLayer(), event);
						});

						// Update the component
						this.update();
					};

					// Update the circle marker
					CircleMarkerComponent.prototype.update = function() {
						var self = this;

						// Microbatch handler
						self.microbatch.rtime = new Date();
						if(self.microbatch.timeout === false) {
							self.microbatch.timeout = true;
							setTimeout(eventEnd, self.microbatch.delta);
						}					
						function eventEnd() {
							if (new Date() - self.microbatch.rtime < self.microbatch.delta)
								setTimeout(eventEnd, self.microbatch.delta);
							else {
								self.microbatch.timeout = false;
								run();
							}               
						};
						
						function run() {
							var object = self.data;

							// Update the radius 
							if(scope.options.radius.indexOf('{') == -1)
								self.node.setRadius(scope.options.radius);
							else {
								var radius = eval('(' + scope.options.radius + ')');
								var radius = eval('(' + scope.options.radius + ')');											
								for(var idx = 0; idx < Object.keys(radius).length; idx++) {
									var rad = Object.keys(radius)[idx];
									if(radius[rad] == true) {
										self.node.setRadius(rad);
										break;
									}
								}											
							}									
												
							// Update the styles if a path marker
							if(scope.options.path != undefined) {
								var opts = $.extend({}, scope.options.path.options);
								var color = ctrl.getColor(self._key);								
								if(color != undefined) {
									opts.color = color;
									opts.fillColor = color;
								}
								else {
									opts.color = Map.getColor(opts, 'color', object);
									opts.fillColor = Map.getColor(opts, 'fillColor', object);
								}
								self.node.setStyle(opts);
							}
												
							// Update the popup, position, and title
							Map.updatePopup(scope, self);
							Map.updatePosition(scope, self);
							Map.updateTitle(scope, ctrl, self);
						}
					};
					
					// Update the marker visibility
					CircleMarkerComponent.prototype.updateVisibility = function(notifyLayer) {
						if(this.visible == true) {
							this.layer.node.addLayer(this.node);
							this.update();
							if(notifyLayer != false) ctrl.onComponentVisibilityChange(this);
						}
						else if(this.visible == false) {
							this.layer.node.removeLayer(this.node);
							if(notifyLayer != false) ctrl.onComponentVisibilityChange(this);
						}
					};
				}
			}

			// Transclude own scope
			transclude(scope, function (clone, scope) {
				element.append(clone);
			});
			
			// Set the element ID to scope if it exists
			if(attrs.id != undefined) scope.elementId = attrs.id;
			
			// Set the name to scope if it exists
			if(attrs.name != undefined) scope.elementName = attrs.name;
			
			// Read the options from attributes
			scope.options = {
				"disabled":			attrs.disabled == undefined || attrs.disabled.length == 0 ?
										false :
										eval(attrs.disabled), 
									// Optional. Disable the layer.
									
				"position":			attrs.position, 
									// Mandatory. Position for the point. Define as an array with object properties for lat and lon.
									//   Example 1: [object.lat, object.lon]
									//     This will plot the object at the position object.lat and object.lon where object is the
									//     data row, and lat and lon are the properties of that object that point to latitude and 
									//     longitude respectively.

				"circleMarker":		{},
									// Mandatory. Automatically generated for this directive type.
									
				"popup":			$(element).find('livemap-popup').length != 1 ?
										undefined :
										$(element).find('livemap-popup').html(),
									// Optional. Popup HTML content. Defined in a tag structure underneath this element.
									//   Example:
									//     <livemap-popup ng-non-bindable>
									//        <span>
									//		    <h3>Flight: {{::object.flightNumber}}</h3>
									//	     </span>
									//     </livemap-icon-label>
									// 
									//   See all notes above as for iconLabelContent. Same rules apply here.
									
				"radius":			attrs.radius,
									// Mandatory. Radius of the circle marker in pixels.
									// May be a literal value:
									//	 Example 1: 10
									//     Radius will always be 10px
									// Or a conditional object. Keys in the object define the radius. Values define boolean
									//   expressions. The first expression that evaluates to true will be assigned as radius.
									//   Example 2: {'10': object.status == 'Approach', '20': object.status == 'Circling', '30': true}
									//     This will give 10px if object has status Approach, 20px if object has status Circling, 
									//     30px otherwise.									
			};

			// Get path options
			scope.options.path = {
				options:	Map.getPathOptions(attrs)
			}
						
			// Register a component factory on the layer
			ctrl.registerComponentFactory({
				onInsert: function(object){
					return createComponentObject(object);
				},
			});
			
			// Create a new map component for the given object
			var createComponentObject = function(object) {
				var component = new CircleMarkerComponent(scope.options, object);
				return component;
			};
						
		}
	};	
});

app.directive('livemapPolyline', function($log, SimpleDateFormat, NumberFormat, Overlay, Map) {
	return {
		replace: false,
		transclude: true,
		restrict: 'E',
		scope: {},
		require: '^livemapLayer',
		link: function(scope, element, attrs, ctrl, transclude) {
			// DEFINE OBJECTS
			{
				// PolylineComponent object
				{
					function PolylineComponent(options, object) {
						this._key 			= object._key;
						this.added			= false;
						this.data 			= object;
						this.node 			= undefined;
						this.polylineMode	= options.polylineMode;
						this.points			= options.points;
						this.popup			= undefined;
						this.selected 		= false;
						this.title 			= undefined;
						this.visible 		= true;

						// Create a popup if specified
						if(options.popup != undefined) {
							this.popup 	= Map.createPopup(scope, options.popup);
						}
											
						// These properties are used for microbatching the updates
						this.microbatch = {
							rtime:		new Date(1, 1, 2000, 12,00,00),
							timeout:	false,
							delta:		0
						}
					};
/*
					PolylineComponent.prototype.bringToBack = function() { this.node.bringToBack();	};
					PolylineComponent.prototype.bringToFront = function() { this.node.bringToFront(); };
*/					
					// Draw the polyline
					PolylineComponent.prototype.draw = function() {
						var self = this;
						
						// Create the Leaflet node
						var opts = $.extend({}, scope.options.polyline.options, scope.options.path.options);
						this.node = L.polyline([], opts);			
						
						// Add click event handler
						this.node.on('click', function(event) {
							ctrl.select(self);
						});

						// Add context event handler
						this.node.on('contextmenu', function(event) {
							ctrl.showMapMenu(self.data, ctrl.getLayer(), event);
						});

						// Update the component
						this.update();
					};

					// Update the polyline
					PolylineComponent.prototype.update = function() {
						var self = this;

						// Microbatch handler
						self.microbatch.rtime = new Date();
						if(self.microbatch.timeout === false) {
							self.microbatch.timeout = true;
							setTimeout(eventEnd, self.microbatch.delta);
						}					
						function eventEnd() {
							if (new Date() - self.microbatch.rtime < self.microbatch.delta)
								setTimeout(eventEnd, self.microbatch.delta);
							else {
								self.microbatch.timeout = false;
								run();
							}               
						};
						
						function run() {
							var object = self.data;

							// Update the styles if a path marker
							if(scope.options.path != undefined) {
								var opts = $.extend({}, scope.options.path.options);
								var color = ctrl.getColor(self._key);
								if(color != undefined) {
									opts.color = color;
									opts.fillColor = color;
								}
								else {
									opts.color = Map.getColor(opts, 'color', object);
									opts.fillColor = Map.getColor(opts, 'fillColor', object);
								}
								self.node.setStyle(opts);
							}
												
							// Update the popup, position, and title
							Map.updatePopup(scope, self);
							self.updatePosition();
							Map.updateTitle(scope, ctrl, self);
						}
					};
					
					// Update the polyline position
					PolylineComponent.prototype.updatePosition = function() {
						var waypointArr = this.getWaypointArray();
						if(waypointArr.length > 0) {
							this.node.setLatLngs(waypointArr);
							if(this.added == false) {
								if(this.visible == true) {
									this.layer.node.addLayer(this.node);
								}
								this.added = true;
							}
						}
						else if(this.added == true) {
							this.layer.node.removeLayer(this.node);
							this.added = false;
						}
						
					};
					
					// Update the polyline visibility
					PolylineComponent.prototype.updateVisibility = function(notifyLayer) {
						if(this.visible == true) {
							this.layer.node.addLayer(this.node);
							this.update();
							if(notifyLayer != false) ctrl.onComponentVisibilityChange(this);
						}
						else if(this.visible == false) {
							this.layer.node.removeLayer(this.node);
							if(notifyLayer != false) ctrl.onComponentVisibilityChange(this);
						}
					};
					
					// Returns the waypoint array from the component data
					PolylineComponent.prototype.getWaypointArray = function() {
						var latLng = [];
						if(this.polylineMode == 'single') {
							var object = this.data;
							var position = eval(scope.options.position);
							for(var idx = 0; idx < position.length; idx++) {
								var thisPosition = position[idx];
								if(thisPosition[0]!= null && thisPosition[1] != null)
									latLng.push(L.latLng(thisPosition));		
							}
						}
						else if(this.polylineMode == 'grouped') {
							var points = this.data[this.points];
							
							// If there is an order by specified, order the array now
							if(this.data._orderBy != undefined) {
								self = this;
								points = points.sort(function(a,b) {
									for(var sortIdx = 0; sortIdx < self.data._orderBy.length; sortIdx++) {
										var compare = function(a, b, dir, field) {
											return dir * (a[field] == b[field] ? 0 : a[field] < b[field] ? 1 : -1);
										};
										var thisSort = self.data._orderBy[sortIdx];
										var dir = thisSort[1] == 'desc' ? 1 : -1;
										var val = compare(a, b, dir, thisSort[0]);
										if(val != 0) return val;
									}
								});
							}
							
							// Loop over the points and push into the array
							for(var idx = 0; idx < points.length; idx++) {
								var object = points[idx];
								var position = eval(scope.options.position);
								if(position[0] != null && position[1] != null) {
									latLng.push(L.latLng(position));
								}
							}
						}

						return latLng;
					};
					
				}
			}

			// Transclude own scope
			transclude(scope, function (clone, scope) {
				element.append(clone);
			});
			
			// Set the element ID to scope if it exists
			if(attrs.id != undefined) scope.elementId = attrs.id;
			
			// Set the name to scope if it exists
			if(attrs.name != undefined) scope.elementName = attrs.name;
			
			// Read the options from attributes
			scope.options = {
				"disabled":			attrs.disabled == undefined || attrs.disabled.length == 0 ?
										false :
										eval(attrs.disabled), 
									// Optional. Disable the layer.
						
				"points":			attrs.points,
									// Mandatory for grouped. Property on the object that forms an array of points. 
									
				"position":			attrs.position, 
									// Mandatory. Position for the point. Define as an array with object properties for lat and lon.
									//   Example 1: [object.lat, object.lon]
									//     Sets the vertex at the position object.lat and object.lon where object is the
									//     element in the 'points' array, and lat and lon are the properties of that points object 
									//     are latitude and longitude respectively. The points attribute is mandatory for this mode.
									//   Example 2: [[object.startLat, object.startLon],[object.endLat,object.endLon]]
									//     This applies only to polylineMode of single where each object defines a full polyline
									//     Sets the starting vertex at the position object.startLat and object.startLon and the 
									//     ending vertex at the position object.endLat and object.endLon, where object is the
									//     data row, and startLat, startLon, endLat, and endLon are all properties of that object 
									//     that point to starting latitude and longitude and ending latitude and longitude respectively.
									//     The points attribute is not required for this mode.

				"polyline":			{},
									// Mandatory. Automatically generated for this directive type.
									
				"polylineMode":		attrs.polylineMode == undefined ? 'grouped' : attrs.polylineMode,
									// Optional. Polyline vertex mode. Valid values are: single or grouped. If omitted
									//   then grouped polyline will be used
									//   single 	- one object record is a single straight line, defining both start and end points
									//   grouped 	- one object record has a single attribute containing an array of vertexes 
									
				"popup":			$(element).find('livemap-popup').length != 1 ?
										undefined :
										$(element).find('livemap-popup').html(),
									// Optional. Popup HTML content. Defined in a tag structure underneath this element.
									//   Example:
									//     <livemap-popup ng-non-bindable>
									//        <span>
									//		    <h3>Flight: {{::object.flightNumber}}</h3>
									//	     </span>
									//     </livemap-icon-label>
									// 
									//   See all notes above as for iconLabelContent. Same rules apply here.
			};

			// Get path options
			scope.options.path = {
				options:	Map.getPathOptions(attrs)
			}
						
			// Add polyline options
			scope.options.polyline.options = {
				"smoothFactor":		attrs.smoothFactor == undefined || attrs.smoothFactor.length == 0 ?
										1.0 :
										eval(attrs.smoothFactor),
									// Optional. How much to simplify the polyline at each zoom level. Greater means better performance
									//   less means more accurate. Must be a number.
				
				"noClip":			attrs.noClip == undefined || attrs.noClip.length == 0 ?
										false :
										eval(attrs.noClip),
									// Optional. Disabled polyline clipping. Must be true or false.
			};
			
			// Register a component factory on the layer
			ctrl.registerComponentFactory({
				onInsert: function(object){
					return createComponentObject(object);
				},
			});
			
			// Create a new map component for the given object
			var createComponentObject = function(object) {
				var component = new PolylineComponent(scope.options, object);
				return component;
			};
						
		}
	};	
});

app.directive('livemapIconMarker', function($log, $compile, Map) {
	return {
		replace: false,
		transclude: true,
		restrict: 'E',
		scope: {},
		require: '^livemapLayer',
		link: function(scope, element, attrs, ctrl, transclude) {
			// DEFINE OBJECTS
			{
				// IconMarkerComponent object
				{
					function IconMarkerComponent(content, object, idx) {
						this._key 				= object._key;
						this.added				= false;
						this.data 				= object;
						this.node 				= undefined;
						this.popup				= undefined;
						this.selected 			= false;
						this.title 				= undefined;
						this.visible 			= true;
						this.iconOpts			= undefined;
						this.hasComplexContent	= content != undefined;
						
						// Create a popup if specified
						if(scope.options.popup != undefined) {
							this.popup = Map.createPopup(scope, scope.options.popup);
						}
						
						// Create the icon options object if specified
						if(scope.options.icon != undefined) {
							this.iconOpts = $.extend({}, scope.options.icon.options);
							if(scope.options.icon.url.indexOf('{') == -1)
								this.iconOpts.iconUrl = scope.options.icon.url;
							else {
								var i = eval('(' + scope.options.icon.url + ')');											
								for(var idx = 0; idx < Object.keys(i).length; idx++) {
									var url = Object.keys(i)[idx];
									if(i[url] == true) {
										this.iconOpts.iconUrl = url;
										break;
									}
								}
							}
							
							// Create the icon
							var Icon = L.Icon.Label.extend({
								options: this.iconOpts
							});				
							this.icon = new Icon({});						
							this.icon.options.labelText = '';

							// If the icon has complex content, setup the compile function and scope
							if(this.hasComplexContent) {
								this.contentScope = scope.$new(true);
								this.contentScope.Math = window.Math;
								this.contentScope.formatDateTime = function(dateStr, format) {
									return SimpleDateFormat.format(dateStr, format);
								};
								this.contentScope.formatNumber = function(numStr, format) {
									return NumberFormat.format(numStr, format);
								};
								this.content = $(document.createElement('div'));
								this.content.empty().html(content);
								$compile(this.content.contents())(this.contentScope);								
							}
						}
						
						// These properties are used for microbatching the updates
						this.microbatch = {
							rtime:		new Date(1, 1, 2000, 12,00,00),
							timeout:	false,
							delta:		0
						}
												
					};
/*
					IconMarkerComponent.prototype.bringToBack = function() { };
					IconMarkerComponent.prototype.bringToFront = function() { };
*/					
					// Draw the marker
					IconMarkerComponent.prototype.draw = function() {
						var self = this;
						
						// Create the node
						var opts = $.extend({}, scope.options.marker.options);
						this.node = L.marker([0,0], opts); // Position will be updated later
						
						// Add click event handler
						this.node.on('click', function(event) {
							ctrl.select(self);
						});

						// Add context event handler
						this.node.on('contextmenu', function(event) {
							ctrl.showMapMenu(self.data, ctrl.getLayer(), event);
						});

						// Set the icon to the node if specified
						if(this.icon != undefined) {
							this.node.setIcon(this.icon);
							
							// This is stupid but leaflet appends nodes asynchronously
							//   so need to do this
							var r = function() {
								setTimeout(function() {
									if(self.node._icon == undefined)
										r();
									else
										$(self.node._icon).find('.leaflet-marker-iconlabel').append(self.content);
								},0);
							}
							r();
						}
					
						// Update the component
						this.update();							
					};
					
					// Update the marker
					IconMarkerComponent.prototype.update = function() {
						var self = this;

						// Microbatch handler
						self.microbatch.rtime = new Date();
						if(self.microbatch.timeout === false) {
							self.microbatch.timeout = true;
							setTimeout(eventEnd, self.microbatch.delta);
						}					
						function eventEnd() {
							if (new Date() - self.microbatch.rtime < self.microbatch.delta)
								setTimeout(eventEnd, self.microbatch.delta);
							else {
								self.microbatch.timeout = false;
								run();
							}               
						};
						
						function run() {
							// Create the icon if specified
							if(scope.options.icon != undefined) {
								// Update the icon url
								if(scope.options.icon.url.indexOf('{') > -1) {
									var object = self.data;
									var i = eval('(' + scope.options.icon.url + ')');											
									for(var idx = 0; idx < Object.keys(i).length; idx++) {
										var url = Object.keys(i)[idx];
										if(i[url] == true) {
											self.iconOpts.iconUrl = url;
											break;
										}
									}
								}

								if(self.hasComplexContent == false) {
									self.node.setIcon(L.icon(self.iconOpts));																
								}
								else {
									$(self.node._icon).find('.leaflet-marker-icon').attr('src', self.iconOpts.iconUrl);
									self.contentScope.object = self.data;
									self.contentScope.component = self;
								}
							}

							// Update the popup
							Map.updatePopup(scope, self);

							Map.updatePosition(scope, self);
							Map.updateTitle(scope, ctrl, self);
							
							// Update z-index based on whether the marker is selected
							if(self.selected == true)
								self.node.setZIndexOffset(10000);
							else
								self.node.setZIndexOffset(scope.options.marker.options.zIndexOffset);
							
							// Apply scope
							scope.$apply();
						}
					};

					// Change the marker visibility
					IconMarkerComponent.prototype.updateVisibility = function(notifyLayer) {
						if(this.visible == true) {
							this.layer.node.addLayer(this.node);
							$(this.node._icon).find('.leaflet-marker-iconlabel').append(this.content);
							this.update();
							if(notifyLayer != false) ctrl.onComponentVisibilityChange(this);
						}
						else if(this.visible == false) {
							$(this.content).remove();
							this.layer.node.removeLayer(this.node);
							if(notifyLayer != false) ctrl.onComponentVisibilityChange(this);
						}
					};

				}

			}
			
			// Transclude own scope
			transclude(scope, function (clone, scope) {
				element.append(clone);
			});
			
			// Set the element ID to scope if it exists
			if(attrs.id != undefined) scope.elementId = attrs.id;
			
			// Set the name to scope if it exists
			if(attrs.name != undefined) scope.elementName = attrs.name;
			
			// Read the options from attributes
			scope.options = {
				"disabled":			attrs.disabled == undefined || attrs.disabled.length == 0 ?
										false :
										eval(attrs.disabled), 
									// Optional. Disable the layer.
									
				"position":			attrs.position, 
									// Mandatory. Position for the point. Define as an array with object properties for lat and lon.
									//   Example 1: [object.lat, object.lon]
									//     This will plot the object at the position object.lat and object.lon where object is the
									//     data row, and lat and lon are the properties of that object that point to latitude and 
									//     longitude respectively.

				"marker":			{},
									// Mandatory. Automatically generated for this directive type.

				"icon":				attrs.icon == undefined || attrs.icon.length == 0 ?
										undefined :
										{url:	attrs.icon},
									// Optional but you won't see anything if not specified. Icon URL for image to display as standard marker. 
									// May be a literal value:
									//	 Example 1: img/lights/green-light.png
									//     This will always render green-light.png
									// Or a conditional object. Keys in the object define the image URLs. Values define boolean
									//   expressions. The first expression that evaluates to true will get assigned.
									//   Example 2: {'img/lights/green-light.png': object.status == 'Approach', 'img/lights/blue-light.png': object.status == 'Circling', 'img/lights/red-light.png': true}
									//     This will give a green-light.png if object has status Approach, blue-light.png if 
									//     object has status Circling, or red-light.png otherwise.
									
				"iconLabel":		attrs.iconLabel == undefined || attrs.iconLabel.length == 0 ?
										undefined :
										eval(attrs.iconLabel) == true ? {} : undefined,
									// Optional. Apply an icon label. Must be true or false.
									
				"iconLabelContent":	$(element).find('livemap-icon-label').length != 1 ?
										undefined :
										$(element).find('livemap-icon-label').html(),
									// Optional. Icon label HTML content. Defined in a tag structure underneath this element.
									//   Example:
									//     <livemap-icon-label ng-non-bindable>
									//       <span>
									//         <div style="background-color:white;font-size:20px;">
									//			  <img src="img/other/status-ok.png" height="20px" ng-if="object.status == 'Approach'"></img>
									//			  <img src="img/other/status-unknown.png" height="20px" ng-if="object.status == 'Circling'"></img>
									//			  {{::object.flightNumber}}
									//		   </div>
									//	     </span>
									//     </livemap-icon-label>
									//
									//   Notes:
									//     1. <livemap-icon-label> MUST have ng-non-bindable
									//     2. First element within <livemap-icon-label> MUST be a single root element, recommend <span> or <div>
									//     3. Any content you wish between the root element tags
									//     4. Can use any Angular binding on the object, which represents the row
									//     5. Recommend to use bind-once notation (::) {{::object.flightNumber}} to improve performance
									//     6. Can use ng-if, ng-switch, and other structures if desired
									
				"popup":			$(element).find('livemap-popup').length != 1 ?
										undefined :
										$(element).find('livemap-popup').html(),
									// Optional. Popup HTML content. Defined in a tag structure underneath this element.
									//   Example:
									//     <livemap-popup ng-non-bindable>
									//        <span>
									//		    <h3>Flight: {{::object.flightNumber}}</h3>
									//	     </span>
									//     </livemap-icon-label>
									// 
									//   See all notes above as for iconLabelContent. Same rules apply here.
			};

			// Get marker options
			if(scope.options.marker != undefined) {
				scope.options.marker.options = {
					"clickable":		attrs.clickable == undefined || attrs.clickable.length == 0 ?
											true :
											eval(attrs.clickable),
										// Optional. Make the marker clickable. Doesn't seem to work with Leaflet. 
									
					"opacity":			attrs.opacity == undefined || attrs.opacity.length == 0 ?
											1.0 :
											eval(attrs.opacity),
										// Optional. Opacity for the marker.

					"riseOnHover":		attrs.riseOnHover == undefined || attrs.riseOnHover.length == 0 ?
											false :
											eval(attrs.riseOnHover),
										// Optional. Rise over other elements on mouseover. Must be true or false.

					"riseOffset":		attrs.riseOffset == undefined || attrs.riseOffset.length == 0 ?
											250 :
											eval(attrs.riseOffset),
										// Optional. Z-index offset on hover. Ignored if riseOnHover is false.

					"zIndexOffset":		attrs.zIndexOffset == undefined || attrs.zIndexOffset.length == 0 ?
											0 :
											eval(attrs.zIndexOffset),
										// Optional. Z-index offset for normal condition.
				};
			}
			
			// Get icon options
			if(scope.options.icon != undefined) {
				scope.options.icon.options = {
					"iconSize":		attrs.iconSize == undefined || attrs.iconSize.length == 0 ?
										[0, 0] :
										eval(attrs.iconSize),
									// Optional. Size of the icon image in pixels. Object array [x, y] where x and y are numbers.
									
					"iconAnchor":	attrs.iconAnchor == undefined || attrs.iconAnchor.length == 0 ?
										[0, 0] :
										eval(attrs.iconAnchor),
									// Optional. Coordinates of the icon relative to its top left corner. Object array [x, y] where x and y are numbers.

					"popupAnchor":	attrs.popupAnchor == undefined || attrs.popupAnchor.length == 0 ?
										[0, 0] :
										eval(attrs.popupAnchor),
									// Optional. Coordinates of the point where popups open, relative to the icon anchor. Object array [x, y] where x and y are numbers.

					"shadowUrl":	attrs.shadowUrl == undefined || attrs.shadowUrl.length == 0 ?
										undefined :
										attrs.shadowUrl,
									// Optional. Url for shadow image.

					"shadowSize":	attrs.shadowSize == undefined || attrs.shadowSize.length == 0 ?
										undefined :
										eval(attrs.shadowSize),
									// Optional. Shadow size. Object array [x, y] where x and y are numbers.

					"shadowAnchor":	attrs.shadowAnchor == undefined || attrs.shadowAnchor.length == 0 ?
										undefined :
										eval(attrs.shadowAnchor),
									// Optional. Shadow anchor. Object array [x, y] where x and y are numbers.									
				};
				
				// Get icon label options
				if(scope.options.iconLabel != undefined) {
					scope.options.iconLabel.options = {
						"labelAnchor":		attrs.labelAnchor == undefined || attrs.labelAnchor.length == 0 ?
												new L.Point(0, 0) :
												new L.Point(eval(attrs.labelAnchor)[0], eval(attrs.labelAnchor)[1]),
											// Optional. Label anchor position. Object array [x, y] where x and y are numbers.

						"labelClassName":	attrs.labelClasses == undefined || attrs.labelClasses.length == 0 ?
												undefined :
												attrs.labelClasses,
											// Optional. Label class names. These will be applied to the wrapper div in the DOM.

						"wrapperAnchor":	attrs.wrapperAnchor == undefined || attrs.wrapperAnchor.length == 0 ?
												new L.Point(0, 0) :
												new L.Point(eval(attrs.wrapperAnchor)[0], eval(attrs.wrapperAnchor)[1]),
											// Optional. Wrapper anchor. Object array [x, y] where x and y are numbers.									
					};
					
					// Merge attributes
					for(var attrname in scope.options.iconLabel.options)
						scope.options.icon.options[attrname] = scope.options.iconLabel.options[attrname];
				}
			}
			
			// Register a component factory on the layer
			ctrl.registerComponentFactory({
				onInsert: function(object){
					return createComponentObject(object);
				},
			});
			
			// Create a new map component for the given object
			var createComponentObject = function(object) {
				return new IconMarkerComponent(scope.options.iconLabelContent, object);
			};
						
		}
	};	
});

app.directive('livemapFilterPanel', function($log, Modal) {
	return {
		templateUrl: './templates/live-map-filter-panel.html',
		replace: false,
		transclude: true,
		restrict: 'E',
		scope: {},
		require: '^livemap',
		link: function(scope, element, attrs, ctrl, transclude) {		

			// Get the overlay panel and move to the livemap-container
			//   This allows the div to be sized to the full height of the map
			var overlay = $(element).find('.livemap-overlay').appendTo('.livemap-container');
			
			// Show the map menu overlay when clicking the overlay button
			$(element).find('.livemap-overlay-button').on('click', function(e) {
				$(overlay).show();
				scope.layers = ctrl.getLayers();
				scope.$apply();
				return false;
			});
			
			// Hide the map menu overlay when clicking the overlay close button
			$(overlay).find('.livemap-overlay-close-icon').on('click', function(e) {
				$(overlay).hide();
				return false;
			});
		
			// Capture double clicks on this panel to avoid zooming the map below
			$(overlay).on('dblclick', function(e) {
				return false;
			});
			
			// Display the dataset modal
			scope.displayDatasetModal = function(options, dataset) {
				if(options.datasetModal == undefined) return;
				
				// Setup modal options
				var modalOptions = {
					template: '<dataset-modal></dataset-modal>',
					inputContext: {
						dataset: dataset,
						display: options.datasetModal == 'R' || options.datasetModal == 'RW',
						editor:  options.datasetModal == 'RW',
					}
				};

				// Show the modal with the options
				var modal = Modal.show(modalOptions);
			};
			
		}
	};	
});

