app.directive('livetable', function($log, $filter, $compile, SimpleDateFormat, NumberFormat, Overlay, Modal, Clipboard, Selection, ContextMenu) {
	return {
		templateUrl: './templates/live-table.html',
		replace: false,
		transclude: true,
		restrict: 'E',
		scope: {
			dataset: 	'=',
			filter:		'=',
			selected:	'='
		},
		require: '?^livetableTabs',		// optionally require a table tabs directive
		controller: function($scope) {
		},
		link: function(scope, element, attrs, ctrl) {
			// DEFINE OBJECTS
			{
				// LiveTable object
				{
					function LiveTable() {
						this.columnsModel 	= undefined;
						this.keyColumnModel	= undefined;
						this.order			= undefined;
						this.tableBody		= undefined;
						
						// These properties are used for microbatching the updates
						this.microbatch = {
							rtime:		new Date(1, 1, 2000, 12,00,00),
							timeout:	false,
							delta:		0
						}				
					};
					
					LiveTable.prototype.draw = function() {
					};
					
					LiveTable.prototype.update = function() {
						var self = this;
						
						// Microbatch handler
						self.microbatch.rtime = new Date();
						if(self.microbatch.timeout === false) {
							self.microbatch.timeout = true;
							setTimeout(eventEnd, self.microbatch.delta);
						}					
						function eventEnd() {
							if (new Date() - self.microbatch.rtime < self.microbatch.delta)
								setTimeout(eventEnd, self.microbatch.delta);
							else {
								self.microbatch.timeout = false;
								run();
							}               
						};
						
						// Run the update
						function run() {
							// Get the scroll top
							var scrollTop = $(element).find('.livetable-table-body').scrollTop();
							
							// Apply filters
							scope.$applyAsync(function() {
								scope.filterset = scope.rows.toArray();
								var order = self.order;
								if(order.length == 0) order = [self.keyColumnModel];
								scope.filterset = $filter('objectOrderBy')(scope.filterset, order, scope.snapshot);
								scope.filterset = $filter('objectFilter')(scope.filterset, scope.search, scope.options.searchCols, scope.filter, self.columnModel);
								scope.results = $filter('objectLimitTo')(scope.filterset, scope.pagination);
							
								// Detach existing and reattach in correct order
								self.tableBody.children('.livetable-structure-row-data').detach();						
								for(var idx = 0; idx < scope.results.length; idx++) {
									// Render the nodes. If the node is already rendered, it'll just be updated.  
									//   We do a lazy render only when row is displayed, so always need to render here.
									//   Append node in correct location based on new filtered order.
									scope.results[idx].render(self.tableBody);
									self.tableBody.append(scope.results[idx].node);							
								}

								// Reset the scroll top
								$(element).find('.livetable-table-body').scrollTop(scrollTop);
							});
						};
					};
				}

				// LiveTableColumnModel object
				{
					function LiveTableColumnModel(table, field, options, ordering) {
						this.field 		= field;
						this.options 	= options == undefined ? {} : options;
						this.ordering 	= ordering;
						this.table		= table;

						if(this.options == undefined) this.options = {};
					};
					
					LiveTableColumnModel.prototype.bodyClasses = function(object, data) {
						var c = "";
						if(scope.options.ordering == true && scope.options.orderClasses == true && this.ordering != undefined)
							c = c + " ordered";
						if(this.options.classes != undefined)
							c = c + ' ' + this.options.classes;
						if(this.options.conditionalClasses != undefined) {											
							var cond = eval('(' + this.options.conditionalClasses + ')');
							for(var idx = 0; idx < Object.keys(cond).length; idx++) {
								var cls = Object.keys(cond)[idx];
								if(cond[cls] == true) c = c + ' ' + cls;
							}
						}
						return c.trim();
					};
					
					LiveTableColumnModel.prototype.headerClasses = function() {
						var c = "";
						if(scope.options.ordering == true && this.options.orderable != false) {
							c = c + " orderable";
							if(this.ordering == 'asc') c = c + ' ordered-asc';
							else if(this.ordering == 'desc') c = c + ' ordered-desc';
							else if(this.ordering == undefined) c = c + ' unordered';
						}
						return c.trim();
					};
					
					LiveTableColumnModel.prototype.format = function(data) {
						if(this.options.format != undefined) {
							if(this.field.type == "timestamp") return SimpleDateFormat.format(data, this.options.format);
							if(this.field.type == "double") return NumberFormat.format(data, this.options.format);
						}
						return data;
					};
					
					LiveTableColumnModel.prototype.order = function() {
						if(scope.options.ordering == false || this.options.orderable == false) return;
						if(this.ordering == 'asc') {
							this.ordering = 'desc';
							if(scope.options.orderMulti == false) {
								if(this.table.order[0].field.name != this.field.name)
									this.table.order[0].ordering = undefined;
								this.table.order[0] = this;				
							}
						}
						else if(this.ordering == 'desc') { 
							this.ordering = undefined;
							var index = this.table.order.indexOf(this);
							if(scope.options.orderMulti == false) {
								if(this.table.order[0].field.name != this.field.name)
									this.table.order[0].ordering = undefined;
								this.table.order[0] = this;
							}
							else
								this.table.order.splice(index, 1);
						}
						else if(this.ordering == undefined) {
							this.ordering = 'asc';
							if(scope.options.orderMulti == false && this.table.order.length > 0) {
								if(this.table.order[0].field.name != this.field.name)
									this.table.order[0].ordering = undefined;
								this.table.order[0] = this;
							}
							else
								this.table.order.push(this);
						}
						this.table.update();
					};
					
					LiveTableColumnModel.prototype.orderIndex = function() { return this.table.order.indexOf(this); };
					LiveTableColumnModel.prototype.style = function() { return this.options.width == undefined ? {} : {'width': this.options.width}; };
					LiveTableColumnModel.prototype.title = function() { if(this.options.title == undefined) return this.field.name; else return this.options.title; };
				}

				// LiveTableRow object
				{
					function LiveTableRow(object, selected) {
						this._key 		= object._key;
						this.cells 		= [];
						this.data 		= object;
						this.node 		= undefined;
						this.odd 		= false;
						this.selected 	= (selected == undefined ? false : selected);
					};
					
					LiveTableRow.prototype.render = function() {
						if(this.node == undefined) {
							// Render the node and cells
							this.node = $(document.createElement('div')).attr('id', this._key);
							for(var idx = 0; idx < this.cells.length; idx++)
								this.cells[idx].render();
							
							// Register event listeners
							var self = this;
							this.node.on('click', null, this, function(event) {
								scope.select(self);
							});
						}
						this.update();
					};
					
					LiveTableRow.prototype.rowClasses = function() {
						var c = "livetable-structure-row livetable-structure-row-data";
						if(scope.options.stripeClasses == true && this.odd == true)
							c = c + " stripe-odd";
						else if(scope.options.stripeClasses == true && !this.odd == true)
							c = c + " stripe-even";
						if(scope.options.selectable == true)
							c = c + " selectable";
						if(this.selected == true)
							c = c + " selected"
						return c.trim();
					};
					
					LiveTableRow.prototype.update = function() {
						if(this.node == undefined) return;
						this.node.removeClass();
						this.node.addClass(this.rowClasses());
						for(var idx = 0; idx < this.cells.length; idx++)
							this.cells[idx].update();
						
					};
				}

				// LiveTableRowCell object
				{
					function LiveTableRowCell(columnModel, object, row) {
						this.columnModel 		= columnModel;
						this.object 			= object;
						this.row 				= row;
						this.node 				= undefined;
						this.contentScope 		= undefined;
						this.hasComplexContent 	= this.columnModel.options.content != undefined;

						if(this.hasComplexContent) {
							this.contentScope = scope.$new(true);
							this.contentScope.Math = window.Math;
							this.contentScope.formatDateTime = function(dateStr, format) {
								return SimpleDateFormat.format(dateStr, format);
							};
							this.contentScope.formatNumber = function(numStr, format) {
								return NumberFormat.format(numStr, format);
							};
						}
					};
					
					LiveTableRowCell.prototype.cellClasses = function() {
						return 'body-cell datatype-' + this.columnModel.field.type + ' table-' + scope.elementId + '-' + this.columnModel.field.name + 
									' dataset-' + scope.dataset.getId() + '-' + this.columnModel.field.name + ' ' + 
									this.columnModel.bodyClasses(this.object, this.object[this.columnModel.field.name]);
					};
					
					LiveTableRowCell.prototype.render = function() {
						var self = this;
						if(this.columnModel.options.visible == false) return;
						this.node = $(document.createElement('div'));
						if(this.hasComplexContent == true) {
							$(this.node).empty().html(this.columnModel.options.content);
							$compile($(this.node).contents())(this.contentScope);
						}
						this.node.contextmenu(function(e) { return scope.bodyMenu(self.object, self.row, self.columnModel, e); });
						this.row.node.append(this.node);
					};
					
					LiveTableRowCell.prototype.update = function() {
						if(this.node == undefined) return;
						this.node.attr('class', this.cellClasses()).css(this.columnModel.style());
					
						// Update the contentScope if it has complex content
						if(this.hasComplexContent) {
							var display = this.columnModel.format(this.object[this.columnModel.field.name]);
							this.contentScope.object = this.object;
							this.contentScope.data = this.object[this.columnModel.field.name];
							this.contentScope.display = display;
						}
						else {
							var data = this.object[this.columnModel.field.name];
							var display = data == undefined ? "" : this.columnModel.format(data);
							this.node.html(display); 						
						}
						
						// TODO
						//this.columnModel.content(this.context, this.node, this.object, this.object[this.columnModel.field.name]);
					};
				}
			}
			
			// Add flexbox classes to the directive element
			$(element).addClass('fill-remainder flex-vertical-container');
			
			// Set the element ID to scope if it exists
			if(attrs.id != undefined) scope.elementId = attrs.id;
			
			// Set the name to scope if it exists
			if(attrs.name != undefined) scope.elementName = attrs.name;
			
			// Read the options from attributes
			scope.options = {
				"configIcon": 		attrs.configIcon == undefined ? 
										false : 
										eval(attrs.configIcon),
									// Optional. Show or hide config icon. May be true or false.
										
				"datasetIcon": 		attrs.datasetIcon == undefined ? 
										false : 
										eval(attrs.datasetIcon),
									// Optional. Show or hide dataset icon field. May be true or false.
										
				"datasetModal":		attrs.datasetModal == "R" || attrs.datasetModal == "RW" ?
										attrs.datasetModal :
										undefined,
									// Optional. Show the dataset modal if set. May be one of:
									//   R -- read-only modal
									//   RW -- read-write modal
										
				"filter":			(attrs.filter != undefined && attrs.filter.length > 0),
									// Optional. Custom data filter enabled. May be true or false.
									// This attribute is actually an object binding.
						
				"filterClear":		attrs.filterClear == undefined ?
										true :
										eval(attrs.filterClear),
									// Optional. Enables or disables clearing of the external data filter using
									//   the clear filter object.
									
				"info": 			attrs.info == undefined ? 
										false : 
										eval(attrs.info),
									// Optional. Show or hide information display field. May be true or false.
										
				"ordering": 		attrs.ordering == undefined ? 
										false : 
										eval(attrs.ordering),
									// Optional. Enable column header ordering. May be true or false.
									
				"order":			attrs.order == undefined || attrs.order == "" || attrs.ordering == "false" ? 
										[['_key', 'asc']] : 
										eval(attrs.order),
									// Optional. Initial sort order to apply if ordering is enabled. Array in the format
									// [[property1, 'asc'|'desc'], [property2, 'asc'|'desc'], ...]
									// Default is always ascending on _key
									
				"orderMulti":		attrs.orderMulti == undefined ? 
										false : 
										eval(attrs.orderMulti),
									// Optional. Allow multiple column ordering if ordering is enabled. May be true or false.
									
				"orderClasses":		attrs.orderClasses == undefined ? 
										false : 
										eval(attrs.orderClasses),
									// Optional. Enable highlighting of ordered columns if ordering is enabled. May be true or false.
									
				"paginationSize": 	attrs.paginationSize == undefined || attrs.paginationSize  == "" ? 
										25 : 
										eval(attrs.paginationSize),
									// Optional. Pagination size row count. Must be an integer.
									
				"searchable": 		attrs.searchable == undefined ? 
										false : 
										eval(attrs.searchable),
									// Optional. Enable table searchable. May be true or false.
									
				"searchCols":		attrs.searchCols == undefined || attrs.searchCols == "" ? 
										eval("[]") : 
										eval(attrs.searchCols), 
									// Optional. Array of columns to search on if enabled.
									// [null,{'search':'Sea'},null,{'search':'^[0-9]','escapeRegex':false},null]
									
				"searchBoxLabel": 	attrs.searchBoxLabel == undefined ? 
										true : 
										eval(attrs.searchBoxLabel),
									// Optional. Show or hide search box label. May be true or false.
										
				"searchDelay":		attrs.searchDelay == undefined ? 
										100 : 
										eval(attrs.searchDelay),
									// Optional but recommended. Delay to wait before executing search in msec if searing is enabled. Must be integer.
										
				"selectable":		(attrs.selected != undefined && attrs.selected.length > 0),
									// Optional. User can select table row and event will be emitted upwards 
									//   in scope. If this is true, then id must be specified. May be true or false.
					
				"stripeClasses":	attrs.stripeClasses == undefined || attrs.stripeClasses == "" ? 
										false : 
										eval(attrs.stripeClasses),
									// Optional. Apply stripe classes to table. May be true or false.
									
				"title":			attrs.title,
									// Optional. Display title in the table header section.
			};

			// Read the column model options from attributes
			scope.options.columnModel = [];			
			$(element).find('column').each(function(index, columnElement) {
				var options = {
					"node":					columnElement,
					
					"classes":				$(columnElement).attr('classes'),
											// Optional. List of classes to apply to cells in this column.

					"conditionalClasses":	$(columnElement).attr('conditional-classes') == undefined || $(columnElement).attr('conditional-classes') == "" ?
												undefined :
												$(columnElement).attr('conditional-classes'),
											// Optional. Conditional classes to apply based on cell value.
					
					"content":				$(columnElement).find('column-content').length != 1 ?
												undefined :
												$(columnElement).find('column-content').html(),
											// Optional. Column content.
										
					"format":				$(columnElement).attr('format'),
											// Optional. Formatting to apply to the data in the column.

					"name":					$(columnElement).attr('name'),
											// Mandatory. Column name.
											
					"orderable":			$(columnElement).attr('orderable') == undefined || $(columnElement).attr('orderable') == "" ? 
												true : 
												eval($(columnElement).attr('orderable')),
											// Optional. Allow ordering on this column. May be true or false.

					"searchable":			$(columnElement).attr('searchable') == undefined || $(columnElement).attr('searchable') == "" ? 
												true : 
												eval($(columnElement).attr('searchable')),
											// Optional. Allow searching on this column. May be true or false.

					"title":				$(columnElement).attr('title'),
											// Optional. Title to display.

					"visible":				$(columnElement).attr('visible') == undefined || $(columnElement).attr('visible') == "" ? 
												true : 
												eval($(columnElement).attr('visible')),
											// Optional. Show or hide the column. May be true or false.

					"width":				$(columnElement).attr('width') == undefined || $(columnElement).attr('width') == "" ? 
												undefined : 
												$(columnElement).attr('width')
											// Optional. Width of the column. May be number in px or percentage.
				};
				scope.options.columnModel.push(options);
			});
			
			// Read the context menu options from attributes
			scope.bodyContextMenu = [];
			$(element).find('body-context-menu').each(function(index, bodyContextMenuElement) {
				var options = {
					"divider":				$(bodyContextMenuElement).attr('divider'),
					
					"label":				$(bodyContextMenuElement).attr('label'),
					
					"displayCondition":		$(bodyContextMenuElement).attr('display-condition'),
					
					"eventId":				$(bodyContextMenuElement).attr('event-id'),
				};
				scope.bodyContextMenu.push(options);
			});
			
			// Declare a row collection and a results array for rows to display once filters are applied
			scope.rows = new Collection(['_key']);
			scope.results = [];
			
			// Declare the overlay counter
			scope.overlay = 0;
			
			// Setup pagination for infinite scrolling
			scope.pagination = {
				size:	scope.options.paginationSize,
				begin: 	0,
				end:	scope.options.paginationSize
			};
			
			// Watch for the dataset creation
			scope.$watch('dataset', function() {
				if(scope.dataset != undefined) {
					scope.registerSink();
				}
			});
			
			// Watch the overlay count and show and hide overlay div
			scope.$watch('overlay', function() {
				if(scope.overlay > 0)
					Overlay.show(element);
				else
					Overlay.hide(element);
			});
			
			// Watch the search box to update the table data
			scope.$watch('search', function() {
				if(scope.table != null) scope.table.update();
			});
			
			// Watch the column filters to update the table
			scope.$watch(function() {
				if(scope.table == undefined || scope.table.columnModel == undefined) return false;
				var s = "";
				for(var idx = 0; idx < scope.table.columnModel.length; idx++) {			
					var column = scope.table.columnModel[idx];
					s = s + column.search;
				}
				return s;
			}, function(newVal, oldVal) {
				if(scope.table != null) scope.table.update();
			});
				
			// Watch the external filter to update the table
			scope.$watch('filter', function() {
				if(scope.table != null) scope.table.update();
			});
			
			// Watch the external select to update the table
			scope.$watch('selected', function() {
				var item = undefined;
				if(scope.selected != undefined) {
					item = scope.rows.get(scope.selected._key);
				}
				item = Selection.watchSelected(scope.selectedRow, item);
				if(item == false) return;
				if(item == undefined) {
					scope.selectedRow = undefined;
				}
				else {
					scope.selectedRow = item;
				}
			});
			
			// Watch the search box, column filters, and filter object to mark as filtered
			//   This will display the filter icon in the UI.
			scope.$watch(function() {
				// Determine if the table is filtered by the search box
				if(!(scope.search == undefined || scope.search == "") == true) return true;
				
				// Determine if the table if filtered by a column filter box
				if(scope.table != undefined && scope.table.columnModel != undefined) {
					for(var idx = 0; idx < scope.table.columnModel.length; idx++) {			
						var column = scope.table.columnModel[idx];
						if(column.search != undefined && column.search.length > 0) {
							return true;
						}
					}
				}
				
				// Determine if the table is filtered by an external filter object
				if(!(scope.filter == undefined || Object.keys(scope.filter).length == 0) == true) return true;
				
				return false;
			}, function(newVal, oldVal) {
				scope.filtered = newVal;
			});
			
			// Watch the scroll bar to change the pagination
			$(element).find('.livetable-table-body').scroll(function(event) {
				var scrollPercentage = $(this).scrollTop() / ($(element).find('.livetable-table-body').find('.livetable-structure-table').height() - $(this).height());
				if(scrollPercentage >= 0.75 && scope.results.length < scope.rows.length()) {
					scope.pagination.end = scope.pagination.end + scope.pagination.size;
					scope.table.update();
				}
			});
							
			// Create and register a sink
			scope.registerSink = function() {
				var sink = {				
					onSchemaChange:	function(schema) {
						scope.table = createTable(schema);
						scope.$applyAsync();
						scope.table.draw();
					},
					onSnapshotStart: function() {
						scope.snapshot = true;
						scope.overlay = scope.overlay + 1;
						scope.$applyAsync();
					},
					onSnapshotEnd: function() {
						scope.snapshot = undefined;
						scope.overlay = scope.overlay - 1;
						scope.$applyAsync();
					},
					onInsert: function(object) {
						var row = createRow(object, scope.table);
						scope.rows.put(row);
						scope.table.update();
						scope.$applyAsync();
					},
					onUpdate: function(object) {
						var row = scope.rows.get(object._key);
						row.data = object;
						row.update();
						scope.$applyAsync();
					},
					onDelete: function(object) {
						// Unselect if selected
						if(scope.selected != undefined && scope.selected._key == object._key) {
							var item = scope.rows.get(scope.selected._key);
							scope.select(item);
						}
						// Remove the row from the collection
						scope.rows.remove(object._key);
						scope.table.update();
						scope.$applyAsync();
					},
					onClear: function() {
						// Remove all rows from the collection
						scope.rows.removeAll();
						scope.table.update();
						scope.select(undefined);
						scope.$applyAsync();
					}
				};
				
				// Register the datasink with the dataset
				var datasink = new DataSink(scope.elementName, 'LiveTable', sink);
				scope.dataset.registerSink(datasink);
			}

			// Create the new table
			var createTable = function(schema) {
				// Get the configured sort for defaults
				var initialOrder = {};
				if(scope.options.ordering == true) {
					var len = scope.options.orderMulti == false ? 1 : scope.options.order.length;
					for(var idx = 0; idx < len; idx++)
						initialOrder[scope.options.order[idx][0]] = scope.options.order[idx][1];
				}
							
				// Create the table
				table = new LiveTable();
				table.tableBody = $(element).find('.livetable-table-body').find('.livetable-structure-table');				
				
				// Create the column model
				table.columnModel = [];

				// Always define the _key column
				table.keyColumnModel = new LiveTableColumnModel(table, {name: '_key', type: 'string'}, undefined, undefined);
				
				// If there are columns defined in the directive definition, use that as the column model
				if(scope.options.columnModel.length > 0) {
					for(var idx = 0; idx < scope.options.columnModel.length; idx++) {
						var columnModelOptions = scope.options.columnModel[idx];
						var field = schema.getField(columnModelOptions.name);
						if(field == undefined) continue;
						var columnModel = new LiveTableColumnModel(table, field, columnModelOptions, initialOrder[field.name]);
						table.columnModel.push(columnModel);
					}
				}
				// Otherwise use all fields in the table schema
				else {
					for(var idx = 0; idx < schema.getFieldLength(); idx++) {
						var field = schema.getFieldByIndex(idx);
						var columnModel = new LiveTableColumnModel(table, field, undefined, initialOrder[field.name]);
						table.columnModel.push(columnModel);
					}
				}
				
				// Apply the initial ordering
				table.order = [];

				if(scope.options.ordering == true) {
					var len = scope.options.orderMulti == false ? 1 : scope.options.order.length;
					for(var idx = 0; idx < len; idx++) {
						for(var col = 0; col < table.columnModel.length; col++) {
							if(table.columnModel[col].field.name == scope.options.order[idx][0])
								table.order.push(table.columnModel[col]);
						}
					}
				}				
				
				// This may be a re-create of a table already created. If there are any rows in the
				//   rows collection, regenerate the rows now.
				for(var idx = 0; idx < scope.rows.length(); idx++) {
					var oldrow = scope.rows.getByIndex(idx);
					var row = createRow(oldrow.data, table, oldrow.selected);
					scope.rows.put(row);
				}

				return table;
			};
			
			// Create a new row
			var createRow = function(object, table, selected) {
				var row = new LiveTableRow(object, selected);

				// Loop over the column models and create a table cell instance for each column in this row
				for(var idx = 0; idx < table.columnModel.length; idx++) {
					var columnModel = table.columnModel[idx];
					var tableRowCell = new LiveTableRowCell(columnModel, object, row);
					row.cells.push(tableRowCell);					
				}
				
				return row;
			};
			
			// Select a row internally
			scope.select = function(row) {		
				var item = Selection.select(scope.options.selectable, scope.selectedRow, row);
				if(item == false) return;
				if(item == undefined) {
					scope.selectedRow = undefined;
					scope.selected = undefined;
				}
				else {
					scope.selectedRow = item;
					scope.selected = item.data;
				}
				scope.$applyAsync();
			};			
			
			// HEADER MENU ACTIONS
			// Display the header context menu
			scope.headerMenu = function(column, e) {
				var menu = ContextMenu.show(e, element, ".header-context-menu");				
				return false;
			};			
			
			// Clear all ordering
			scope.clearOrdering = function() {
				for(var idx = 0; idx < scope.table.columnModel.length; idx++) {
					scope.table.columnModel[idx].ordering = undefined;
				}
				scope.table.order = [];
				scope.table.update();
			};
			
			// Clear search and column filters
			scope.clearSearches = function() {
				if(scope.options.filterClear == true)
					scope.filter = undefined;
				scope.search = undefined;
				for(var idx = 0; idx < scope.table.columnModel.length; idx++) {
					scope.table.columnModel[idx].search = undefined;
				}
				scope.table.update();
			};
			
			// Toggle visibility of field searches
			scope.toggleFieldSearches = function() {
				if(scope.searchesVisible == undefined)
					scope.searchesVisible = true;
				else
					scope.searchesVisible = undefined;
			};			
			
			// Display the dataset modal
			scope.displayDatasetModal = function() {
				if(scope.options.datasetModal == undefined) return;
				
				// Setup modal options
				var modalOptions = {
					template: '<dataset-modal></dataset-modal>',
					inputContext: {
						dataset: scope.dataset,
						display: scope.options.datasetModal == 'R' || scope.options.datasetModal == 'RW',
						editor:  scope.options.datasetModal == 'RW',
						onModalClose: ctrl != undefined ? ctrl.onModalClose : undefined,
					}
				};

				// Show the modal with the options
				var modal = Modal.show(modalOptions);
			};

			// Display table config modal
			scope.tableConfig = function() {
				// TODO table config
				$log.debug('not implemented yet');
			};
			
			// BODY MENU ACTIONS
			// Display the body context menu
			scope.bodyMenu = function(object, row, columnModel, e) {				
				// Add custom body context menu items
				var customMenuItems = [];
				for(var idx = 0; idx < scope.bodyContextMenu.length; idx++) {
					var bodyContextMenuItem = scope.bodyContextMenu[idx];
					var customMenuItem = {
						self:		this,
						config:		bodyContextMenuItem,
						divider:	bodyContextMenuItem.divider != undefined ? eval(bodyContextMenuItem.divider) : false,
						eventId:	bodyContextMenuItem.eventId,
						label:		bodyContextMenuItem.label,
						display:	function() {
							var include = true;
							if(this.config.displayCondition != undefined)
								include = eval(this.config.displayCondition);
							return include;
						},
						action:		function(e) {
							scope.customAction(object, row, columnModel, e);
						}
					};					
					customMenuItems.push(customMenuItem);
				}
				
				// Create the menu
				var menu = ContextMenu.show(e, element, ".body-context-menu", customMenuItems);			
				
				// Unbind existing click handlers
				$(menu).find('.copy-cell').unbind('click');
				$(menu).find('.copy-row').unbind('click');
				$(menu).find('.filter-cell').unbind('click');

				// Bind new click handlers
				$(menu).find('.copy-cell').click(function(e) {
					scope.copyCell(object, row, columnModel, e);
				});
				
				$(menu).find('.copy-row').click(function(e) {
					scope.copyRow(object, row, columnModel, e);
				});
				
				$(menu).find('.filter-cell').click(function(e) {
					scope.filterCell(object, row, columnModel, e);
				});
				
				return false;
			};
			
			// Determine if copy is supported
			scope.copySupported = document.queryCommandSupported('copy');
			
			// Copy cell to clipboard
			scope.copyCell = function(object, row, columnModel, e) {
				var menu = $(element).find(".body-context-menu");
				var copyText = object[columnModel.field.name];
				Clipboard.copy(copyText, menu);				
			};

			// Copy row to clipboard
			scope.copyRow = function(object, row, columnModel, e) {
				var mode = 'csv'
				var copyText = ''				
				if(mode == 'csv') {
					for(var idx = 0; idx < Object.keys(object).length; idx++) {
						if(idx > 0) copyText = copyText + ',';
						var k = Object.keys(object)[idx];
						copyText = copyText + object[k];
					}
				}
				else if(mode == 'json') {
					copyText = JSON.stringify(object, null, 2);
				}

				var menu = $(element).find(".body-context-menu");				
				Clipboard.copy(copyText, menu);				
			};

			// Filter by selected cell
			scope.filterCell = function(object, row, columnModel, e) {
				var fieldName = columnModel.field.name;
				var value = object[fieldName];
				var f = {};
				f[fieldName] = value;
				scope.filter = f;
				scope.$apply();
			};
			
			// Custom action
			scope.customAction = function(object, row, columnModel, e) {
				var operation = $(e.target).attr('event-id');
				var topic = 'custom:' + scope.elementId;
				scope.$emit(topic, operation, object, row, columnModel);
			};
		}
	};	
});


app.directive('livetableTabs', function($log) {
	return {
		templateUrl: './templates/live-table-tabs.html',
		replace: false,
		transclude: true,
		restrict: 'E',
		scope: {
			datasource:	'='
		},
		controller: function($scope) {
			this.onModalClose = function() {
				$scope.onModalClose();
			};
		},
		link: function(scope, element, attrs) {
			// Add flexbox classes to the directive element
			$(element).addClass('flex-fill-vertical-container flex-fill-vertical-remainder');
			
			// Set the element ID to scope if it exists
			if(attrs.id != undefined) scope.elementId = attrs.id;
			
			// Set the name to scope if it exists
			if(attrs.name != undefined) scope.elementName = attrs.name;

			// Select the query tab
			scope.selectQueryTab = function(query) {
				scope.selectedQuery = query;				
			}
			
			// Add a new query
			scope.addQuery = function() {
				$log.debug('add query');
			}

			// Close the query
			scope.closeQuery = function(query) {
				$log.debug('close query');
			}

			// On modal close, select the tab for the currently selected query
			scope.onModalClose = function() {
				$('.nav-tabs a[href="#' + scope.selectedQuery.id + '"]').tab('show');
			}
		}
	};	
});
