// Applies a sort to the list of items using the order definition
app.filter('objectOrderBy', function($log) {
	return function(items, order, snapshot) {
		if(snapshot == true) return items;
		var filtered = [];
		angular.forEach(items, function(item) {
			filtered.push(item);
		});

		var compare = function(a, b, dir, field) {
			return dir * (a.data[field] == b.data[field] ? 0 : a.data[field] < b.data[field] ? 1 : -1);
		};
			
		// This is default sort algorithm
		/*filtered.sort(function (a, b) {
			var compare = function(a, b, dir, field) {
				return dir * (a[field] == b[field] ? 0 : a[field] > b[field] ? 1 : -1);
			}
			
			for(var idx = 0; idx < order.length; idx++) {
				var col = order[idx];
				var dir = col.ordering == 'desc' ? -1 : 1;
				var v = compare(a, b, dir, col.field.name);
				if(v != 0) return v;
			}
			return 0;
		});*/
		// return filtered;
		// End default sort
		
		// This is merge sort algorithm, might be more efficient
		var algo = function(a, b) {
			for(var idx = 0; idx < order.length; idx++) {
				var col = order[idx];
				var dir = col.ordering == 'desc' ? -1 : 1;
				var v = compare(a, b, dir, col.field.name);
				if(v != 0) return v;
			}
			return 0;
		};
		
		var sort = function(array) {
			var len = array.length;
			if(len < 2) 
				return array;

			var pivot = Math.ceil(len/2);
			return merge(sort(array.slice(0,pivot)), sort(array.slice(pivot)));
		};

		var merge = function(left, right) {
			var result = [];
			while((left.length > 0) && (right.length > 0)) {		
				if(algo(left[0], right[0]) > 0)
					result.push(left.shift());
				else
					result.push(right.shift());
			}

			result = result.concat(left, right);
			return result;
		};

		return sort(filtered);
		// End merge sort
	};
});

// Applies a filter on the list of items using three different sources:
//   1. search -- Search box
//   2. filter -- External filter object
//   3. columns -- Column filters
app.filter('objectFilter', function($log){
	return function(items, search, searchCols, filter, columns){
		if(items == undefined || items.length == 0) return items;
		
		var filtered = [];
		angular.forEach(items, function(item) {
			var include = true;

			// Filter by the search box text
			//   Search all of the searchCols if specified, or all cols if not
			//   Simple inexact match
			if(search != undefined && search.length > 0) {
				var searchInclude = false;
				
				// Search only specified columns
				if(searchCols.length > 0) {
					for(var idx = 0; idx < searchCols.length; idx++) {
						var col = searchCols[idx];
						if(item.data[col] != undefined && (item.data[col] + "").toLowerCase().indexOf(search.toLowerCase()) >= 0) {
							searchInclude = true;
							break;
						}
					}
				}
				// Otherwise search all columns
				else {
					for(var idx = 0; idx < Object.keys(item.data).length; idx++) {
						var col = Object.keys(item.data)[idx];
						if(col == 'tupleId') continue;
						if(item.data[col] != undefined && (item.data[col] + "").toLowerCase().indexOf(search.toLowerCase()) >= 0) {
							searchInclude = true;
							break;
						}
					}
				}
				include = searchInclude;
			}
			
			// Filter by external filter object
			//   Search all the columns specified as keys in the filter object
			//   Exact match
			//   For example:
			//     filter = {sku: '1234', color: 'blue'}
			//     returns only rows where sku=1234 and color=blue
			if(include == true && filter != undefined && Object.keys(filter).length > 0) {			
				var filterInclude = true;
				var filterKeys = Object.keys(filter);
				for(var idx = 0; idx < filterKeys.length; idx++) {
					if(item.data[filterKeys[idx]] != filter[filterKeys[idx]]) {
						filterInclude = false;
						break;
					}
				}
				include = filterInclude;
			}
			
			// Filter by column filter box
			//   Search all the columns with column-level filters applied
			//   Inexact match, also supports inequality operators
			//     >, <, >=, <=, ==, !=
			if(include == true) {
				var fieldSearchInclude = true;
				for(var idx = 0; idx < columns.length; idx++) {
					var column = columns[idx];
					if(column.search == undefined) continue;
					var d = item.data[column.field.name];
					var t = column.search;
					if(t.indexOf('>') == 0 || t.indexOf('<') == 0 || t.indexOf('!') == 0 || t.indexOf('==') == 0) {
						var expression = t;//.replace(/([.*+?^=!:${}()|\[\]\/\\])/g, "\\$1");
						var expression = expression
							.replace(new RegExp('>', 'g'), ' d >')
							.replace(new RegExp('<', 'g'), ' d <')
							.replace(new RegExp('!', 'g'), ' d !')
							.replace(new RegExp('==', 'g'), ' d ==');							
						//$log.debug(expression);
						if(d != undefined && eval(expression) == false) {
							fieldSearchInclude = false;
							break;
						}
					}
					else if((d + "").toLowerCase().indexOf(t) == -1) {
						fieldSearchInclude = false;
						break;
					}
				}			
				include = fieldSearchInclude;
			}
			
			// If include is still true, push the item into the return array
			if(include == true) filtered.push(item);
		});
		return filtered;
	};
});

// Applies a filter to limit returned data to pagination configuration
app.filter('objectLimitTo', function($log){
	return function(items, pagination){
		if(items == undefined || pagination == undefined) return items;
		var begin = pagination.begin == undefined ? 0 : pagination.begin;
		var end = pagination.end == undefined || pagination.end == 0 ? begin + pagination.size : pagination.end;
		return items.slice(begin, end);
	};
});

