app.service('Application', function($log, $rootScope) {
	return {
		show: function(nav) {
			$('#' + nav + '-app').show();
			$rootScope.$broadcast('app', nav);
		}		
	};
});

app.service('Overlay', function($log) {
	return {
		show: function(element) {
			var preloader = $(element).closest('.overlay-container').find('.preloader');
			if(preloader.length > 0) return;
			var parentContainer = $(element).closest('.overlay-container');
			if(parentContainer.length == 0) return;			
			var over = "<div class='preloader'></div>";
			$(over).appendTo(parentContainer.css("position", "relative"));	
		},	
		hide: function(element) {
			var preloader = $(element).closest('.overlay-container').find('.preloader');
			if(preloader.length == 0) return;
			preloader.remove();
		}
	};
});

app.service('Properties', function ($log, $rootScope) {
	var properties = {};

	return {
		getProperty: function (key) {
			return properties[key];
		},
		setProperty: function(key,value) {
			properties[key] = value;
			$rootScope.$broadcast(key, value);
		},
		getSessionProperty: function (key) {
			return window.sessionStorage[key];
		},
		setSessionProperty: function(key,value) {
			window.sessionStorage[key] = value;
		},
		getSessionObject: function (key) {
			return JSON.parse(window.sessionStorage[key]);
		},
		setSessionObject: function(key,value) {
			window.sessionStorage[key] = JSON.stringify(value);
		}		
	};
});

app.service('Clipboard', function($log) {
	return {
		copy: function(copyText, element) {
			var input = $(document.createElement('textarea')).val(copyText);
			$(element).append(input);
			input.select();
			
			try {
				var successful = document.execCommand('copy');
				var msg = successful ? 'successful' : 'unsuccessful';
				//console.log('Copying text command was ' + msg);
			} catch (err) {
				//console.log('Oops, unable to copy');
			}

			$(input).remove();				
			window.getSelection().removeAllRanges();
		},	
	};
});

app.service('EventListener', function($log, $window, $timeout) {
	return {
		windowResize: function(resize) {
			// Bind listener on window resize to adjust table heights for auto-sized tables
			angular.element($window).bind('resize', function(){
				var waitForFinalEvent = (function () {
					var timers = {};
					return function (callback, ms, uniqueId) {
						if (!uniqueId)
							uniqueId = "Don't call this twice without a uniqueId";
						if (timers[uniqueId])
							clearTimeout (timers[uniqueId]);
						timers[uniqueId] = setTimeout(callback, ms);
					};
				})();

				waitForFinalEvent(function(){
					if(resize != undefined) resize();
				}, 10, new Date().getTime() + "");
			});	
		},

		componentDisplay: function(element, display) {
			// This table may be in a tab. If it is, bind listener on document ready to redraw the table
			angular.element($(element).closest('.tab-pane')).ready(function() {
				// Delay setting the callback until document renders
				$timeout(function(){
					var parentId = $(element).closest('.tab-pane').attr('id');
					if(parentId != undefined && parentId != "{{thisQuery.id}}") {
						$('a[href=#' + parentId + ']').on('shown.bs.tab', function(e) {
							if(display != undefined) display();
						});
					}
				},100);
			});	
			
			// This table may be in a hideable application. If it is, bind listener on show of the application 
			//   to redraw the table
			angular.element($(element).closest('.application')).ready(function() {
				// Delay setting the callback until document renders
				$timeout(function(){
					// Listen for show events. This only works because of addition of custom jQuery handler for
					//   show/hide (see top of ng-directives.js.
					var application = $(element).closest('.application').on('show', function() {
						if(display != undefined)
							display();
					});
				},100);
			});	
			
			// This table may be in a hideable div. If it is, bind listener on show of the div 
			//   to redraw the table
			angular.element($(element).closest('.visibility-toggle')).ready(function() {
				// Delay setting the callback until document renders
				$timeout(function(){
					// Listen for show events. This only works because of addition of custom jQuery handler for
					//   show/hide (see top of ng-directives.js.
					var application = $(element).closest('.visibility-toggle').on('show', function() {
						if(display != undefined) display();
					});
					$(element).closest('.visibility-toggle').trigger("show");					
				},100);
			});			

			// This table may be in a an accordion pane. If it is, bind listener on show of the div 
			//   to redraw the table
			angular.element($(element).closest('.application')).ready(function() {
				// Delay setting the callback until document renders
				$timeout(function(){
					// Listen for show events. This only works because of addition of custom jQuery handler for
					//   show/hide (see top of ng-directives.js.
					var application = $(element).closest('.collapse').on('show.bs.collapse', function() {
						if(display != undefined)
							display();
					});
				},100);
			});	
			
		}
	};
});

app.service('SimpleDateFormat', function($log) {
	return {
		format: function(data, format) {			
			if(data == undefined) return "";				
			if(format == undefined) return data;
			
			var date = data;
			if(!(date instanceof Date)) {
				date = new Date(date);
				// Firefox hack
				//date = new Date(data.replace(' ', 'T'));
			}

			// Array of short month names
			var shortMonth = ['Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec'];	
			
			// Array of date format tokens
			var o = {
				"yyyy+" : 	date.getFullYear(),
				"yy+" : 	(date.getFullYear() + "").substring((date.getFullYear() + "").length - 2),
				"dd+" : 	(date.getDate() < 10 ? "0" : "") + date.getDate(),
				"HH+" : 	(date.getHours() < 10 ? "0" : "") + date.getHours(),
				"mm+" : 	(date.getMinutes() < 10 ? "0" : "") + date.getMinutes(),
				"ss+" : 	(date.getSeconds() < 10 ? "0" : "") + date.getSeconds(),
				"S" : 		(date.getMilliseconds() < 10 ? "00" : date.getMilliseconds() < 100 ? "0" : "") + date.getMilliseconds(),
				// Month has to come last and has to be in this order due to the string substitution
				"MMM+" : 	shortMonth[date.getMonth()],
				"MM+":		(date.getMonth() + 1 < 10 ? "0" : "") + (date.getMonth() + 1),
				"Z":		(date.getTimezoneOffset()),
			}

			var formatted = format;
			for(var k in o) {
				if(new RegExp("("+ k +")").test(formatted))
					formatted = formatted.replace(RegExp.$1,o[k]);
			}
				
			return formatted;
		}
	};
});

app.service('NumberFormat', function($log) {
	return {
		format: function(data, format) {		
			if(format === undefined || data === null || data === undefined)
				return data;

			var f = format;
			if(f.indexOf('.') == -1)
				f = f + '.';
			var parts = f.split('.');			
			if(parts.length == 2)
				return parseFloat(data).toFixed(parts[1].length);
			
			return data;
		}
	};
});

app.service('Modal', function($log, $rootScope, $controller, $compile, $document) {
	return {
		show: function(options) {
			// options.template -- template
			// options.templateUrl -- templateUrl
			// options.scope -- scope
			// options.inputContext -- inputs
			// options.appendElement -- append to element

			var self = this;
			
			// Load the template from the URL
			if(options.templateUrl != undefined) {
				$.ajax({
					type: 'GET',
					url: options.templateUrl,
					success: function(data, textStatus, xhr) {
						self.processTemplate(options, data);
					},
					error: function(xhr, textStatus, errorThrown) {
						$log.debug('err');
					}
				});	
			}
			else {
				self.processTemplate(options, options.template);
			}
		},
		
		processTemplate: function(options, template) {
			// Setup the scope
			var modalScope = options.scope;
			if(modalScope == undefined)
				modalScope = $rootScope.$new();

			// Add a close function to destroy the added modal
			modalScope.close = function() {
				$(modalElement).on('hidden.bs.modal', function (e) {
					modalScope.$destroy();
					modalElement.remove();
					if(modalScope.inputContext.onModalClose != undefined)
						modalScope.inputContext.onModalClose();
				})				
				modalElement.modal('hide');
			};

			// Add inputs to the scope
			if(options.inputContext != undefined) {  
				modalScope.inputContext = options.inputContext;
			}
			
			// Create the template, compile and link
			var modalElementTemplate = angular.element(template);  
			var linkFn = $compile(modalElementTemplate);  
			var modalElement = linkFn(modalScope); 
			
			// Get the append element, and append the modal
			var appendElement = options.appendElement;
			if(options.appendElement == undefined)
				appendElement = $document.find('body');
			appendElement.append(modalElement);

			// Create the modal object
			var modal = {  
				scope: modalScope,
				element: modalElement,
			}; 	

			// Show the modal, timeout required to render properly
			setTimeout(function() {
				modalElement.modal('show');
			},50);
				
			return modal;
		}
	}
});

app.service('Selection', function($log) {
	return {
		select: function(selectable, selectedItem, item) {
			if(selectable == false) return;				
			return this.changeSelectedItem(selectedItem, item, 'internal');
		},
		
		watchSelected: function(selectedItem, item) {
			return this.changeSelectedItem(selectedItem, item, 'watch');
		},
		
		// Change the selected item to the specified item
		changeSelectedItem: function(selectedItem, item, source) {
			// If selectedItem is currently undefined, and item is not undefined, 
			//   then just select the new item
			if(selectedItem == undefined) {
				if(item != undefined) {
					item.selected = true;
					item.update();
					return item;
				}
			}
			// Otherwise there is something selected already
			else {
				// If the item is undefined and source is watch, then unselect
				if(item == undefined && source == 'watch') {
					var prevItem = selectedItem;
					prevItem.selected = false;
					prevItem.update();
					return undefined;
				}
				// If the item is defined and the source is internal and _key
				//   matches, then unselect
				else if(item != undefined && source == 'internal' && item._key == selectedItem._key) {
					var prevItem = selectedItem;
					prevItem.selected = false;
					prevItem.update();
					return undefined;
				}
				// Otherwise if the item is defined and the _key does not match,
				//   then unselect the existing and select the new
				else if(item != undefined && item._key != selectedItem._key) {
					var prevItem = selectedItem;
					prevItem.selected = false;
					prevItem.update();
					item.selected = true;
					item.update();
					return item;
				}
			}
			
			return false;
		}
	}
});

app.service('ContextMenu', function($log) {
	return {
		show: function(e, element, menuClass, customMenuItems, conditionalMenuItems) {
			// Hide any context menus 
			$(document).find('.context-menu').css({display: 'none'});

			// Get the container and menu
			var container = $(element).parent();
			var menu = $(element).find(menuClass);
			
			// Remove any previously added custom elements
			$(menu).find('.custom-menu-item').remove();
				
			// Count the existing menu items
			var menuItemCount = $(menu).find('a').length;
			
			// Hide any conditional menu items
			if(conditionalMenuItems != undefined && conditionalMenuItems.length > 0) {
				for(var idx = 0; idx < conditionalMenuItems.length; idx++) {
					var conditionalMenuItem = conditionalMenuItems[idx];
					if(conditionalMenuItem.display == true)
						$(menu).find('.' + conditionalMenuItem.menuItemClass).show();
					else {
						$(menu).find('.' + conditionalMenuItem.menuItemClass).hide();
						menuItemCount = menuItemCount - 1;
					}
				}
			}
			
			// Add any custom menu items
			var divider = null;
			if(customMenuItems != undefined && customMenuItems.length > 0) {
				var customMenuItemCount = 0;
				
				// If there are existing menu items, add a divider
				if(menuItemCount > 0) {
					divider = $(document.createElement('li')).attr('class', 'divider custom-menu-item');
					$(menu).find('ul').append(divider);
				}
					
				// Loop over the custom menu items, and append
				for(var idx = 0; idx < customMenuItems.length; idx++) {
					var menuItem = customMenuItems[idx];
					var include = true;
					if(menuItem.display != undefined)
						include = menuItem.display();
					
					// If include is true, append the menu item
					if(include == true) {
						menuItemCount = menuItemCount + 1;
						customMenuItemCount = customMenuItemCount + 1;

						// Determine if a divider
						if(menuItem.divider == true) {
							var listItem = $(document.createElement('li')).attr('class', 'divider custom-menu-item');
							$(menu).find('ul').append(listItem);
						}
						// Otherwise create the list item and a href and append
						else {
							var listItem = $(document.createElement('li'));
							var ahref = $(document.createElement('a')).attr('tabindex', '-1').attr('href', '#').
								attr('class', 'custom-menu-item').attr('event-id', menuItem.eventId).
								text(menuItem.label);
							$(listItem).append(ahref);
							$(menu).find('ul').append(listItem);
							
							// Add click handler to emit an event
							$(listItem).click(function(e) {
								menuItem.action(e);
							});
						}
					}					
				}
				
				// If custom count is 0, remove the divider
				if(customMenuItemCount == 0) 
					$(divider).remove();				
			}
			
			// Final check to see if there is anything in the menu
			if(menuItemCount == 0) return undefined;

			// Calculate the position of the menu based on event location
			var x = e.clientX;
			var y = e.clientY;
			var width = menu.width();
			
			// Show the menu
			if(x + width > window.innerWidth)
				x = x - width;
			menu.css({position: 'fixed', left: x, top: y, display: 'block'});
			
			return menu;
		}
	}
});
