app.service('Map', function($log, $compile, SimpleDateFormat, NumberFormat) {
	return {

		// Creates a new popup
		createPopup: function(scope, content) {
			// DEFINE OBJECTS
			{
				function MapPopup(scope, content) {
					this.contentScope = scope.$new(true);
					this.contentScope.Math = window.Math;
					this.contentScope.formatDateTime = function(dateStr, format) {
						return SimpleDateFormat.format(dateStr, format);
					};
					this.contentScope.formatNumber = function(numStr, format) {
						return NumberFormat.format(numStr, format);
					};
					this.content = $(document.createElement('div'));
					this.content.empty().html(content);
					$compile(this.content.contents())(this.contentScope);
				};
				
				MapPopup.prototype.update = function(component) {
					this.contentScope.object = component.data;
					this.contentScope.component = component;
					component.node.bindPopup('');
					component.node._popup.setContent(this.content[0]);
				};
			}
			
			// Create and return a new popup
			return new MapPopup(scope, content);			
		},
		
		// Updates the popup content
		updatePopup: function(scope, component) {
			var popup = component.popup;
			if(popup == undefined) return;
			popup.contentScope.object = component.data;
			popup.contentScope.component = component;
			popup.update(component);
		},
		
		// Returns the color from available configuration
		getColor: function(options, property, object, component) {
			if(options[property] == undefined) return undefined;

			// Determine what kind of color definitions we have
			if(options[property].indexOf('{') == 0 && options[property].indexOf('}')  > -1) {
				var colorObj = eval('(' + options[property] + ')');
				var colors = Object.keys(colorObj);
				for(var idx = 0; idx < colors.length; idx++) {
					var color = colors[idx];
					var value = colorObj[color];
					if(value == true) return color;
				}
				return '#03f';
			}
			else
				return options[property];
		},

		// Returns the path options from the attributes
		getPathOptions: function(attrs) {
			var options = {
				"className":		attrs.classes == undefined || attrs.classes.length == 0 ?
										undefined :
										attrs.classes,
									// Optional. Class name to apply to the element.
								
				"clickable":		attrs.clickable == undefined || attrs.clickable.length == 0 ?
										true :
										eval(attrs.clickable),
									// Optional. Make the object clickable. 
								
				"stroke":			attrs.stroke == undefined || attrs.stroke.length == 0 ?
										true :
										eval(attrs.stroke),
									// Optional. Whether to draw the stroke along the path. Set to false to disable borders on polygons or circles.
								
				"color":			attrs.color == undefined || attrs.color.length == 0 ?
										'#03f' :
										attrs.color,
									// Optional. Stroke color. May be a constant or a conditional object:
									//   For example: {'red': object.status == 'Approach', 'yellow': object.status == 'Circling', 'green': true}: 
									//   Will give the color for the first expression that evaluates to true. Expression must evaluate to
									//   boolean and can use "object" to refer to the object being rendered.

								
				"dashArray":		attrs.dashArray == undefined || attrs.dashArray.length == 0 ?
										null :
										attrs.dashArray,
									// Optional. String that defines a stroke dash pattern.
									// Reference: https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/stroke-dasharray
								
				"lineCap":			attrs.lineCap == undefined || attrs.lineCap.length == 0 ?
										null :
										attrs.lineCap,
									// Optional. String that defines shape to use at the end of the stroke.
									// Reference: https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/stroke-linecap
								
				"lineJoin":			attrs.lineJoin == undefined || attrs.lineJoin.length == 0 ?
										null :
										attrs.lineJoin,
									// Optional. String that defines shape to use at corners of a stroke.
									// Reference: https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/stroke-linejoin
								
				"opacity":			attrs.opacity == undefined || attrs.opacity.length == 0 ?
										0.5 :
										eval(attrs.opacity),
									// Optional. Opacity for the line stroke.

				"weight":			attrs.weight == undefined || attrs.weight.length == 0 ?
										5 :
										attrs.weight,
									// Optional. Stroke width in pixels.

				"fill":				attrs.fill == undefined || attrs.fill.length == 0 ?
										false :
										eval(attrs.fill),
									// Optional. Whether to fill the path. Set to false to disable fill on polygons or circles.
								
				"fillColor":		attrs.fillColor == undefined || attrs.fillColor.length == 0 ?
										'#03f' :
										attrs.fillColor,
									// Optional. Fill color. May be a constant or a conditional object:
									//   For example: {'red': object.status == 'Approach', 'yellow': object.status == 'Circling', 'green': true}: 
									//   Will give the color for the first expression that evaluates to true. Expression must evaluate to
									//   boolean and can use "object" to refer to the object being rendered.
								
				"fillOpacity":		attrs.fillOpacity == undefined || attrs.fillOpacity.length == 0 ?
										0.2 :
										eval(attrs.fillOpacity),
									// Optional. Opacity for the fill.

				"zIndexOffset":		attrs.zIndexOffset == undefined || attrs.zIndexOffset.length == 0 ?
										undefined :
										attrs.zIndexOffset,
									// Optional. z-index offset within the SVG group.
			};
			return options;
		},
		
		// Returns the weight from available configuration
		getWeight: function(options, property, object, component) {
			if(options[property] == undefined) return undefined;

			// Determine what kind of weight definitions we have
			if(options[property].toString().indexOf('{') == 0 && options[property].toString().indexOf('}')  > -1) {
				var weightObj = eval('(' + options[property] + ')');
				var weights = Object.keys(weightObj);
				for(var idx = 0; idx < weights.length; idx++) {
					var weight = weights[idx];
					var value = weightObj[weight];
					if(value == true) return weight;
				}
				return 3;
			}
			else
				return options[property];
		},

		// Updates the component position and popup position, if open
		updatePosition: function(scope, component) {
			var object = component.data;
			var position = eval(scope.options.position);
			
			if(position[0] != null && position[1] != null) {
				var latLng = L.latLng(position);
				component.node.setLatLng(latLng);
				if(component.added == false) {
					if(component.visible == true) {
						component.layer.node.addLayer(component.node);
					}
					component.added = true;
				}
				if(component.node.getPopup() != undefined)
					component.node.getPopup().setLatLng(latLng);
			}
			else if(component.added == true) {
				component.layer.node.removeLayer(component.node);
				component.added = false;
			}
		},

		// Updates the component title
		updateTitle: function(scope, ctrl, component) {
			var object = component.data;
			component.title = eval(ctrl.getObjectTitleExpression());
		},
		
	};
});
