// This function adds show/hide handlers on div elements
(function ($) {
	$.each(['show', 'hide'], function (i, ev) {
		var el = $.fn[ev];
		$.fn[ev] = function () {
			this.trigger(ev);
			return el.apply(this, arguments);
		};
	});
})(jQuery);


// Hide any context menu when clicking anywhere on the document
angular.element(document).ready(function() {
	$(document).on("click contextmenu scroll", function(e){
		$('.context-menu').hide();
	});
});				


// Context menu directive
app.directive('ngContextMenu', function($log, $parse) {
    return function(scope, element, attrs) {
        var fn = $parse(attrs.ngContextMenu);
        element.bind('contextmenu', function(event) {
            scope.$apply(function() {
                //event.preventDefault();
                fn(scope, {$event:event});
            });
        });
    };
});


// Enter directive
app.directive('ngEnter', function ($log) {
	return function (scope, element, attrs) {
		element.bind("keydown keypress", function (event) {
			if(event.which === 13) {
				scope.$apply(function (){
					scope.$eval(attrs.ngEnter);
				});
				event.preventDefault();
			}
		});
	};
});


// For SVG and HTML element attributes to avoid late binding error in Javascript console
angular.forEach(['src', 'x', 'y', 'r', 'width', 'height', 'transform', 'stroke', 'fill'], function(name) {
  var ngName = 'ng' + name[0].toUpperCase() + name.slice(1);
  app.directive(ngName, function() {
    return function(scope, element, attrs) {
      attrs.$observe(ngName, function(value) {
        attrs.$set(name, value); 
      })
    };
  });
});


// Navigation
app.directive('navigation', function($log, Overlay) {
	return {
		templateUrl: '/framework/templates/navigation.html',
		replace: false,
		transclude: true,
		restrict: 'E',
		scope: {	
			datasets:	'=',
			navigator:	'='
		},
		controller: function($scope) {
		},
		link: function(scope, element, attrs, ctrl, transclude) {	
			// Set the element ID and name to scope if they exist
			if(attrs.id != undefined) scope.elementId = attrs.id;
			if(attrs.name != undefined) scope.elementName = attrs.name;
			
			// Add fill class to the directive element
			$(element).addClass('flex-fill-vertical');

			// Process the configuration
			var processConfiguration = function(data) {
				scope.configuration = data;
			};

			// Load configuration from the configUri and process
			$.ajax({
				type: 'GET',
				url: attrs.configUri,
				dataType: 'json',
				cache: false,
				async: false,
				success: function(data, textStatus, xhr) {
					processConfiguration(data);
				},
				error: function(xhr, textStatus, errorThrown) {
					$log.debug(xhr);
					scope.warning = "Navigation configuration uri not found or invalid";
				}
			});	
			
			// Click action on a nav item
			scope.click = function(e, item) {
				if(item.actions == undefined || item.actions.length == 0) return;
				for(var idx = 0; idx < item.actions.length; idx++) {
					var thisAction = item.actions[idx];
					if(thisAction.action == 'navigate')
						scope.navigate(thisAction.navigate);
					else if(thisAction.action == 'parameters')
						scope.parameters(thisAction, item);
					else if(thisAction.action == 'filter') 
						scope.filter(thisAction, item);
					else if(thisAction.action == 'marked') 
						scope.mark(thisAction, item);
				}
			}
		
			// Retrieve the dataset for the given id
			var getDataset = function(id) {
				for(var idx = 0; idx < scope.datasets.length; idx++) {
					var thisDataset = scope.datasets[idx];
					if(thisDataset.id == id) 
						return thisDataset;
				}
				return null;
			};
			
			// Navigate to a target
			scope.navigate = function(targetId) {
				$('a[href="#' + targetId + '"]').tab('show');
				$('a[href="#' + targetId + '"]').addClass('active');
				scope.navigator.target = targetId;
				scope.$applyAsync();
			}
			
			// Apply parameters
			scope.parameters = function(action, item) {
				if(scope.datasets == undefined) return;
				var thisDataset = getDataset(action.target);
				if(thisDataset == null) return;
				
				var targetParameters = {};
				for(var parameterIdx = 0; parameterIdx < Object.keys(action.parameters).length; parameterIdx++) {
					var parameterName = Object.keys(action.parameters)[parameterIdx];
					var parameterExpression = action.parameters[parameterName];
					var parameterValue = eval(parameterExpression);
					targetParameters[parameterName] = parameterValue;
				}
				thisDataset.applyParameters(targetParameters);
			}
			
			// Apply filter
			scope.filter = function(action, item) {
				if(scope.datasets == undefined) return;
				var thisDataset = getDataset(action.target);
				if(thisDataset == null) return;

				thisDataset.trigger('filter', this, [action.filter]);
			};

			// Mark objects
			scope.mark = function(action, item) {
				if(scope.datasets == undefined) return;
				var thisDataset = getDataset(action.target);
				if(thisDataset == null) return;

				// Loop over the target dataset and apply conditions
				var targetObjects = [];
				for(var datasetIdx = 0; datasetIdx < thisDataset.data.length; datasetIdx++) {
					var target = thisDataset.data[datasetIdx];
					var result = eval(action.condition);
					if(result == true)
						targetObjects.push(target);
				}
				
				thisDataset.trigger('marked', this, targetObjects);
			};
		
			// If there is a navigator object, bind a function to change navigation
			if(scope.navigator != undefined) {
				scope.navigator.navigate = scope.navigate;
			}
			
			// If there is an initial value, navigate to that now
			if(scope.configuration.navigation.initial != undefined) {
				setTimeout(function() {
					scope.navigate(scope.configuration.navigation.initial);
				},500);
			}
			
		}
		
	};	
});
