app.directive('liveGrid', function($log, Overlay) {
	return {
		templateUrl: '/framework/templates/live-grid.html',
		replace: false,
		transclude: true,
		restrict: 'E',
		scope: {
			dataset: 	'=',
		},
		controller: function($scope) {
		},
		link: function(scope, element, attrs, ctrl, transclude) {	
			// Set the element ID and name to scope if they exist
			if(attrs.id != undefined) scope.elementId = attrs.id;
			if(attrs.name != undefined) scope.elementName = attrs.name;
			
			// Add fill class to the directive element
			$(element).addClass('flex-fill-vertical');

			// Load configuration from the configUri and process
			$.ajax({
				type: 'GET',
				url: attrs.configUri,
				dataType: 'json',
				cache: false,
				success: function(data, textStatus, xhr) {
					processConfiguration(data);
				},
				error: function(xhr, textStatus, errorThrown) {
					$log.debug(xhr);
					scope.warning = "Grid configuration uri not found or invalid";
				}
			});	

			
			// Process the configuration
			var processConfiguration = function(data) {
				// Process the configuration and set on the scope
				scope.configuration = DataFlow.LiveGrid.GridConfiguration.processConfiguration(data);
				
				// Watch the dataset for creation, once created stop watching and
				//   register the new sink. Register listeners and watchers.
				var watchDS = scope.$watch('dataset', function() {
					if(scope.dataset == undefined) return;
					watchDS();					
					registerSink();
				});
			};
			
			// REGISTER			
			
			// Register the sink on the dataset
			var registerSink = function() {
				// Get the grid container div to hold the grid
				var containerDiv = $(element).find('.grid-container')[0];

				// Create the data sink and register on the dataset
				try {
					var id = 'LiveGrid.Datasink' + (scope.elementId != undefined ? '[' + scope.elementId + ']' : '');
					scope.sink = new DataFlow.GridDatasink(id, scope.configuration, scope.dataset, containerDiv, overlay);
					scope.grid = scope.sink.grid;
					scope.dataset.registerSink(scope.sink);
				}
				catch(err) {
					scope.warning = err.message;
					console.log(err);
				}
			};
						
			// MISCELLANEOUS
			// Overlay handler
			scope.overlayCount = 0;
			var overlay = function(flag) {
				if(flag == true) 
					scope.overlayCount = scope.overlayCount + 1;
				else
					scope.overlayCount = scope.overlayCount - 1;
				if(scope.overlayCount > 0)
					Overlay.show(element);
				else
					Overlay.hide(element);				
			};
			
		}
	};	
});
