app.directive('liveLeafletMap', function($log, Overlay) {
	return {
		templateUrl: '/framework/templates/live-map.html',
		replace: false,
		transclude: true,
		restrict: 'E',
		scope: true,
		controller: function($scope) {
		},
		link: function(scope, element, attrs, ctrl, transclude) {	
			// Set the element ID and name to scope if they exist
			if(attrs.id != undefined) scope.elementId = attrs.id;
			if(attrs.name != undefined) scope.elementName = attrs.name;
			
			// Add fill class to the directive element
			$(element).addClass('flex-fill-vertical');

			// Extract the layer to dataset mappings
			var datasetMap = {};
			for(var idx = 0; idx < Object.keys(attrs).length; idx++) {
				var key = Object.keys(attrs)[idx];
				if(key.startsWith('dataset')) {
					datasetMap[key.substring('dataset'.length)] = attrs[key];
				}
			}
			
			// Load configuration
			$.ajax({
				type: 'GET',
				url: attrs.configUri,
				dataType: 'json',
				cache: false,
				success: function(data, textStatus, xhr) {
					processConfiguration(data);
				},
				error: function(xhr, textStatus, errorThrown) {
					$log.debug(xhr);
					scope.warning = "Leaflet Map configuration uri not found or invalid";
				}
			});	

			// Process the configuration
			var processConfiguration = function(data) {
				// Set the configuration on the scope
				scope.configuration = DataFlow.LiveLeafletMap.MapConfiguration.processConfiguration(data);

				// Create the map object
				var id = "LiveLeafletMap.Map" + (scope.elementId != undefined ? '[' + scope.elementId + ']' : '');
				var containerDiv = $(element).find('.map-container')[0];
				var map = new DataFlow.LiveLeafletMap.Map(id, scope.configuration, containerDiv);
				
				var watchDS = scope.$watch(function() {
						var result = true;
						for(var id in datasetMap) {
							if(scope[datasetMap[id]] == undefined) return false;
						}
						return result;
					}, 
					function(result) {
						if(result == false) return;
						watchDS();
						for(var id in datasetMap)
							registerSink(id, scope[datasetMap[id]], map);
					}
				);
			};
			
			// REGISTER			
			
			// Register the sink on the dataset
			var registerSink = function(id, dataset, map) {
				// Create the data sink and register on the dataset
				try {
					var thisId = 'LiveLeafletLayer.Datasink.' + id + (scope.elementId != undefined ? '[' + scope.elementId + ']' : '');
					var thisConfiguration = scope.configuration.leaflet.layers[id];
					if(thisConfiguration == undefined) return;
					var sink = new DataFlow.LeafletLayerDatasink(thisId, thisConfiguration, dataset, map, overlay);
					dataset.registerSink(sink);
				}
				catch(err) {
					scope.warning = err.message;
					console.log(err);
				}
			};
						
			// MISCELLANEOUS
			// Overlay handler
			scope.overlayCount = 0;
			var overlay = function(flag) {
				if(flag == true) 
					scope.overlayCount = scope.overlayCount + 1;
				else
					scope.overlayCount = scope.overlayCount - 1;
				if(scope.overlayCount > 0)
					Overlay.show(element);
				else
					Overlay.hide(element);				
			};
			
		}
	};	
});


