app.directive('livePassthrough', function($log, $compile, Overlay) {
	return {
		templateUrl: '/framework/templates/live-passthrough.html',
		replace: false,
		transclude: true,
		restrict: 'E',
		scope: {
			dataset: 	'=',
		},
		controller: function($scope) {
		},
		link: function(scope, element, attrs, ctrl, transclude) {	
			// Set the element ID and name to scope if they exist
			if(attrs.id != undefined) scope.elementId = attrs.id;
			if(attrs.name != undefined) scope.elementName = attrs.name;
			
			// Add fill class to the directive element
			$(element).addClass('flex-fill-vertical');

			// Load configuration from the configUri and process
			$.ajax({
				type: 'GET',
				url: attrs.configUri,
				dataType: 'json',
				cache: false,
				success: function(data, textStatus, xhr) {
					processConfiguration(data);
				},
				error: function(xhr, textStatus, errorThrown) {
					$log.debug(xhr);
					scope.warning = "Passthrough configuration uri not found or invalid";
				}
			});	

			// Process the configuration
			var processConfiguration = function(data) {
				// Set the configuration on the scope
				scope.configuration = DataFlow.LivePassthrough.PassthroughConfiguration.processConfiguration(data);
				
				// Watch the dataset for creation, once created stop watching and
				//   register the new sink. Register listeners and watchers.
				var watchDS = scope.$watch('dataset', function() {
					if(scope.dataset == undefined) return;
					watchDS();					
					registerSink();
				});
			};
			
			// REGISTER			
			
			// Register the sink on the dataset
			var registerSink = function() {
				// Get the passthrough container div to hold the passthrough
				var containerDiv = $(element).find('.passthrough-container')[0];

				// Create the data sink and register on the dataset
				try {
					var id = 'LivePassthrough.Datasink' + (scope.elementId != undefined ? '[' + scope.elementId + ']' : '');
					scope.sink = new DataFlow.PassthroughDatasink(id, scope.configuration, scope.dataset, containerDiv, overlay, function(){scope.$apply();});
					scope.passthrough = scope.sink.passthrough;
					scope.format = function(val, format) {
						return scope.passthrough.format(val, format);
					}
					
					scope.colormap = function(val, name) {
						return scope.passthrough.colormap(val, name);
					}
					
					scope.dataset.registerSink(scope.sink);
				}
				catch(err) {
					scope.warning = err.message;
					console.log(err);
				}
			};
						
			// MISCELLANEOUS
			// Overlay handler
			scope.overlayCount = 0;
			var overlay = function(flag) {
				if(flag == true) 
					scope.overlayCount = scope.overlayCount + 1;
				else
					scope.overlayCount = scope.overlayCount - 1;
				if(scope.overlayCount > 0)
					Overlay.show(element);
				else
					Overlay.hide(element);				
			};
			
			// Forms object to hold form data
			scope.forms = {};
			
			// Send tuple
			scope.sendTuple = function(uri, inputStream, object) {
				DataFlow.LiveViewTupleDatasource.sendTuple(new Date(), uri, inputStream, object);
			};

			// Reset the specified form or all forms
			scope.resetForm = function(form) {
				if(form == undefined)
					scope.forms = {};
				else
					scope.forms[form] = undefined;
			};
		}
	};	
});
