app.directive('livePlotlyChart', function($log, Overlay, EventListener) {
	return {
		templateUrl: '/framework/templates/live-chart.html',
		replace: false,
		transclude: true,
		restrict: 'E',
		scope: {
			dataset: 	'=',
		},
		controller: function($scope) {
		},
		link: function(scope, element, attrs, ctrl, transclude) {		
			// Set the element ID and name to scope if they exist
			if(attrs.id != undefined) scope.elementId = attrs.id;
			if(attrs.name != undefined) scope.elementName = attrs.name;
			
			// Add fill class to the directive element
			$(element).addClass('flex-fill-vertical');

			// Load configuration
			$.ajax({
				type: 'GET',
				url: attrs.configUri,
				dataType: 'json',
				cache: false,
				success: function(data, textStatus, xhr) {
					processConfiguration(data);
				},
				error: function(xhr, textStatus, errorThrown) {
					$log.error(errorThrown);
					scope.warning = "Chart configuration uri not found or invalid";
				}
			});	

			// Process the configuration
			var processConfiguration = function(data) {
				// Set the configuration on the scope
				scope.configuration = DataFlow.LivePlotlyChart.PlotlyConfiguration.processConfiguration(data);
				
				// Watch the dataset for creation, once created stop watching and
				//   register the new sink. Register listeners and watchers.
				var watchDS = scope.$watch('dataset', function() {
					if(scope.dataset == undefined) return;
					watchDS();					
					registerSink();
				});
			};
			
			// Register the sink on the dataset
			var registerSink = function() {
				// Get the chart container div to hold the chart
				var containerDiv = $(element).find('.chart-container')[0];

				// Create the data sink and register on the dataset
				try {
					var id = 'LivePlotlyChart.Datasink' + (scope.elementId != undefined ? '[' + scope.elementId + ']' : '');
					scope.sink = new DataFlow.PlotlyChartDatasink(id, scope.configuration, scope.dataset, containerDiv, overlay);
					scope.chart = scope.sink.chart;
					scope.dataset.registerSink(scope.sink);
				}
				catch(err) {
					scope.warning = err.message;
					console.log(err);
				}
			};
						
			// MISCELLANEOUS
			// Overlay handler
			scope.overlayCount = 0;
			var overlay = function(flag) {
				if(flag == true) 
					scope.overlayCount = scope.overlayCount + 1;
				else
					scope.overlayCount = scope.overlayCount - 1;
				if(scope.overlayCount > 0)
					Overlay.show(element);
				else
					Overlay.hide(element);				
			};

			
			// Bind listener on window resize to reflow chart
			EventListener.windowResize(function() {
				if(scope.sink == null || scope.sink.chart == null) return;
				setTimeout(function() {
					scope.sink.chart.resize();
				},0);
			});
			
			// Bind listener on component display for tabs, applications, and accordion divs
			EventListener.componentDisplay(element, function() {
				if(scope.sink == null || scope.sink.chart == null) return;
				setTimeout(function() {
					scope.sink.chart.resize();
				},100);
			});
					
		}
	};	
});
