app.service('Overlay', function($log) {
	return {
		show: function(element) {
			var preloader = $(element).closest('.overlay-container').find('.preloader');
			if(preloader.length > 0) return;
			var parentContainer = $(element).closest('.overlay-container');
			if(parentContainer.length == 0) return;			
			var over = "<div class='preloader'></div>";
			$(over).appendTo(parentContainer.css("position", "relative"));	
		},	
		hide: function(element) {
			var preloader = $(element).closest('.overlay-container').find('.preloader');
			if(preloader.length == 0) return;
			preloader.remove();
		}
	};
});

app.service('EventListener', function($log, $window, $timeout) {
	return {
		windowResize: function(resize) {
			// Bind listener on window resize to adjust table heights for auto-sized tables
			angular.element($window).bind('resize', function(){
				var waitForFinalEvent = (function () {
					var timers = {};
					return function (callback, ms, uniqueId) {
						if (!uniqueId)
							uniqueId = "Don't call this twice without a uniqueId";
						if (timers[uniqueId])
							clearTimeout (timers[uniqueId]);
						timers[uniqueId] = setTimeout(callback, ms);
					};
				})();

				waitForFinalEvent(function(){
					if(resize != undefined) resize();
				}, 10, new Date().getTime() + "");
			});	
		},

		componentDisplay: function(element, display) {
			// This table may be in a tab. If it is, bind listener on document ready to redraw the table
			angular.element($(element).closest('.tab-pane')).ready(function() {
				// Delay setting the callback until document renders
				$timeout(function(){
					var parentId = $(element).closest('.tab-pane').attr('id');
					if(parentId != undefined) {
						$('a[href=#' + parentId + ']').on('shown.bs.tab', function(e) {
							if(display != undefined) display();
						});
					}
				},100);
			});	
			
			// This table may be in a hideable application. If it is, bind listener on show of the application 
			//   to redraw the table
			angular.element($(element).closest('.application')).ready(function() {
				// Delay setting the callback until document renders
				$timeout(function(){
					// Listen for show events. This only works because of addition of custom jQuery handler for
					//   show/hide (see top of ng-directives.js.
					var application = $(element).closest('.application').on('show', function() {
						if(display != undefined)
							display();
					});
				},100);
			});	
			
			// This table may be in a an accordion pane. If it is, bind listener on show of the div 
			//   to redraw the table
			angular.element($(element).closest('.application')).ready(function() {
				// Delay setting the callback until document renders
				$timeout(function(){
					// Listen for show events. This only works because of addition of custom jQuery handler for
					//   show/hide (see top of ng-directives.js.
					var application = $(element).closest('.collapse').on('show.bs.collapse', function() {
						if(display != undefined)
							display();
					});
				},100);
			});	
			
		}
	};
});
