/**
 * Color map returns a consistent color for a given value from
 * an array of colors. 
 * The colorscale can be specified as one of three ways:
 *   null - then a default color scale will be used
 *   object - defines mappings where the key is the field value and
 *     the value is the color to return for that value
 *   array - defines an array of color string
 * @constructor 
 * @param {String} name - name of the color map
 * @param {Object|String[]} colors - optional, object defining a mapping, or
 *                            and array of color strings, or null
 */
function ColorMap(name, colors) {
	var defaultColorScale = ['#1f77b4','#aec7e8','#ff7f0e','#ffbb78','#2ca02c','#98df8a',
		'#d62728','#ff9896','#9467bd','#c5b0d5','#8c564b','#c49c94','#e377c2','#f7b6d2',
		'#7f7f7f','#c7c7c7','#bcbd22','#dbdb8d','#17becf','#9edae5'];
	
	this.name 		= name;
	this.mode		= colors == null || Array.isArray(colors) ? 'scale' : 'mapping';

	this.colorscale	= null;
	this.colorIdx 	= 0;
	this.map 		= {};

	if(this.mode == 'scale')
		this.colorscale = colors == null || colors.length == 0 ? defaultColorScale : colors;
	else if(this.mode == 'mapping') {
		for(var key in colors) {
			this.map[key] = colors[key];
		}
	}
};

/**
 * Returns a color for the specified value. If the value was never requested
 * previously then the color map will return the next color in the scale.
 * Once the color scale has been exhausted, it will reset back to the first
 * element and duplicate colors will be returned. If the value was previously
 * requested then that color will be returned again.
 * @param {Object} val - the value to retrieve, typically a string but can be 
 *                   any data type
 * @return {String} a color
 */
ColorMap.prototype.getColor = function(val) {
	var color = this.map[val];
	if(color == undefined && this.mode == 'scale') {
		color = this.colorscale[this.colorIdx++];
		if(this.colorIdx == this.colorscale.length) {
			this.colorIdx = 0;
		}
		this.map[val] = color; 
	}
	return color;
};

