/**
 * Represents a schema for a dataset or a datasource.
 * @constructor
 */
function DatasetSchema() {
	this.fields 	= [];
	this.fieldMap 	= {};
};

/**
 * Adds a new field to the schema. If the field with the given name
 * already exists then the type will be updated. Otherwise a new field
 * will be added.
 * @param {String} name - field name
 * @param {String} type - field data type
 */
DatasetSchema.prototype.addField = function(name, type) {
	// Retrieve the existing field.
	var field = this.fieldMap[name];
	
	// If field is not found, create it and add to the schema
	if(field == undefined) {
		field = {
			name:	name,
			type:	type
		};
		this.fieldMap[name] = field;
		this.fields.push(field);
	}
	// Otherwise update the type
	else {
		field.type = type;
	}
};