/**
 * Represents the interface definition for a datasink object. This class should not 
 * be used for an implementation but rather as a reference for sink definition.
 * @constructor
 * @param {Object} dataset - dataset that this sink will be attached to
 * @param {Object} configuration - configuration object defining the sink
 */
function AbstractDatasink(configuration, dataset) {
	this.dataset 	= dataset;
	this.id 		= configuration.id;
}

/**
 * Event handler for dataset events. 
 * @param {String} eventType - string describing the event that occurred
 * @param {Object} object - data object for the event, or null
 */
AbstractDatasink.prototype.trigger = function(eventType, object) {
	// Datasource event types that must be handled:
	//   schemaChange
	//   snapshotStart
	//   snapshotEnd
	//   clear
	//   insert
	//   update
	//   upsert
	//   delete
	
	// Datasink event types that must be handled:
	//   hover
	//   marked
	this.didEvent[eventType] = true;
	
	if(eventType == 'schemaChange') {
		console.log('AbstractDatasink ' + this.id + ' schema change');
		var schema = object;
		//console.log(schema);
	}
	else if(eventType == 'snapshotStart') {
		console.log('AbstractDatasink ' + this.id + ' snapshot start');
	}
	else if(eventType == 'snapshotEnd') {
		console.log('AbstractDatasink ' + this.id + ' snapshot end');
	}
	else if(eventType == 'clear') {
		console.log('AbstractDatasink ' + this.id + ' clear');
	}
	else if(eventType == 'insert') {
		console.log('AbstractDatasink ' + this.id + ' insert');
		console.log(object);
	}
	else if(eventType == 'update') {
		console.log('AbstractDatasink ' + this.id + ' update');
		console.log(object);
	}
	else if(eventType == 'upsert') {
		console.log('AbstractDatasink ' + this.id + ' upsert');
		console.log(object);
	}
	else if(eventType == 'delete') {
		console.log('AbstractDatasink ' + this.id + ' delete');
		console.log(object);
	}
	else if(eventType == 'hover') {
		console.log('AbstractDatasink ' + this.id + ' hover');
		console.log(object);
	}
	else if(eventType == 'marked') {
		console.log('AbstractDatasink ' + this.id + ' marked');
		console.log(object);
	}
};