;(function(angular){
	// TODO Start Change
	var moduleName 			= 'com.tibco.sb.ldm.web.plugins.livehighcharts';
	var directiveName		= 'liveHighchartsChartVisDisplay';
	// END Change
	
	
	
	'use strict';
	angular
		.module(moduleName)
		.directive(directiveName, VisDisplay);

	VisDisplay.$inject = ['$log', '$http', 'VisualizationContract'];

	function VisDisplay($log, $http, VisualizationContract){

		var logger = $log.getInstance('/' + moduleName + '/vis');

		VisCtrl.$inject = ['$scope', '$element'];
		return {
			restrict: 'A',
			templateUrl: 'plugins/' + moduleName + '/vis.tpl.html',
			scope: false,
			controller: VisCtrl,
			controllerAs: 'vm', 			//the controller instance will be referred to as "vm" in the vis.tpl.html file
			bindToController: true			//scope values will be bound to the controller instance
		};

		function VisCtrl($scope, $element){
			var vm = this;				//Maintain a pointer to the controller instance, named after the "controllerAs" attribute of the directive		
			var dataUnsubscriberFunc;	//The function which is to be called to unsubscribe a data subscription
			
			// Create a dataset to handle sink -> source events from the datasink
			var dataset = {
				trigger:	function(eventType, sink, object) {
					if(eventType == 'hover') {

					}
					else if(eventType == 'marked') {
						var thisObject = null;
						if(object != null) {
							if(Array.isArray(object) == true && object.length > 0)
								thisObject = object[0];
							else if(Array.isArray(object) == false)
								thisObject = object;
						}
						$scope.contract.handleDataSelected($scope.visualization.config.dataSourceId, {fieldMap: thisObject == null ? getEmptyObject() : thisObject});
					}					
					else if(eventType == 'filter') {

					}					
				}
			};

			// Retrieve the container div for the visualization
			var containerDiv = $element.select('.chart-container')[0];
			
			// Chart resize on window resize
			var resizer;
			$scope.rootElement.resize(function () {
				clearTimeout(resizer);
				resizer = setTimeout(function() {
					if(vm.sink == undefined || vm.sink.chart == undefined) return;
					vm.sink.chart.resize();
				}, 0);
			});
			
			// Add action on destroy of the directive
			$scope.$on('$destroy', handleDirectiveDestroy);

			// Map callback functions to the appropriate contract properties
			$scope.contract.onConfigChanged = onConfigChanged;
			$scope.contract.onDataSourceChange = onDataSourceChange;
			$scope.contract.onSchemaSet = onSchemaSet;
			$scope.contract.onData = onData;

			// Call initial config changed
			onConfigChanged();

			
			// Validate and initialize
			
			// Validate the configuration object
			function validateConfiguration() {
				var config = $scope.visualization.config;
				var result = {
					uri: angular.isString(config.uri) && config.uri.length > 0,
					json: angular.isString(config.json) && config.json.length > 0
				};
				if(!result.uri && !result.json) {
					vm.warning = "Configuration URI or JSON are not set";
				} else {
					vm.warning = undefined;
				}
				return result;
			}

			// Initialize with the new configuration object
			function init(config) {
				// Process configuration and adjust defaults for LV Web card
				var configuration = DataFlow.LiveHighchartsChart.HighchartsConfiguration.processConfiguration(config);	
				configuration.highcharts.credits = {enabled: false};
				configuration.highcharts.exporting = {enabled: false};
				configuration.options.marking.mode = 'single';
				
				// Set configuration on scope
				$scope.configuration = configuration;
				logger.log('onConfigChange.init - Initializing with configuration:', config);
				
				// Create the datasink and clear any data present
				vm.sink = new DataFlow.HighchartsChartDatasink('LiveHighchartsChart.Datasink', configuration, dataset, containerDiv);
				vm.data = {};
				
				// Create unsubscriber function
				dataUnsubscriberFunc = $scope.contract.subscribeToData($scope.visualization.config.dataSourceId);
			}

			// Get an empty object
			function getEmptyObject() {
				var object = {};
				for(var idx = 0; idx < vm.schema.fields.length; idx++) {
					var thisField = vm.schema.fields[idx];
					object[thisField.name] = null;
				}
				return object;
			}
			
			
			// Configuration change
			function onConfigChanged(){
				//console.log('onConfigChanged');
				var valid = validateConfiguration();
				var configuration;

				if ((valid.json || valid.uri) && angular.isFunction(dataUnsubscriberFunc)) {
					dataUnsubscriberFunc();
				}

				// Prioritize JSON over URI
				if(valid.json) {
					try {
						configuration = JSON.parse($scope.visualization.config.json);
						return init(configuration);
					}
					catch(err) {
						return handleFail(err.message);
					}
				}
				else if(valid.uri) {
					$http({method: 'GET', url: $scope.visualization.config.uri})
						.then(
							function successCallback(response) {
								init(response.data);
							},
							function errorCallback(response) {
								handleFail("Chart configuration uri not found or invalid");
							}
						).catch(handleFail.bind(undefined, "Chart configuration uri not found or invalid"));
				}

				function handleFail(message) {
					logger.warn('onConfigChange.handleFail - ' + message);
					vm.warning = message;
				}
			}

			// DataSource change
			function onDataSourceChange(addedDataIds, updatedDataIds, deletedDataIds){
				//console.log('onDataSourceChange');
			}

			// Schema set
			function onSchemaSet(dataSourceId, queryEventData){
				if(queryEventData.schema.fields == undefined) return;

				// Clear existing data
				vm.sink.trigger('clear');
				vm.data = {};
				
				// Set the schema to the controller
				vm.schema = queryEventData.schema;

				// Create the sink schema and trigger an event
				var schema = new DataFlow.DatasetSchema();
				for(var idx = 0; idx < queryEventData.schema.fields.length; idx++) {
					var thisField = queryEventData.schema.fields[idx];
					schema.addField(thisField.name, thisField.type);
				}
				vm.sink.trigger('schemaChange', schema)
			}

			// Data event, dispatch to appropriate data handler
			function onData(dataSourceId, dataStore, dataEvents){
				for(var i = 0; i < dataEvents.length; i++){
					logger.log('onData: ', dataEvents[i]);
					if(dataEvents[i].type === VisualizationContract.DataEvent.Types.ADD){
						handleNewData(dataEvents[i].data);
					}
					else if(dataEvents[i].type === VisualizationContract.DataEvent.Types.UPDATE){
						handleUpdateData(dataEvents[i].data, dataEvents[i].affectedPositions.oldValues);
					}
					else if(dataEvents[i].type === VisualizationContract.DataEvent.Types.DELETE){
						handleDeletedData(dataEvents[i].data);
					}
				}
			}

			function handleNewData(newData){
				var object = {_key: newData.id};
				angular.extend(object, newData.fieldMap);
				vm.data[object._key] = object;
				vm.sink.trigger('insert', object);
			}

			function handleUpdateData(updateData, oldValues){
				var object = vm.data[updateData.id];
				if(object == undefined) return;
				angular.extend(object, updateData.fieldMap);
				vm.sink.trigger('update', object);
			}

			function handleDeletedData(deleteData){
				var object = vm.data[deleteData.id];
				if(object == undefined) return;
				vm.sink.trigger('delete', object);
				delete vm.data[deleteData.id];
			}

			function handleDirectiveDestroy(event){
				//do visualization clean up and unsubscribe from all subscribed data sources
				dataUnsubscriberFunc();
				vm = null;
				dataUnsubscriberFunc = null;
			}

		}

	}

})(angular);