package com.streambase.mqtt.broker;

import static java.nio.charset.StandardCharsets.UTF_8;

import com.streambase.mqtt.broker.MoquetteMQTTBroker.PayloadType;
import com.streambase.sb.ByteArrayView;
import com.streambase.sb.CompleteDataType;
import com.streambase.sb.Schema;
import com.streambase.sb.StreamBaseException;
import com.streambase.sb.Tuple;
import com.streambase.sb.TupleException;

import io.moquette.interception.AbstractInterceptHandler;
import io.moquette.interception.messages.InterceptAcknowledgedMessage;
import io.moquette.interception.messages.InterceptConnectMessage;
import io.moquette.interception.messages.InterceptConnectionLostMessage;
import io.moquette.interception.messages.InterceptDisconnectMessage;
import io.moquette.interception.messages.InterceptPublishMessage;
import io.moquette.interception.messages.InterceptSubscribeMessage;
import io.moquette.interception.messages.InterceptUnsubscribeMessage;
import io.netty.buffer.ByteBuf;

class PublisherListener extends AbstractInterceptHandler {
	
	public final static String FIELD_TOPIC = "Topic";
	public final static String FIELD_QOS = "Qos";
	public final static String FIELD_PAYLOAD = "Payload";
	public final static String FIELD_CLIENT_ID = "ClientId";
	public final static String FIELD_KEEP_ALIVE = "KeepAlive";
	public final static String FIELD_PASSWORD = "Password";
	public final static String FIELD_PROTOCOL_NAME = "ProtocolName";
	public final static String FIELD_PROTOCOL_VERSION = "ProtocolVersion";
	public final static String FIELD_WILL_MESSAGE = "WillMessage";
	public final static String FIELD_WILL_QOS = "WillQos";
	public final static String FIELD_WILL_TOPIC = "WillTopic";
	public final static String FIELD_IS_CLEAN_SESSION = "IsCleanSession";
	public final static String FIELD_IS_DUP_FLAG = "IsDupFlag";
	public final static String FIELD_IS_PASSWORD_FLAG = "IsPasswordFlag";
	public final static String FIELD_IS_RETAIN_FLAG = "IsRetainFlag";
	public final static String FIELD_IS_USER_FLAG = "IsUserFlag";
	public final static String FIELD_IS_WILL_FLAG = "IsWillFlag";
	public final static String FIELD_IS_WILL_RETAIN = "IsWillRetain";
	public final static String FIELD_REQUSTED_QOS = "RequestedQos";
	public final static String FIELD_TOPIC_FILTER = "TopicFilter";
	public final static String FIELD_USERNAME = "Username";
	public final static String FIELD_MESSAGE = "Message";
	public final static String FIELD_GUID = "GUID";
	public final static String FIELD_PACKET_ID = "PacketId";
	
	private MoquetteMQTTBroker moquetteMQTTBroker;
	
	public PublisherListener(MoquetteMQTTBroker moquetteMQTTBroker) {
		this.moquetteMQTTBroker = moquetteMQTTBroker;		
	}
	
    @Override
    public String getID() {
        return "SBEmbeddedPublishListener";
    }

    public static Schema createPublishSchema(PayloadType payloadType) {
    	return new Schema("", new Schema.Field[] {
				new Schema.Field(FIELD_TOPIC, CompleteDataType.forString()),
				new Schema.Field(FIELD_QOS, CompleteDataType.forInt()),
				new Schema.Field(FIELD_PAYLOAD, payloadType.equals(PayloadType.STRING) ? CompleteDataType.forString() : CompleteDataType.forBlob()),
				new Schema.Field(FIELD_CLIENT_ID, CompleteDataType.forString()),
				new Schema.Field(FIELD_USERNAME, CompleteDataType.forString()),
				new Schema.Field(FIELD_IS_DUP_FLAG, CompleteDataType.forBoolean()),
				new Schema.Field(FIELD_IS_RETAIN_FLAG, CompleteDataType.forBoolean()),
		});
    }
    
    public static Schema createConnectSchema() {
    	return new Schema("", new Schema.Field[] {
				new Schema.Field(FIELD_CLIENT_ID, CompleteDataType.forString()),
				new Schema.Field(FIELD_KEEP_ALIVE, CompleteDataType.forInt()),
				new Schema.Field(FIELD_PASSWORD, CompleteDataType.forBlob()),
				new Schema.Field(FIELD_PROTOCOL_NAME, CompleteDataType.forString()),
				new Schema.Field(FIELD_PROTOCOL_VERSION, CompleteDataType.forInt()),
				new Schema.Field(FIELD_QOS, CompleteDataType.forInt()),
				new Schema.Field(FIELD_USERNAME, CompleteDataType.forString()),
				new Schema.Field(FIELD_WILL_MESSAGE, CompleteDataType.forBlob()),
				new Schema.Field(FIELD_WILL_QOS, CompleteDataType.forInt()),
				new Schema.Field(FIELD_WILL_TOPIC, CompleteDataType.forString()),
				new Schema.Field(FIELD_IS_CLEAN_SESSION, CompleteDataType.forBoolean()),
				new Schema.Field(FIELD_IS_DUP_FLAG, CompleteDataType.forBoolean()),
				new Schema.Field(FIELD_IS_PASSWORD_FLAG, CompleteDataType.forBoolean()),
				new Schema.Field(FIELD_IS_RETAIN_FLAG, CompleteDataType.forBoolean()),
				new Schema.Field(FIELD_IS_USER_FLAG, CompleteDataType.forBoolean()),
				new Schema.Field(FIELD_IS_WILL_FLAG, CompleteDataType.forBoolean()),
				new Schema.Field(FIELD_IS_WILL_RETAIN, CompleteDataType.forBoolean()),
		});
    }
    
    public static Schema createDisconnectSchema() {
    	return new Schema("", new Schema.Field[] {
				new Schema.Field(FIELD_CLIENT_ID, CompleteDataType.forString()),
				new Schema.Field(FIELD_USERNAME, CompleteDataType.forString()),
		});
    }
    
    public static Schema createConnectionLostSchema() {
    	return new Schema("", new Schema.Field[] {
				new Schema.Field(FIELD_CLIENT_ID, CompleteDataType.forString()),
				new Schema.Field(FIELD_USERNAME, CompleteDataType.forString()),
		});
    }
    
    public static Schema createSubscribeSchema() {
    	return new Schema("", new Schema.Field[] {
    			new Schema.Field(FIELD_CLIENT_ID, CompleteDataType.forString()),
    			new Schema.Field(FIELD_REQUSTED_QOS, CompleteDataType.forInt()),
    			new Schema.Field(FIELD_TOPIC_FILTER, CompleteDataType.forString()),
    			new Schema.Field(FIELD_USERNAME, CompleteDataType.forString()),
		});
    }
    
    public static Schema createUnsubscribeSchema() {
    	return new Schema("", new Schema.Field[] {
    			new Schema.Field(FIELD_CLIENT_ID, CompleteDataType.forString()),
    			new Schema.Field(FIELD_TOPIC_FILTER, CompleteDataType.forString()),
    			new Schema.Field(FIELD_USERNAME, CompleteDataType.forString()),
		});
    }
    
    public static Schema createMessageSchema(PayloadType payloadType) {
    	return new Schema("", new Schema.Field[] {
    			new Schema.Field(FIELD_CLIENT_ID, CompleteDataType.forString()),
    			new Schema.Field(FIELD_GUID, CompleteDataType.forString()),    			
    			new Schema.Field(FIELD_PAYLOAD, payloadType.equals(PayloadType.STRING) ? CompleteDataType.forString() : CompleteDataType.forBlob()),
    			new Schema.Field(FIELD_QOS, CompleteDataType.forInt()),
				new Schema.Field(FIELD_TOPIC, CompleteDataType.forString()),
		});
    }
    
    public static Schema createMessageAcknowledgedSchema(PayloadType payloadType) {
    	return new Schema("", new Schema.Field[] {
				new Schema.Field(FIELD_MESSAGE, CompleteDataType.forTuple(createMessageSchema(payloadType))),
				new Schema.Field(FIELD_PACKET_ID, CompleteDataType.forInt()),
				new Schema.Field(FIELD_TOPIC, CompleteDataType.forString()),
				new Schema.Field(FIELD_USERNAME, CompleteDataType.forString()),
		});
    }
    
    private void setPayloadField(Tuple tuple, ByteBuf payload) throws TupleException {
    	if (payload == null) {
    		return;
    	}
		switch (moquetteMQTTBroker.getPayloadType()) {
			case BLOB:
				tuple.setBlobBuffer(FIELD_PAYLOAD, ByteArrayView.makeCopiedView(payload.array()));
				break;
			case STRING:
			default:
				tuple.setString(FIELD_PAYLOAD, payload.toString(UTF_8));
				break;    		
		} 
    }
    
    @Override
    public void onPublish(InterceptPublishMessage msg) {    	
    	Tuple tuple = moquetteMQTTBroker.getOutputSchema(MoquetteMQTTBroker.PORT_PUBLISH).createTuple();
    	try {
			tuple.setString(FIELD_TOPIC, msg.getTopicName());
			tuple.setInt(FIELD_QOS, msg.getQos().value());
			setPayloadField(tuple, msg.getPayload());
			tuple.setString(FIELD_CLIENT_ID, msg.getClientID());
			tuple.setString(FIELD_USERNAME, msg.getUsername());
			tuple.setBoolean(FIELD_IS_DUP_FLAG, msg.isDupFlag());
			tuple.setBoolean(FIELD_IS_RETAIN_FLAG, msg.isRetainFlag());
			moquetteMQTTBroker.sendOutput(MoquetteMQTTBroker.PORT_PUBLISH, tuple);
		} catch (StreamBaseException e) {
			moquetteMQTTBroker.logger.error(String.format("Error creating publish output for mqtt message: %s", e.getMessage()), e);
		}
    }
    
    @Override
    public void onConnect(InterceptConnectMessage msg) {
    	Tuple tuple = moquetteMQTTBroker.getOutputSchema(MoquetteMQTTBroker.PORT_CONNECT).createTuple();
    	try {			
			tuple.setString(FIELD_CLIENT_ID, msg.getClientID());
			tuple.setInt(FIELD_KEEP_ALIVE, msg.getKeepAlive());
			if (msg.isPasswordFlag() && msg.getPassword() != null) {
				tuple.setBlobBuffer(FIELD_PASSWORD, ByteArrayView.makeCopiedView(msg.getPassword()));
			}
			tuple.setString(FIELD_PROTOCOL_NAME, msg.getProtocolName());
			tuple.setInt(FIELD_PROTOCOL_VERSION, msg.getProtocolVersion());
			tuple.setInt(FIELD_QOS, msg.getQos().value());			
			tuple.setString(FIELD_USERNAME, msg.getUsername());
			if (msg.isWillFlag() && msg.getWillMessage() != null) {
				tuple.setBlobBuffer(FIELD_WILL_MESSAGE, ByteArrayView.makeCopiedView(msg.getWillMessage()));
			}
			tuple.setInt(FIELD_WILL_QOS, msg.getWillQos());
			tuple.setString(FIELD_WILL_TOPIC, msg.getWillTopic());
			tuple.setBoolean(FIELD_IS_CLEAN_SESSION, msg.isCleanSession());
			tuple.setBoolean(FIELD_IS_DUP_FLAG, msg.isDupFlag());
			tuple.setBoolean(FIELD_IS_PASSWORD_FLAG, msg.isPasswordFlag());
			tuple.setBoolean(FIELD_IS_RETAIN_FLAG, msg.isRetainFlag());
			tuple.setBoolean(FIELD_IS_USER_FLAG, msg.isUserFlag());
			tuple.setBoolean(FIELD_IS_WILL_FLAG, msg.isWillFlag());
			tuple.setBoolean(FIELD_IS_WILL_RETAIN, msg.isWillRetain());
			moquetteMQTTBroker.sendOutput(MoquetteMQTTBroker.PORT_CONNECT, tuple);
		} catch (StreamBaseException e) {
			moquetteMQTTBroker.logger.error(String.format("Error creating connect output for mqtt message: %s", e.getMessage()), e);
		}    	
    }

    @Override
    public void onDisconnect(InterceptDisconnectMessage msg) {
    	Tuple tuple = moquetteMQTTBroker.getOutputSchema(MoquetteMQTTBroker.PORT_DISCONNECT).createTuple();
    	try {			
			tuple.setString(FIELD_CLIENT_ID, msg.getClientID());
			tuple.setString(FIELD_USERNAME, msg.getUsername());
			moquetteMQTTBroker.sendOutput(MoquetteMQTTBroker.PORT_DISCONNECT, tuple);
		} catch (StreamBaseException e) {
			moquetteMQTTBroker.logger.error(String.format("Error creating disconnect output for mqtt message: %s", e.getMessage()), e);
		}    
    }

    @Override
    public void onConnectionLost(InterceptConnectionLostMessage msg) {
    	Tuple tuple = moquetteMQTTBroker.getOutputSchema(MoquetteMQTTBroker.PORT_CONNECTION_LOST).createTuple();
    	try {			
			tuple.setString(FIELD_CLIENT_ID, msg.getClientID());
			tuple.setString(FIELD_USERNAME, msg.getUsername());
			moquetteMQTTBroker.sendOutput(MoquetteMQTTBroker.PORT_CONNECTION_LOST, tuple);
		} catch (StreamBaseException e) {
			moquetteMQTTBroker.logger.error(String.format("Error creating connection lost output for mqtt message: %s", e.getMessage()), e);
		}   
    }

    @Override
    public void onSubscribe(InterceptSubscribeMessage msg) {
    	Tuple tuple = moquetteMQTTBroker.getOutputSchema(MoquetteMQTTBroker.PORT_SUBSCRIBE).createTuple();
    	try {			
			tuple.setString(FIELD_CLIENT_ID, msg.getClientID());
			tuple.setInt(FIELD_REQUSTED_QOS, msg.getRequestedQos().value());
			tuple.setString(FIELD_TOPIC_FILTER, msg.getTopicFilter());
			tuple.setString(FIELD_USERNAME, msg.getUsername());
			moquetteMQTTBroker.sendOutput(MoquetteMQTTBroker.PORT_SUBSCRIBE, tuple);
		} catch (StreamBaseException e) {
			moquetteMQTTBroker.logger.error(String.format("Error creating subscribe output for mqtt message: %s", e.getMessage()), e);
		} 
    }

    @Override
    public void onUnsubscribe(InterceptUnsubscribeMessage msg) {
    	Tuple tuple = moquetteMQTTBroker.getOutputSchema(MoquetteMQTTBroker.PORT_UNSUBSCRIBE).createTuple();
    	try {			
			tuple.setString(FIELD_CLIENT_ID, msg.getClientID());			
			tuple.setString(FIELD_TOPIC_FILTER, msg.getTopicFilter());
			tuple.setString(FIELD_USERNAME, msg.getUsername());
			moquetteMQTTBroker.sendOutput(MoquetteMQTTBroker.PORT_UNSUBSCRIBE, tuple);
		} catch (StreamBaseException e) {
			moquetteMQTTBroker.logger.error(String.format("Error creating unsubscribe output for mqtt message: %s", e.getMessage()), e);
		}
    }

    @Override
    public void onMessageAcknowledged(InterceptAcknowledgedMessage msg) {
    	Tuple tuple = moquetteMQTTBroker.getOutputSchema(MoquetteMQTTBroker.PORT_MESSAGE_ACKNOWLEDGED).createTuple();
    	try {			
    		Tuple messageTuple = tuple.getAllocatedTuple(FIELD_MESSAGE);
    		if (msg.getMsg() != null) {
	    		messageTuple.setString(FIELD_CLIENT_ID, msg.getMsg().getClientID());
	    		if (msg.getMsg().getGuid() != null) {
	    			messageTuple.setString(FIELD_GUID, msg.getMsg().getGuid().toString());
	    		}
	    		messageTuple.setInt(FIELD_QOS, msg.getMsg().getQos().value());
	    		messageTuple.setString(FIELD_TOPIC, msg.getMsg().getTopic()); 
    		}
			tuple.setInt(FIELD_PACKET_ID, msg.getPacketID());
			tuple.setString(FIELD_TOPIC, msg.getTopic());
			tuple.setString(FIELD_USERNAME, msg.getUsername());
			moquetteMQTTBroker.sendOutput(MoquetteMQTTBroker.PORT_MESSAGE_ACKNOWLEDGED, tuple);
		} catch (StreamBaseException e) {
			moquetteMQTTBroker.logger.error(String.format("Error creating message acknowledged output for mqtt message: %s", e.getMessage()), e);
		}
    }
}